package com.bocloud.cmp.common;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.bocloud.cmp.entity.Dictionary;
import com.bocloud.cmp.internal.SmsDictionaryService;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.License;
import com.megatron.framework.core.CurrentService;

import lombok.extern.slf4j.Slf4j;

/**
 * 证书节点service
 * 
 * @author ymh
 *
 */
@Component
@Slf4j
public class LicenseSerivce implements InitializingBean {

	private static final String PUBILC_CLOUD_DICTIONARY = "PUBLIC_CLOUD";

	@Autowired
	private ServerRepository serverRepository;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private CloudServerRepository cloudServerRepository;
	@Autowired
	protected SmsDictionaryService smsDictionaryService;
	@Autowired
	private CurrentService currentService;
	
	public GeneralResult licenseCheck(Long vendorId, Integer pm, Integer vm) {
		try {
			License license = currentService.getLicense();
			if (null == license) {
				return new GeneralResult(false, "证书信息获取异常，请联系管理员!");
			}
			Integer ratio = license.getRatio() == null ? 10 : license.getRatio();
			Integer node = license.getNode() == null ? 1000000 : license.getNode();
			int nodeCount = this.currentServiceNodes(ratio, pm, vm);
			log.info("current service node count : " + nodeCount);
			if (nodeCount > node) {
				return new GeneralResult(false, "当前系统节点数（宿主机和云主机）已超过license限制!");
			}
			return new GeneralResult(true, "license校验通过");
		} catch (Exception e) {
			log.error("Get error message：{}", e);
			return new GeneralResult(false, "license校验失败：" + e.getMessage());
		}
	}

	public int currentServiceNodes(Integer ratio) {
		return this.currentServiceNodes(ratio, 0, 0);
	}

	public int currentServiceNodes(Integer ratio, Integer pm, Integer vm) {
		try {
			// 根据字典查询当前系统中配置的公有云类型
			GeneralResult<List<Dictionary>> dictionarieResult = smsDictionaryService
					.dictionaries(PUBILC_CLOUD_DICTIONARY);
			if (dictionarieResult.isFailed()) {
				return 0;
			}
			List<Dictionary> dictionaries = dictionarieResult.getData();
			List<String> pubClouds = dictionaries.stream().map(Dictionary::getValue).collect(Collectors.toList());
			// TCE HCSO APS在私有云类型中  但是没有宿主机资源 所以走公有云的逻辑
			pubClouds.add("TCE");
			pubClouds.add("HCSO");
			pubClouds.add("APSARASTACK");
			// 计算宿主机数量
			Integer serverCounts = serverRepository.licenseNodeCount();
			Integer pmCount = serverCounts + pm;

			Integer vmCount = 0;
			// 查询公有云虚机数量
			vmCount = cloudServerRepository.vendorCount(pubClouds);
			vmCount += vm;
			vmCount = vmCount % ratio > 0 ? vmCount / ratio + 1 : vmCount / ratio;
			// F5 一个平台算一个节点
			List<String> types = Arrays.asList(new String[] { "F5" });
			int nodeCount = pmCount + vmCount + cloudVendorRepository.listByTypes(types, null).size();
			return nodeCount;
		} catch (Exception e) {
			log.error("查询当前系统节点数异常：" + e);
			return 0;
		}
	}

	/**
	 * 计算当前服务节点数
	 */
	@Scheduled(cron = "${cmp.service.node.scheduler:0 0/5 * * * ?}")
	protected void updateServiceNodes() {
		try {
			if (!currentService.isLeader()) {
				return;
			}
			License license = currentService.getLicense();
			if (null != license) {
				Integer ratio = license.getRatio() == null ? 10 : license.getRatio();
				currentService.setUsed(this.currentServiceNodes(ratio));
			}
		} catch (Exception e) {
			log.error("证书检验失败：" + e);
		}
	}

	@Override
	public void afterPropertiesSet() throws Exception {
//		this.updateServiceNodes();
	}

}
