package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.service.CloudServerService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "云主机管理")
@RestController
@RequestMapping("/v1/vms")
public class CloudServerController {

    @Autowired
    private CloudServerService cloudServerService;

    /**
     * 云主机列表查询
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询云主机列表")
    @GetMapping
    public GeneralResult<GridBean<CloudServer>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServerService.list(pager, context);
    }

    /**
     * 云主机列表查询
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询云主机列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.PAGE, required = false) Integer page,
                                   @RequestParam(value = Common.ROWS, required = false) Integer rows,
                                   @RequestParam(Common.CONDITION) String condition, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServerService.condition(page, rows, condition, context);
    }

    /**
     * 查看云主机统计
     *
     * @param range
     * @param hostId
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询云主机统计")
    @GetMapping("/file")
    public GeneralResult statistics(@RequestParam(value = "range") String range,
                                    @RequestParam(value = "hostId") Long hostId, @Value(Common.REQ_CONTEXT) RequestContext context) {
//        return vmExtendService.stats(range, hostId, context);
        return null;
    }

    /**
     * @param context
     * @return com.bocloud.common.model.GeneralResult
     * @Author: lxy
     * @Description: 云平台下云主机信息（个数、cpu、men、disk）统计
     * @Date 2021/5/26 16:42
     * @Param hostId
     * @throws:
     **/
    @Operation(tags = {"CMC", "CSC"}, summary = "宿主机下云主机信息统计")
    @GetMapping("/selectAllByVendorId")
    public GeneralResult selectAllByVendorId(@RequestParam(value = "vendorId", required = false) Long vendorId,
                                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServerService.selectAllByVendorId(vendorId, context);
    }

    /**
     * 导出云主机列表
     *
     * @param pager
     * @param condition
     * @param action
     * @param context
     * @param request
     * @param response
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "导出云主机列表")
    @GetMapping("/download")
    public void download(Pager pager, @RequestParam(value = Common.CONDITION, required = false) String condition,
                         @RequestParam(value = Common.ACTION, required = false) String action,
                         @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request,
                         HttpServletResponse response) {
//        GeneralResult<Map<String, Object>> GeneralResult =
//            fileDownloadService.vmDownload(pager, condition, action, context);
//        if (GeneralResult.isSuccess()) {
//            String content;
//            try {
//                Map<String, Object> result = GeneralResult.getData();
//                String fileName = result.get("fileName").toString();
//                if (fileName.contains("pdf")) {
//                    response.setContentType("application/pdf");
//                } else {
//                    response.setContentType("application/vnd.ms-excel");
//                }
//                final String userAgent = request.getHeader("user-agent").toLowerCase();
//                if (userAgent.contains("trident") || userAgent.contains("edge")) {
//                    fileName = URLEncoder.encode(fileName, "UTF-8");
//                } else {
//                    fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
//                }
//                response.setCharacterEncoding("utf-8");
//                response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
//                OutputStream outputStream = response.getOutputStream();
//                content = (String)result.get("data");
//                if (content != null) {
//                    byte[] bytes = Base64.getDecoder().decode(content);
//                    outputStream.write(bytes);
//                    outputStream.close();
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//            }
//        }
    }

    /**
     * 查看云主机详细信息
     *
     * @param id
     * @param context
     * @param action
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看云主机详情")
    @GetMapping("/{id}")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context,
                                @RequestParam(value = Common.ACTION, required = false) String action,
                                @RequestParam(value = Common.PARAMS, required = false) String params) {
        return cloudServerService.details(id, context, action, params);
    }

    /**
     * 根据集群维度或平台维度汇总云主机数量
     *
     * @param params
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "根据集群维度或平台维度汇总云主机数量")
    @GetMapping("/summary")
    public GeneralResult chart(@RequestParam(value = Common.PARAMS, required = true) String params) {
        return cloudServerService.chart(params);
    }

    /**
     * 查询模板
     *
     * @param id
     * @param vendorId
     * @param vdcId
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询模板")
    @GetMapping("/{id}/templates")
    public GeneralResult<List<CloudServer>> listTemplate(@PathVariable(value = Common.ID) Long id,
                                                         @RequestParam(value = Common.VENDORID, required = false) Long vendorId,
                                                         @RequestParam(value = "vdcId", required = false) Long vdcId) {
        return cloudServerService.listTemplate(vendorId, vdcId);
    }

    @Operation(summary = "流水号配置")
    @PostMapping(value = "/configSeq")
    public GeneralResult configSeq(@RequestParam(value = Common.PARAMS, required = false) String params,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServerService.configSeq(params, context);
    }

    @Operation(summary = "流水号配置详情")
    @GetMapping(value = "/getBusinessById/{id}")
    public GeneralResult getBusinessById(@PathVariable(value = Common.ID) Long id) {
        return cloudServerService.getBusinessById(id);
    }

    @Operation(tags = {"CMC", "CSC"}, summary = "云主机查询可调整的规格配置")
    @GetMapping("/{id}/available/sizes")
    public GeneralResult availableSizes(@PathVariable(value = Common.ID) Long id) {
        return cloudServerService.availableSizes(id);
    }

    @Operation(tags = {"CMC", "CSC"}, summary = "开启通用化")
    @PatchMapping("/{id}/generalize")
    public GeneralResult generalize(@PathVariable(value = Common.ID) Long id,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServerService.generalize(id, context);
    }

}
