package com.bocloud.cmp.controller;

import com.bocloud.cmp.domain.DataCenterBean;
import com.bocloud.cmp.entity.DataCenter;
import com.bocloud.cmp.service.DataCenterService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 数据中心相关接口
 *
 * @author dmw
 */
@Tag(name = "数据中心")
@RestController
@RequestMapping("/v1/dcs")
public class DataCenterController {

    @Autowired
    private DataCenterService dataCenterService;

    /**
     * 获取数据中心列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询数据中心列表")
    @GetMapping
    public GeneralResult<GridBean<DataCenter>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataCenterService.list(pager, context);
    }

    /**
     * 添加数据中心
     *
     * @param center
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加数据中心")
    @PostMapping
    public GeneralResult<DataCenter> create(@RequestBody DataCenter center, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataCenterService.create(center, context.getTarget());
    }

    /**
     * 修改数据中心
     *
     * @param id
     * @param bean
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改数据中心")
    @PutMapping("/{id}")
    public GeneralResult<DataCenter> modify(@PathVariable(value = Common.ID) Long id, @RequestBody DataCenterBean bean,
                                            @Value(Common.REQ_CONTEXT) RequestContext context) {
        bean.setId(id);
        return dataCenterService.modify(bean, context.getTarget());
    }

    /**
     * 移除数据中心
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除数据中心")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return dataCenterService.remove(id, context.getTarget());
    }

    /**
     * 查看数据中心详细信息
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看数据中心详情")
    @GetMapping("/{id}")
    public GeneralResult<DataCenter> detail(@PathVariable(value = Common.ID) Long id) {
        return dataCenterService.detail(id);
    }

    /**
     * 根据数据中心查询 资源的统计信息
     *
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "统计数据中心的资源")
    @GetMapping("/resource/count")
    public GeneralResult resourceCount(@RequestParam Long dcId) {
        return dataCenterService.resourceCount(dcId);
    }

    /**
     * 资源数据总览
     * @param dcId 地市id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "资源数据总览")
    @GetMapping("/resource/overview")
    public GeneralResult resourceOverview(@RequestParam(value = "dcId",required = false) Long dcId) {
        return dataCenterService.resourceOverview(dcId);
    }

}
