package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.IpPool;
import com.bocloud.cmp.model.IpInfoModel;
import com.bocloud.cmp.service.IpPoolService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ResultTools;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/ip/pools")
@Tag(name = "IP地址池")
public class IpPoolController {

    @Autowired
    private IpPoolService ipPoolService;

    /**
     * 查询IP地址池列表
     *
     * @param pager
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询IP地址池列表")
    @GetMapping
    public GeneralResult<GridBean<IpPool>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipPoolService.list(pager, context);
    }

    /**
     * 通过condition方式查询IP地址池列表
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "通过condition方式查询IP地址池列表")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipPoolService.condition(condition, context);
    }

    /**
     * 创建
     *
     * @param ipPool
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建IP地址池")
    @PostMapping
    public GeneralResult<IpPool> create(@RequestBody IpPool ipPool, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipPoolService.create(ipPool, context);
    }

    /**
     * 修改
     *
     * @param id
     * @param ipPool
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改IP地址池")
    @PutMapping("/{id}")
    public GeneralResult<IpPool> modify(@PathVariable(value = Common.ID) Long id, @RequestBody IpPool ipPool,
                                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        if (null == ipPool) {
            return ResultTools.formatErrResult();
        }
        ipPool.setId(id);
        return ipPoolService.modify(ipPool, context);
    }

    /**
     * 删除IP地址池
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除IP列表")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return ipPoolService.remove(id, context);
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询IP地址池详情")
    @GetMapping("/{id}")
    public GeneralResult<IpPool> detail(@PathVariable(value = Common.ID) Long id) {
        return ipPoolService.detail(id);
    }

    /**
     * 获取ip列表详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "获取IP地址池IP内列表详情")
    @GetMapping("/{id}/ips")
    public GeneralResult<List<IpInfoModel>> ips(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "cidr", required = false) String cidr) {
        return ipPoolService.ips(id, cidr);
    }
}
