package com.bocloud.cmp.controller;

import com.bocloud.cmp.domain.ServerLabelParam;
import com.bocloud.cmp.entity.Label;
import com.bocloud.cmp.service.LabelService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * @author zhangdi
 * @since 2018/7/2
 */
@RestController
@RequestMapping("/v1/tags")
@Tag(name = "命名规则")
public class LabelController {

    @Autowired
    private LabelService labelService;

    /**
     * 获取标签列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "获取命名规则列表")
    @GetMapping
    public GeneralResult<GridBean<Label>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return labelService.list(pager, context);
    }

    /**
     * 添加标签
     *
     * @param label
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加标签")
    @PostMapping
    public GeneralResult<Label> create(@RequestBody Label label) {
        return labelService.create(label);
    }

    /**
     * 修改标签
     *
     * @param id
     * @param Label
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改标签")
    @PutMapping("/{id}")
    public GeneralResult<Label> modify(@PathVariable(value = Common.ID) Long id, @RequestBody Label Label) {
        Label.setId(id);
        return labelService.modify(Label);
    }

    /**
     * 删除标签
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除标签")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id) {
        return labelService.remove(id);
    }

    /**
     * 查看标签详情
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看标签详情")
    @GetMapping("/{id}")
    public GeneralResult<Label> detail(@PathVariable(value = Common.ID) Long id) {
        return labelService.detail(id);
    }

    /**
     * 查询资源标签
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建资源标签")
    @PostMapping("/resources")
    public GeneralResult createResources(@RequestBody ServerLabelParam serverLabelParam,
                                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return labelService.createResources(serverLabelParam, context);
    }

}