package com.bocloud.cmp.controller;

import com.gitee.starblues.core.PluginInfo;
import com.gitee.starblues.integration.operator.PluginOperator;
import com.megatron.common.model.GeneralResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

@RestController
@RequestMapping("/v1/plugins")
@Tag(name = "插件")
@Slf4j
public class PluginController {

    private final PluginOperator pluginOperator;
    private final ApplicationContext applicationContext;

    @Value("${plugin.repository.path:/home/cmp/plugins/repository}")
    private String pluginRepoPath;

    public PluginController(PluginOperator pluginOperator, ApplicationContext applicationContext) {
        this.pluginOperator = pluginOperator;
        this.applicationContext = applicationContext;
    }

    @GetMapping
    @Operation(summary = "插件列表")
    public GeneralResult list() throws IOException {
        return new GeneralResult(true, pluginOperator.getPluginInfo(), "获取插件列表成功");
    }

    @PutMapping("/{pluginId}/start")
    @Operation(summary = "启动插件")
    public GeneralResult start(@PathVariable(value = "pluginId") String pluginId) {
        boolean success = pluginOperator.start(pluginId);
        return new GeneralResult(success, success ? "启动成功" : "启动失败");
    }

    @PutMapping("/{pluginId}/stop")
    @Operation(summary = "停止插件")
    public GeneralResult stop(@PathVariable(value = "pluginId") String pluginId) {
        boolean success = pluginOperator.stop(pluginId);
        return new GeneralResult(success, success ? "停止成功" : "停止失败");
    }

    @PutMapping("/install")
    @Operation(summary = "安装插件")
    public GeneralResult install(@RequestParam(value = "pluginPath") String pluginPathString) {
        Path pluginPath = Paths.get(pluginRepoPath, applicationContext.getId(), pluginPathString);
        PluginInfo pluginInfo = pluginOperator.install(pluginPath, false);
        return new GeneralResult(true, pluginInfo, "安装成功");
    }

    @PutMapping("/{pluginId}/uninstall")
    @Operation(summary = "卸载插件")
    public GeneralResult uninstall(@PathVariable(value = "pluginId") String pluginId) {
        pluginOperator.uninstall(pluginId, true, true);
        return new GeneralResult(true, "卸载成功");
    }

    @PutMapping("/{pluginId}/upgrade")
    @Operation(summary = "更新插件")
    public GeneralResult upgrade(@PathVariable(value = "pluginId") String pluginId,
                                 @RequestParam(value = "pluginPath") String pluginPathString) {
        pluginOperator.uninstall(pluginId, true, true);
        Path pluginPath = Paths.get(pluginRepoPath, applicationContext.getId(), pluginPathString);
        PluginInfo pluginInfo = pluginOperator.install(pluginPath, false);
        return new GeneralResult(true, pluginInfo, "更新成功");
    }

    @PutMapping("/load")
    @Operation(summary = "加载插件")
    public GeneralResult load(@RequestParam(value = "pluginPath") String pluginPathString) {
        Path pluginPath = Paths.get(pluginRepoPath, applicationContext.getId(), pluginPathString);
        PluginInfo pluginInfo = pluginOperator.load(pluginPath, false);
        return new GeneralResult(true, pluginInfo, "加载成功");
    }

    @PutMapping("/{pluginId}/unload")
    @Operation(summary = "卸载插件")
    public GeneralResult unload(@PathVariable(value = "pluginId") String pluginId) {
        boolean success = pluginOperator.unload(pluginId);
        return new GeneralResult(success, success ? "卸载成功" : "卸载失败");
    }

}
