package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.Region;
import com.bocloud.cmp.entity.Zone;
import com.bocloud.cmp.service.RegionService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 区域控制器(子区域为zone)
 *
 * @author dongkai
 */
@RestController
@RequestMapping("/v1/regions")
@Tag(name = "区域管理")
public class RegionController {

    @Autowired
    private RegionService regionService;

    /**
     * 查询
     *
     * @param vendorId
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看区域列表")
    @GetMapping
    public GeneralResult<List<Region>> list(@RequestParam(value = Common.VENDORID, required = true) Long vendorId,
                                            @RequestParam(value = "resourceType", required = false) String resourceType) {
        return regionService.list(vendorId, resourceType);
    }

    /**
     * 查询
     *
     * @param condition
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看区域列表")
    @GetMapping("/condition")
    public GeneralResult<List<Region>> condition(@RequestParam(Common.CONDITION) String condition,
                                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return regionService.condition(condition, context);
    }

    /**
     * 查询区域下的可用区
     *
     * @param vendorId
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询区域下子区域")
    @GetMapping("/zones")
    public GeneralResult<List<Zone>> listZone(@RequestParam(value = Common.VENDORID, required = false) Long vendorId,
                                              @RequestParam(value = Common.REGIONID, required = false) String regionId,
                                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return regionService.listZone(vendorId, regionId);
    }

    /**
     * 查询当前区域下可用区及备可用区
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(summary = "查询当前区域下可用区及备可用区")
    @GetMapping("/{id}/zones")
    public GeneralResult zones(@PathVariable(value = Common.ID) Long id,
                               @RequestParam(value = Common.TYPE, required = false) String type,
                               @RequestParam(value = Common.VENDORID, required = false) Long vendorId,
                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        return regionService.zones(id, vendorId, type);
    }
}
