package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.ResourceModule;
import com.bocloud.cmp.service.ResourceModuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 数据中心相关接口
 *
 * @author dmw
 */
@Tag(name = "数据中心")
@RestController
@RequestMapping("/v1/resourcemodule")
public class ResourceModuleController {

    @Autowired
    private ResourceModuleService resourceModuleService;

    /**
     * 获取数据中心列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询数据中心列表")
    @GetMapping
    public GeneralResult<GridBean<ResourceModule>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceModuleService.list(pager, context);
    }

    /**
     * 添加数据中心
     *
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加数据中心")
    @PostMapping
    public GeneralResult<ResourceModule> create(@RequestBody ResourceModule resourceModule, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceModuleService.create(resourceModule, context);
    }

    /**
     * 修改数据中心
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改数据中心")
    @PutMapping("/{id}")
    public GeneralResult<ResourceModule> modify(@PathVariable(value = Common.ID) Long id, @RequestBody ResourceModule resourceModule,
                                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceModuleService.modify(resourceModule, context);
    }

    /**
     * 移除数据中心
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除数据中心")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceModuleService.remove(id, context);
    }

    /**
     * 查看数据中心详细信息
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看数据中心详情")
    @GetMapping("/{id}")
    public GeneralResult<ResourceModule> detail(@PathVariable(value = Common.ID) Long id) {
        return resourceModuleService.detail(id);
    }


}
