package com.bocloud.cmp.controller;


import com.bocloud.cmp.service.ResourceScreenService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@Tag(name = "资源概览大屏")
@RestController
@RequestMapping("/v1/resource/screen")
public class ResourceScreenController {

    @Autowired
    private ResourceScreenService resourceLargeScreenService;

    /**
     * 基础信息
     * @param code  省中心or地市编码
     * @param context
     * @return 省中心：专有云主机、公有云主机、上云单位、地市云接入 or 漯河市：政务云平台、上云单位、已交付云主机
     */
    @GetMapping()
    public GeneralResult getBasicinfo(@RequestParam(value = "code") String code, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getBasicinfo(code,context);
    }

    /**
     * 云数据汇总概览：进度条图
     * @param type 省中心专有域：EXCLUSIVE、省中心公有域：PUBLIC、地市云数据汇总：ALL
     * @param code 地市编码： 省中心编码 or 地市编码  【如果是地市编码type可以为空，如果是省中心编码type为必填】
     * @param context
     * @return 省中心专有域云数据汇总概览、省中心公有域云数据汇总概览、所有地市云数据汇总概览 or 漯河市云数据汇总概览
     */
    @GetMapping("/overview")
    public GeneralResult getOverview(@RequestParam(value = "type", required = false) String type,
                                       @RequestParam(value = "code") String code,
                                       @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getOverview(type, code,context);
    }

    /**
     * 云主机交付数量分布:饼图
     * @param code 地市编码
     * @param context
     * @return 省中心云主机交付数量分布 or 漯河市云主机交付数量分布
     */
    @GetMapping("/delivery/quantity")
    public GeneralResult getCloudHostDeliveryQuantity(@RequestParam(value = "code") String code, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getCloudHostDeliveryQuantity(code,context);
    }

    /**
     * 云主机交付情况：趋势图
     * @param code 地市编码 或者 省中心编码
     * @param context
     * @return 省中心云主机交付情况 or 漯河市云主机交付情况
     */
    @GetMapping("/delivery/status")
    public GeneralResult getCloudHostDeliverystatus(@RequestParam(value = "code") String code, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getCloudHostDeliverystatus(code,context);
    }

    /**
     * 云资源使用情况：趋势图
     * @param code 地市编码
     * @param context
     * @return  漯河市云资源使用情况
     */
    @GetMapping("/usage")
    public GeneralResult getCloudUsage(@RequestParam(value = "code") String code, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getCloudUsage(code,context);
    }

    /**
     * 云主机资源分布top5
     * @param code 地市编码
     * @param context
     * @return 云主机资源分布TOP5
     */
    @GetMapping("/distribution")
    public GeneralResult getDistributionData(@RequestParam(value = "code") String code, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getDistributionData(code,context);
    }

    /**
     * 云使用单位资源统计
     * @param code 地市编码
     * @param context
     * @return 云使用单位资源统计（自动轮播）
     */
    @GetMapping("/statistics")
    public GeneralResult getUnitStatistics(@RequestParam(value = "code") String code, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceLargeScreenService.getUnitStatistics(code,context);
    }

    /**
     * 河南区域信息查询
     * @return
     */
    @GetMapping("/city")
    public GeneralResult getCity() {
        return resourceLargeScreenService.getCity();
    }

}
