package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.Tag;
import com.bocloud.cmp.model.VmTagParam;
import com.bocloud.cmp.service.TagService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/tag")
@io.swagger.v3.oas.annotations.tags.Tag(name = "标签")
public class TagController {

    @Autowired
    private TagService tagService;

    @Operation(tags = {"CMC", "CSC"}, summary = "查询标签列表")
    @GetMapping
    public GeneralResult<GridBean<Tag>> list(Pager pager) {
        return tagService.list(pager);
    }

    /**
     * 创建标签
     *
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "创建标签")
    @PostMapping
    public GeneralResult create(@RequestBody VmTagParam vmTagParam) {
        return tagService.create(vmTagParam);
    }

    /**
     * 删除标签
     *
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "删除标签")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id) {
        return tagService.remove(id);
    }
}
