package com.bocloud.cmp.controller;

import com.bocloud.cmp.entity.VendorCategory;
import com.bocloud.cmp.service.VendorCategoryService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 平台类型
 */
@Tag(name = "平台类型")
@RestController
@RequestMapping("/v1/vendorcategory")
public class VendorCategoryController {

    @Autowired
    private VendorCategoryService vendorCategoryService;

    /**
     * 获取平台类型列表
     *
     * @param pager
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查询平台类型列表")
    @GetMapping
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vendorCategoryService.list(pager, context);
    }

    /**
     * 添加平台类型
     *
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "添加平台类型")
    @PostMapping
    public GeneralResult<VendorCategory> create(@RequestBody VendorCategory vendorCategory, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vendorCategoryService.create(vendorCategory, context);
    }

    /**
     * 修改平台类型
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "修改平台类型")
    @PutMapping("/{id}")
    public GeneralResult<VendorCategory> modify(@PathVariable(value = Common.ID) Long id, @RequestBody VendorCategory vendorCategory,
                                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vendorCategoryService.modify(vendorCategory, context);
    }

    /**
     * 移除平台类型
     *
     * @param id
     * @param context
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "移除平台类型")
    @DeleteMapping("/{id}")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return vendorCategoryService.remove(id, context);
    }

    /**
     * 查看平台类型详细信息
     *
     * @param id
     * @return
     */
    @Operation(tags = {"CMC", "CSC"}, summary = "查看平台类型详情")
    @GetMapping("/{id}")
    public GeneralResult<VendorCategory> detail(@PathVariable(value = Common.ID) Long id) {
        return vendorCategoryService.detail(id);
    }

}
