package com.bocloud.cmp.entity;

import java.util.Date;

import org.springframework.format.annotation.DateTimeFormat;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;

/**
 * 块设备实体类
 */
@Table("bucket")
public class Bucket extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("vendor_id")
    private Long vendorId; // 云供应商ID
    @IgnoreAll
    private String vendorName; // 云供应商名称
    @Column("region")
    private Long region; // 镜像所在区域
    @IgnoreAll
    private String regionName; // 镜像所在区域名称
    @Column("space")
    private Double space;// 空间
    @Column("location")
    private String location;// 位置
    @Column("acl")
    private Boolean acl;// 权限
    @Column("expired_time")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredTime;// 过期时间

    @Column("create_time")
    @JsonSerialize(using = DateSerializer.class)
    @com.fasterxml.jackson.databind.annotation.JsonDeserialize(using = com.megatron.common.utils.DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @Column("region_id")
    private String regionId;

    @Column("actual_name")
    private String actualName;
    @Column("tenant_id")
    private Long tenantId; // 租户ID
    @Column("user_id")
    private Long userId; // 对象用户ID
    @Column("traffics")
    private Long traffics; // 下载流量
    @Column("bucket_acl")
    private String bucketAcl;

    @Column("bucket_uuid")
    private Long bucketUuid;
    @Column("object_user_id")
    private Long objectUserId;
    @Column("object_user_uuid")
    private Long objectUserUuid;
    @Column("object_user_name")
    private String objectUserName;

    @Column("quota_max_objects")
    private Long quotaMaxObjects;
    @Column("quota_max_size")
    private Long quotaMaxSize;
    @Column("nfs_client_num")
    private Long nfsClientNum;
    @Column("owner_permission")
    private String ownerPermission;
    @Column("shards")
    private Long shards;
    @Column("allocated_objects")
    private Long allocatedObjects;
    @Column("allocated_size")
    private Long allocatedSize;

    @Column("is_worm")
    private Boolean worm; // 写保护
    @Column("is_versioned")
    private Boolean versioned; // 多版本

    @IgnoreAll
    private String spaceUsed;
    @IgnoreAll
    private int objectCount;
    @IgnoreAll
    private Boolean expired;
    @IgnoreAll
    private Long monthTraffic;

    public Long getObjectUserUuid() {
        return objectUserUuid;
    }

    public void setObjectUserUuid(Long objectUserUuid) {
        this.objectUserUuid = objectUserUuid;
    }

    public String getRegionId() {
        return regionId;
    }

    public void setRegionId(String regionId) {
        this.regionId = regionId;
    }

    public Date getExpiredTime() {
        return expiredTime;
    }

    public void setExpiredTime(Date expiredTime) {
        this.expiredTime = expiredTime;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getVendorId() {
        return vendorId;
    }

    public void setVendorId(Long vendorId) {
        this.vendorId = vendorId;
    }

    public String getVendorName() {
        return vendorName;
    }

    public void setVendorName(String vendorName) {
        this.vendorName = vendorName;
    }

    public Long getRegion() {
        return region;
    }

    public void setRegion(Long region) {
        this.region = region;
    }

    public String getRegionName() {
        return regionName;
    }

    public void setRegionName(String regionName) {
        this.regionName = regionName;
    }

    public Double getSpace() {
        return space;
    }

    public void setSpace(Double space) {
        this.space = space;
    }

    public String getLocation() {
        return location;
    }

    public void setLocation(String location) {
        this.location = location;
    }

    public Boolean getAcl() {
        return acl;
    }

    public void setAcl(Boolean acl) {
        this.acl = acl;
    }

    public String getActualName() {
        return actualName;
    }

    public void setActualName(String actualName) {
        this.actualName = actualName;
    }

    public Long getTenantId() {
        return tenantId;
    }

    public void setTenantId(Long tenantId) {
        this.tenantId = tenantId;
    }

    public Long getUserId() {
        return userId;
    }

    public void setUserId(Long userId) {
        this.userId = userId;
    }

    public Long getTraffics() {
        return traffics;
    }

    public void setTraffics(Long traffics) {
        this.traffics = traffics;
    }

    public String getBucketAcl() {
        return bucketAcl;
    }

    public void setBucketAcl(String bucketAcl) {
        this.bucketAcl = bucketAcl;
    }

    public String getSpaceUsed() {
        return spaceUsed;
    }

    public void setSpaceUsed(String spaceUsed) {
        this.spaceUsed = spaceUsed;
    }

    public int getObjectCount() {
        return objectCount;
    }

    public void setObjectCount(int objectCount) {
        this.objectCount = objectCount;
    }

    public Boolean getExpired() {
        return expired;
    }

    public void setExpired(Boolean expired) {
        this.expired = expired;
    }

    public Long getMonthTraffic() {
        return monthTraffic;
    }

    public void setMonthTraffic(Long monthTraffic) {
        this.monthTraffic = monthTraffic;
    }

    public Long getObjectUserId() {
        return objectUserId;
    }

    public void setObjectUserId(Long objectUserId) {
        this.objectUserId = objectUserId;
    }

    public String getObjectUserName() {
        return objectUserName;
    }

    public void setObjectUserName(String objectUserName) {
        this.objectUserName = objectUserName;
    }

    public Long getBucketUuid() {
        return bucketUuid;
    }

    public void setBucketUuid(Long bucketUuid) {
        this.bucketUuid = bucketUuid;
    }

    public Date getCreateTime() {
        return createTime;
    }

    public void setCreateTime(Date createTime) {
        this.createTime = createTime;
    }

    public Long getQuotaMaxObjects() {
        return quotaMaxObjects;
    }

    public void setQuotaMaxObjects(Long quotaMaxObjects) {
        this.quotaMaxObjects = quotaMaxObjects;
    }

    public Long getQuotaMaxSize() {
        return quotaMaxSize;
    }

    public void setQuotaMaxSize(Long quotaMaxSize) {
        this.quotaMaxSize = quotaMaxSize;
    }

    public Long getNfsClientNum() {
        return nfsClientNum;
    }

    public void setNfsClientNum(Long nfsClientNum) {
        this.nfsClientNum = nfsClientNum;
    }

    public String getOwnerPermission() {
        return ownerPermission;
    }

    public void setOwnerPermission(String ownerPermission) {
        this.ownerPermission = ownerPermission;
    }

    public Long getShards() {
        return shards;
    }

    public void setShards(Long shards) {
        this.shards = shards;
    }

    public Long getAllocatedObjects() {
        return allocatedObjects;
    }

    public void setAllocatedObjects(Long allocatedObjects) {
        this.allocatedObjects = allocatedObjects;
    }

    public Long getAllocatedSize() {
        return allocatedSize;
    }

    public void setAllocatedSize(Long allocatedSize) {
        this.allocatedSize = allocatedSize;
    }

    public Boolean getWorm() {
        return worm;
    }

    public void setWorm(Boolean worm) {
        this.worm = worm;
    }

    public Boolean getVersioned() {
        return versioned;
    }

    public void setVersioned(Boolean versioned) {
        this.versioned = versioned;
    }

}
