package com.bocloud.cmp.entity;

import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 告警实体类
 *
 * @author wangyu
 * @Version 1.0
 * @since 2017年7月10日
 */
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@Table("resource_alarm")
public class ResourceAlarm extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("level")
    private String level;// 警告等级
    @Column("type")
    private String type;// 目标机器类型
    @Column("target")
    private Long target;// 目标机器
    @Column("deal_result")
    private String dealResult;// 处理结果
    @Column("alarm_val")
    private String alarmVal;// 告警的key值
    @Column("gmt_trigger")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtTrigger;// 告警触发时间
    @Column("gmt_confirm")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtConfirm;// 告警确认时间
    @Column("confirmer")
    private String confirmer;// 告警确认人
    @Column("gmt_solve")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtSolve;// 告警解决时间
    @Column("solver")
    private String solver;// 告警解决人
    @Column("vendor_id")
    private Long vendorId; // 云供应商Id
    @Column("rule_id")
    private Long ruleId;// 告警规则ID
    @Column("event_alarm")
    private Boolean eventAlarm;// 是否是事件告警
    // @Column("result_status")
    // private Boolean resultStatus;// 结果状态
    // @Column("result")
    // private String result;// 结果
    @IgnoreAll
    private String vendorName; // 云供应商名称
    @Column("target_name")
    private String targetName; // 告警目标名称-告警对象（机器）

    /**
     * @param level
     * @param type
     * @param target
     * @param alarmVal
     * @param gmtTrigger
     * @param eventAlarm
     */
    public ResourceAlarm(String level, String type, Long target, String alarmVal, Date gmtTrigger, Boolean eventAlarm) {
        super();
        this.level = level;
        this.type = type;
        this.target = target;
        this.alarmVal = alarmVal;
        this.gmtTrigger = gmtTrigger;
        this.eventAlarm = eventAlarm;
    }

    /**
     * @param level
     * @param type
     * @param target
     * @param alarmVal
     * @param gmtTrigger
     * @param eventAlarm
     * @param resultStatus
     * @param result
     */
    public ResourceAlarm(String level, String type, Long target, String alarmVal, Date gmtTrigger, Boolean eventAlarm,
                         Boolean resultStatus, String result) {
        super();
        this.level = level;
        this.type = type;
        this.target = target;
        this.alarmVal = alarmVal;
        this.gmtTrigger = gmtTrigger;
        this.eventAlarm = eventAlarm;
    }

}
