package com.bocloud.cmp.entity;


import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * Server配置
 *
 * @author dongkai
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
@Table("server_config")
public class ServerConfig extends Generic {

    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("username")
    private String username;
    @Column("password")
    private String password;
    @Column("ipmi_account")
    private String ipmiAccount;
    /**
     * 装机节点IPMI所有用户
     */
    @Column("ipmi_users")
    private String ipmiUsers;
    /**
     * 装机节点IPMI所有地址信息
     */
    @Column("ipmi_ips")
    private String ipmiIps;
    @Column("ipmi_password")
    private String ipmiPassword;
    @Column("ipmi_address")
    private String ipmiAddress;
    @Column("ipmi_port")
    private Integer ipmiPort;
    @Column("support_docker")
    private Boolean supportDocker;
    @Column("docker_version")
    private String dockerVersion;
    @Column("ceph_version")
    private String cephVersion;
    @Column("file_system")
    private String fileSystem;
    @Column("mac")
    private String mac;
    @Column("kernel_version")
    private String kernelVersion;
    @Column("virtualized")
    private Boolean virtualized;
    @Column("key_store")
    private String keyStore;
    @Column("vt_mode")
    private String vtMode;
    @Column("os_name")
    private String osName;
    @Column("os_version")
    private String osVersion;
    @Column("ssh_port")
    private Integer sshPort;
    @Column("cpu_vendor")
    private String cpuVendor;
    @Column("cpu_amount")
    private Integer cpuAmount;
    @Column("cpu_mode")
    private String cpuMode;
    @Column("cpu_core")
    private Integer cpuCore;
    @Column("cpu_hz")
    private Long cpuHz;
    @Column("memory_mode")
    private String memoryMode;
    @Column("memory_capacity")
    private Double memoryCapacity;
    @Column("swap_capacity")
    private Double swapCapacity;
    @Column("disk_capacity")
    private Double diskCapacity;
    @Column("cache_capacity")
    private Double cacheCapacity;
    @Column("bios_version")
    private String biosVersion;
    @Column("bios_vendor")
    private String biosVendor;
    @Column("bios_date")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date biosDate;
    @Column("server_id")
    private Long serverId;
    @Column("cpu_used")
    private Long cpuUsed;
    @Column("mem_used")
    private Double memUsed;
    @Column("mem_reserved")
    private Double memReserved; // 保留内存
    @Column("disk_used")
    private Double diskUsed;

    @Column("hmc_cpu")
    private Double hmcCpu; // 虚拟处理器单元数
    @Column("hmc_cpu_reserved")
    private Double hmcCpuReserved; // 保留处理器单元数
    @Column("hmc_cpu_used")
    private Double hmcCpuUsed; // 已用处理器单元数
    
    @Column("cpu_sum")
	private Integer cpuSum; // CPU总核数，也就是常说的48C、96C，用来为大屏统计提供数据来源

    @IgnoreAll
    private Long vdcId;
    /**
     * 厂商信息
     */
    @Column("vendor")
    private String vendor;
    @Column("esx_version")
    private String esxVersion;
    @Column("evc")
    private String evc;
    @Column("assigned_license")
    private String assignedLicense;

    @Column("hdd_count")
    private Integer hddCount;
    @Column("ssd_count")
    private Integer ssdCount;
    @Column("hdd_capacity")
    private Double hddCapacity;
    @Column("ssd_capacity")
    private Double ssdCapacity;

    //云宏
    @Column("cpu_main_frequency")
    private String cpuMainFrequency;//CPU主频
    @Column("cpu_cores_per_socket")
    private Integer cpuCoresPerSocket;


}
