package com.bocloud.cmp.internal;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.entity.Business;
import com.bocloud.cmp.entity.BusinessPool;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2018/12/25
 */
@Component
public class SmsBusinessService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("sms");
	private Map<String, Object> headers = new HashMap<>(4);

	public GeneralResult deleteByPoolId(Long poolGroupId) {
		RemoteService remote = serviceFactory.build(service, "/v1/bizs/pools", HttpMethod.DELETE, headers,
				MapTools.simpleMap("poolGroupId", poolGroupId));
		return remote.invoke();
	}

	public List<BusinessPool> listByPoolGroupId(Long poolGroupId) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "listByPoolGroupId");
		object.put("poolGroupId", poolGroupId);
		RemoteService remote = serviceFactory.build(service, "/v1/bizs/condition", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new ArrayList<>();
		}
		return JSONArray.parseArray(JSONArray.toJSONString(result.getData()), BusinessPool.class);
	}

	public GeneralResult<Business> detail(Long id) {
		RemoteService remoteService = serviceFactory.build(service, "/v1/bizs/" + id, HttpMethod.GET, headers, null);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, Business.class), "success");
	}

	public List<Business> listBusinesssByIds(List<Long> ids) {
		ArrayList<Param> params=null;
		if (null!=ids && !ids.isEmpty()){
			String idString = JSONArray.toJSONString(ids);
			params = Lists.newArrayList(new Param(MapTools.simpleMap("id", idString.substring(1,idString.length()-1)), Sign.IN));
		}
		Map<String, Object> paramMap = MapTools.simpleMap("simple", false);
		paramMap.put("page", 1);
		paramMap.put("rows", Integer.MAX_VALUE);
		paramMap.put(Common.PARAMS, JSONArray.toJSONString(params));
		RemoteService remoteService = serviceFactory.build(service, "/v1/bizs" , HttpMethod.GET, headers, paramMap);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new ArrayList<>();
		}
		String data = JSONObject.toJSONString(result.getData());
		return JSONObject.parseObject(data).getJSONArray("rows").toJavaList(Business.class);
	}

	public Business getByProjectId(Long projectId) {
		RemoteService remoteService = serviceFactory.build(service, "/v1/projects/" + projectId + "/business",
				HttpMethod.GET, headers, null);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return null;
		}
		String data = JSONObject.toJSONString(result.getData());
		return JSONObject.parseObject(data, Business.class);
	}

	public List<BusinessPool> listByBusinessId(Long businessId) {
		RemoteService remote = serviceFactory.build(service, "/v1/bizs/" + businessId + "/pools", HttpMethod.GET,
				headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new ArrayList<>();
		}
		return JSONArray.parseArray(JSONArray.toJSONString(result.getData()), BusinessPool.class);
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}
}
