package com.bocloud.cmp.internal;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.fastjson.JSONArray;
import com.megatron.common.utils.MapTools;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.entity.Department;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * SOC组织机构访问
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月5日
 *
 */
@Component
public class SmsDepartmentService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("sms");
	private Map<String, Object> headers = new HashMap<>(4);

	public GeneralResult<Department> detail(Long id) {
		RemoteService remote = serviceFactory.build(service, "/v1/departments/" + id, HttpMethod.GET, headers, null);
		GeneralResult result = remote.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, Department.class), "success");
	}

	public List<Department> list() {
		RemoteService remoteService = serviceFactory.build(service, "/v1/departments", HttpMethod.GET, headers, MapTools.simpleMap("parentId", 0));
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return null;
		}
		return JSONArray.parseArray(JSONObject.toJSONString(result.getData()), Department.class);
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
		this.headers.put("source", currentService.getService().getName());
	}
}
