package com.bocloud.cmp.internal;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.AccountSecurity;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.cmp.entity.User;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;

/**
 * SOC管理员用户访问
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年2月5日
 *
 */
@Component
public class SmsManagerService implements InitializingBean {

	@Autowired
	private ServiceFactory serviceFactory;
	@Autowired
	private CurrentService currentService;
	private static Service service = Service.create("sms");
	private Map<String, Object> headers = new HashMap<>(4);

	public GeneralResult<AccountSecurity> secKey(String apiKey) {
		Map<String, Object> params = MapTools.simpleMap("apiKey", apiKey);
		RemoteService remoteService = serviceFactory.build(service, "/v1/users/key", HttpMethod.GET, headers,
				params);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, AccountSecurity.class), "success");
	}

	public GeneralResult<User> detail(Long id) {
		RemoteService remoteService = serviceFactory.build(service, "/v1/users/" + id, HttpMethod.GET, headers,
				null);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, User.class), "success");
	}

	public GeneralResult<User> basic(Long id) {
		RemoteService remoteService = serviceFactory.build(service, "/v1/users/basic/" + id, HttpMethod.GET,
				headers, null);
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseObject(data, User.class), "success");
	}

	public GeneralResult<List<User>> listAdminManager() {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "listAdminmanager");
		RemoteService remoteService = serviceFactory.build(service, "/v1/users/condition", HttpMethod.GET, headers,
				MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
		GeneralResult result = remoteService.invoke();
		if (result.isFailed()) {
			return new GeneralResult<>(false, result.getMessage());
		}
		String data = JSONObject.toJSONString(result.getData());
		return new GeneralResult<>(true, JSONObject.parseArray(data, User.class), "success");
	}

	/**
	 * 按权限菜单查询用户
	 *
	 * @param type
	 * @return
	 */
	public GeneralResult listByPermission(String type) {
		JSONObject object = new JSONObject();
		object.put(Common.CONDITION, "listByPermission");
		object.put("router", type);
		Map<String, Object> paramMap = MapTools.simpleMap(Common.CONDITION, JSONObject.toJSONString(object));
		RemoteService remoteService = serviceFactory.build(service, "/v1/users", HttpMethod.GET, headers,
				paramMap);
		return remoteService.invoke();
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
	}

}
