package com.bocloud.cmp.listener;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.repository.ServerRepository;
import com.megatron.common.utils.JSONTools;

import lombok.extern.slf4j.Slf4j;

/**
 * exporter安装结果监听器
 */
@Component("exporterInstallListener")
@Slf4j
public class ExporterInstallListener {

    @Autowired
    private ServerRepository serverRepository;

    @RabbitListener(queues = "exporter.task.result.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void process(Message message) throws Exception {
        String content = new String(message.getBody());
        if (StringUtils.isBlank(content)) {
            // 消息没有内容，直接确认收到 淹没处理
            return;
        }
        log.info("接收exporter安装任务结果信息：" + content);
        JSONObject object = JSONTools.isJSONObj(content);
        if (null == object) {
            // 消息参数格式错误，直接确认收到淹没处理
            return;
        }
        boolean success = (null == object.getBoolean("success") ? false : object.getBoolean("success"));
        serverRepository.updateExporterInstalled(object.getLong("serverId"), success);
    }

}
