package com.bocloud.cmp.listener;


import com.alibaba.fastjson.JSON;
import com.gitee.starblues.core.PluginInfo;
import com.gitee.starblues.integration.operator.PluginOperator;
import lombok.RequiredArgsConstructor;
import org.springframework.cloud.zookeeper.discovery.ZookeeperDiscoveryProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 插件定时任务
 *
 * @author zhangyf
 * @since 2022-12-07
 */
@RequiredArgsConstructor
@Component
public class PluginScheduler {

    private final ApplicationContext applicationContext;
    private final StringRedisTemplate stringRedisTemplate;
    private final ZookeeperDiscoveryProperties zookeeperDiscoveryProperties;
    private final PluginOperator pluginOperator;

    /**
     * 定时将插件状态存在Redis缓存中
     */
    @Async
    @Scheduled(fixedRate = 60000L)
    public void execute() {
        String key = applicationContext.getId() + "_plugins_" +
                zookeeperDiscoveryProperties.getInstanceHost() + ":" + zookeeperDiscoveryProperties.getInstancePort();
        List<PluginInfo> list = pluginOperator.getPluginInfo();
        Map<String, PluginInfo> pluginInfoMap = list.stream().collect(Collectors.toMap(PluginInfo::getPluginId, Function.identity()));
        Map<String, String> pluginInfoMapString = new HashMap<>();
        pluginInfoMap.forEach((k, v) -> {
            pluginInfoMapString.put(k, JSON.toJSONString(v));
        });
        stringRedisTemplate.opsForHash().putAll(key, pluginInfoMapString);
        Object[] deleteKeys = stringRedisTemplate.opsForHash().entries(key).keySet().stream()
                .filter(k -> !pluginInfoMapString.containsKey(k.toString())).toArray();
        if (deleteKeys.length > 0) {
            stringRedisTemplate.opsForHash().delete(key, deleteKeys);
        }
        stringRedisTemplate.expire(key, Duration.ofMinutes(3));
    }
}
