package com.bocloud.cmp.listener;

import com.alibaba.fastjson.JSON;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.gitee.starblues.core.PluginInfo;
import com.gitee.starblues.integration.listener.PluginListener;
import com.gitee.starblues.integration.operator.PluginOperator;
import com.megatron.common.model.OperateResult;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.cloud.zookeeper.discovery.ZookeeperDiscoveryProperties;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

/**
 * 插件状态监听事件
 *
 * @author zhangyf
 */
@RequiredArgsConstructor
@Slf4j
@Component
@Configuration
public class PluginStateListener implements PluginListener {

    private final ApplicationContext applicationContext;
    private final ZookeeperDiscoveryProperties zookeeperDiscoveryProperties;
    private final PluginOperator pluginOperator;
    private final StringRedisTemplate stringRedisTemplate;
    private final AmqpTemplate amqpTemplate;

    @Override
    public void loadSuccess(PluginInfo pluginInfo) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    @Override
    public void unLoadSuccess(PluginInfo pluginInfo) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    @Override
    public void unLoadFailure(PluginInfo pluginInfo, Throwable throwable) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    @Override
    public void startSuccess(PluginInfo pluginInfo) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    @Override
    public void startFailure(PluginInfo pluginInfo, Throwable throwable) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    @Override
    public void stopSuccess(PluginInfo pluginInfo) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    @Override
    public void stopFailure(PluginInfo pluginInfo, Throwable throwable) {
        this.updatePluginInfoCache(pluginInfo.getPluginId());
    }

    private void updatePluginInfoCache(String pluginId) {
        String key = applicationContext.getId() + "_plugins_"
                + zookeeperDiscoveryProperties.getInstanceHost() + ":" + zookeeperDiscoveryProperties.getInstancePort();
        PluginInfo pluginInfo = pluginOperator.getPluginInfo(pluginId);
        if (pluginInfo == null) {
            stringRedisTemplate.opsForHash().delete(key, pluginId);
        } else {
            stringRedisTemplate.opsForHash().put(key, pluginId, JSON.toJSONString(pluginInfo));
        }
        // 给前端发送消息以改变状态
        OperateResult operateResult = new OperateResult(true, "插件状态有更新", "plugin.state", null,
                OperateResult.OperateCategory.Plugin, null);
        this.amqpTemplate.convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#",
                operateResult);
    }
}
