package com.bocloud.cmp.listener;

import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.ExtensionParam;
import com.bocloud.cmp.domain.OrderMessage;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.service.ResourceOperationService;

import lombok.extern.slf4j.Slf4j;

/**
 * 资源延期监听器
 * 
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 *
 */
@Component
@Slf4j
public class ResourceExtensionListener {

	@Autowired
	private ResourceOperationService resourceOperationService;
	private static final String EXTENSION_QUEUE = "iaas.resource.extension.queue";

	@RabbitListener(queues = EXTENSION_QUEUE, admin = "rabbitAdmin", autoStartup = "true")
	public void process(ExtensionParam param) throws Exception {
		try {
			if (null == param) {
				return;
			}
			log.info("接收到资源延期消息：{}", JSONObject.toJSONString(param));
			String category = param.getCategory();
			resourceOperationService.process(ResourceCategory.valueOf(category),
					JSONObject.parseObject(JSONObject.toJSONString(param)), OrderMessage.Operation.Extension);
		} catch (Exception e) {
			log.error("Get error message:{}", e.getMessage());
		}
	}

}
