package com.bocloud.cmp.listener;

import java.nio.charset.Charset;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.service.RosOperationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.JSONTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 资源编排监听器
 *
 */
@Component("rosListener")
@Slf4j
public class RosListener {

    @Autowired
    private RosOperationService rosOperationService;
    @Autowired
    private AmqpTemplate amqpTemplate;

    @RabbitListener(queues = "bocloud.ros.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void process(Message message) {
        try {
            String content = new String(message.getBody(), Charset.forName("UTF-8"));
            processDebugLog("get bocloud message:{}", content);
            if (StringUtils.isBlank(content)) {
                // 消息没有内容，直接确认收到 淹没处理
                return;
            }
            JSONObject object = JSONTools.isJSONObj(content);
            if (null == object) {
                // 消息参数格式错误，直接确认收到淹没处理
                return;
            }
            log.info("收到资源编排创建资源消息{}", content);
            GeneralResult<String> result = rosOperationService.process(content);
            if (result.isFailed() && StringUtils.isNotEmpty(result.getData())) {
                this.amqpTemplate.convertAndSend(result.getData(), result);
            }
        } catch (Exception e) {
            log.error("Get error message:{}", e.getMessage());
        }
    }

    private void processDebugLog(String format, Object... objects) {
        if (log.isDebugEnabled()) {
            log.debug(format, objects);
        }
    }

}
