package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.KubeService;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author weiwei
 */
@Component("kubeServiceRepository")
public class KubeServiceRepository extends JdbcGenericDao<KubeService, Long> {

	public KubeServiceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, npJdbcTemplate, service);
	}

	public List<KubeService> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		return this.list(page, rows, params, sorter, null);
	}

	public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select a.id,a.name from kube_service a where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		List<KubeService> contracts = this.list(KubeService.class, sql, paramMap);
		List<SimpleBean> beans = new ArrayList<SimpleBean>();
		for (KubeService contract : contracts) {
			beans.add(new SimpleBean(contract.getId(), contract.getName()));
		}
		return beans;
	}

	public int count(List<Param> params) throws Exception {
		return this.count(params, null);
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update kube_service set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public KubeService query(Long id) throws Exception {
		String sql = " select a.* , c.name vendor_name, c.type vendor_type, t.name tenant_name,	p.name pool_group_name  from kube_service a left join cloud_vendor c on a.vendor_id = c.id  left join tenant t on a.tenant_id = t.id "
				+ " left join pool_group p on a.poolgroup_id = p.id where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> param = MapTools.simpleMap(Common.ID, id);
		List<KubeService> list = this.list(KubeService.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public boolean removeByVendor(Long vendorId, Long userId) throws Exception {
		String sql = "update kube_service set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and vendor_id = :vendorId";
		Map<String, Object> params = MapTools.simpleMap(Common.VENDORID, vendorId);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<KubeService> listByVendor(Long vendorId) throws Exception {
		String sql = "select a.* from kube_service a where a.is_deleted = 0 and a.vendor_id = :vendorId";
		Map<String, Object> param = MapTools.simpleMap(Common.VENDORID, vendorId);
		List<KubeService> list = this.list(KubeService.class, sql, param);
		return list;
	}

	public KubeService querByName(String serviceName, Long vendorId) throws Exception {
		String sql = "select a.* from kube_service a where a.is_deleted = 0 and a.name = :serviceName and a.vendor_id = :vendorId  ";
		Map<String, Object> param = MapTools.simpleMap("serviceName", serviceName);
		param.put(Common.VENDORID, vendorId);
		List<KubeService> list = this.list(KubeService.class, sql, param);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}

	public int count(List<Param> params, List<Long> projectIds) {
		String sql = "select count(1) from kube_service a where a.is_deleted = 0";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		if (!ListTool.isEmpty(projectIds)) {
			sql += " and a.project_id in (:projectIds) ";
			paramMap.put("projectIds", projectIds);
		}
		sql = getQueryBuilder().buildRaw(sql, params, null, "a");
		return this.countQuery(sql, paramMap).intValue();
	}

	public List<KubeService> list(int page, int rows, List<Param> params, Map<String, String> sorter,
			List<Long> projectIds) {
		String sql = "select a.* , c.name vendor_name, c.type vendor_type, t.name tenant_name, project.visibility project_visibility  from kube_service a left join cloud_vendor c on a.vendor_id = c.id  left join tenant t on a.tenant_id = t.id "
				+ " left join project project on a.project_id = project.id  " + "where a.is_deleted = 0";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		if (!ListTool.isEmpty(projectIds)) {
			sql += " and a.project_id in (:projectIds) ";
			paramMap.put("projectIds", projectIds);
		}
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(KubeService.class, sql, paramMap);
	}

	public int count(List<Param> params, Long tenantId, Long userId, List<Long> poolIds, List<Long> vdcIds) {
		String sql = "select count(1) from kube_service a where a.is_deleted = 0";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		if (null != tenantId && 0L != tenantId) {
			sql += " and a.tenant_id = :tenantId";
			paramMap.put("tenantId", tenantId);
			if (poolIds != null && !poolIds.isEmpty() && vdcIds != null && !vdcIds.isEmpty()) {
				sql += " and a.vdc_id in (:vdcIds)  and a.poolgroup_id in (:poolIds) ";
				paramMap.put("poolIds", poolIds);
				paramMap.put("vdcIds", vdcIds);
			}
		}
		sql = getQueryBuilder().buildRaw(sql, params, null, "a");
		return this.countQuery(sql, paramMap).intValue();
	}

	public List<KubeService> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long tenantId,
			Long userId, List<Long> poolIds, List<Long> vdcIds) {
		String sql = "select a.* , c.name vendor_name, c.type vendor_type, t.name tenant_name,	p.name pool_group_name  from kube_service a left join cloud_vendor c on a.vendor_id = c.id  left join tenant t on a.tenant_id = t.id "
				+ " left join pool_group p on a.poolgroup_id = p.id where a.is_deleted = 0";
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		if (null != tenantId && 0L != tenantId) {
			sql += " and a.tenant_id = :tenantId";
			paramMap.put("tenantId", tenantId);
			if (poolIds != null && !poolIds.isEmpty() && vdcIds != null && !vdcIds.isEmpty()) {
				sql += " and a.vdc_id in (:vdcIds)  and a.poolgroup_id in (:poolIds) ";
				paramMap.put("poolIds", poolIds);
				paramMap.put("vdcIds", vdcIds);
			}
		}
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
		return this.list(KubeService.class, sql, paramMap);
	}

	@SuppressWarnings("unused")
	private <T> String generateIds(List<T> ipPoolIds) {
		StringBuilder builder = new StringBuilder();
		for (int i = 0; i < ipPoolIds.size(); i++) {
			builder.append("'" + ipPoolIds.get(i) + "'").append(",");
		}
		builder.append(" -1 ");
		return builder.toString();
	}

	public List<KubeService> listByNsId(Long nsId, Long vendorId, Long tenantId) {
		String sql = "SELECT	a.*,	c.name vendor_name,	c.type vendor_type,	t.name tenant_name FROM	kube_service a	LEFT JOIN cloud_vendor c ON a.vendor_id = c.id	LEFT JOIN tenant t ON a.tenant_id = t.id WHERE	a.is_deleted = 0 and a.vendor_id = :vendorId ";
		Map<String, Object> param = MapTools.simpleMap(Common.VENDORID, vendorId);
		param.put(Common.VENDORID, vendorId);
		if (null != nsId && 0 != nsId) {
			sql += " and a.ns_id = :nsId  ";
			param.put("nsId", nsId);
		}
		if (null != tenantId && 0 != tenantId) {
			sql += "  and a.tenant_id = :tenantId ";
			param.put("tenantId", tenantId);
		}
		return this.list(KubeService.class, sql, param);
	}

}
