package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.PortGroupRelation;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 资源池端口组网络关联
 *
 * @author tyl
 * @since 2021/4/22 10:00
 */
@Repository("portGroupRelationRepository")
public class PortGroupRelationRepository extends BasicGenericDao<PortGroupRelation, Long> {
	public PortGroupRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, npJdbcTemplate, service);
	}

	public PortGroupRelation query(Long id) {
		String sql = "select * from port_group_relation where id = :id";
		Map<String, Object> paramsMap = MapTools.simpleMap("id", id);
		return this.list(PortGroupRelation.class, sql, paramsMap).stream().findFirst().orElse(null);
	}

	public boolean remove(Long groupId) {
		String sql = "delete from port_group_relation where pool_group_id = :groupId";
		Map<String, Object> paramsMap = MapTools.simpleMap("groupId", groupId);
		return this.execute(sql, paramsMap) > 0;
	}

	public List<PortGroupRelation> listByGroupId(Long groupId) {
		String sql = "select a.*,b.name  ip_pool_name,b.cidr cidr,c.name port_group_name ,b.version version from port_group_relation a "
				+ " left join ip_pool b on a.ip_pool_id = b.id left join port_group c on a.port_group_id = c.id"
				+ " where a.pool_group_id = :groupId ";
		Map<String, Object> paramsMap = MapTools.simpleMap("groupId", groupId);
		return this.list(PortGroupRelation.class, sql, paramsMap);
	}

	public List<PortGroupRelation> queryByIpAndPool(Long poolGroupId, Long ipPoolId) {
		String sql = "select * from port_group_relation where ip_pool_id = :ipPoolId and pool_group_id = :poolGroupId ";
		Map<String, Object> paramsMap = MapTools.simpleMap("poolGroupId", poolGroupId);
		paramsMap.put("ipPoolId", ipPoolId);
		return this.list(PortGroupRelation.class, sql, paramsMap);
	}

	public List<PortGroupRelation> listByIpPoolId(Long ipPoolId) {
		String sql = "select a.* from port_group_relation a " + " where a.ip_pool_id = :ipPoolId ";
		Map<String, Object> paramsMap = MapTools.simpleMap("ipPoolId", ipPoolId);
		return this.list(PortGroupRelation.class, sql, paramsMap);
	}

}
