package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.ResourceModule;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 基础组件 资源模型
 * @author hp
 *
 */
@Repository("resourceModuleRepository")
public class ResourceModuleRepository extends BasicGenericDao<ResourceModule, Long> {

	public ResourceModuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate, CurrentService service) {
		super(jdbcTemplate, namedParameterJdbcTemplate, service);
	}

	public List<ResourceModule> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws Exception {
		String sql = "select * from resource_module where is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		return this.list(ResourceModule.class, sql, paramMap);
	}
	
	public ResourceModule query(Long id) throws Exception {
		String sql = "select * from resource_module a where a.is_deleted = 0 and a.id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		List<ResourceModule> list = this.list(ResourceModule.class, sql, params);
		if (list.isEmpty()) {
			return null;
		}
		return list.get(0);
	}


	public int count(List<Param> params) throws Exception {
		String sql = "select count(1) from resource_module a where a.is_deleted = 0";
		sql = getQueryBuilder().buildRaw(sql, params, null, "a");
		Map<String, Object> paramMap = getQueryBuilder().getParam(params);
		return this.countQuery(sql, paramMap).intValue();
	}

	public boolean remove(Long id, Long userId) throws Exception {
		String sql = "update resource_module set is_deleted = true , gmt_modify = :gmtModify ,mender_id = :menderId where is_deleted = 0 and id = :id";
		Map<String, Object> params = MapTools.simpleMap("id", id);
		params.put("menderId", userId);
		params.put("gmtModify", new Date());
		return this.execute(sql, params) > 0;
	}

	public List<ResourceModule> listByCategoryId(Long categoryId) {
		String sql = "select * from resource_module a where a.is_deleted = 0 and a.vendor_category_id = :categoryId";
		Map<String, Object> params = MapTools.simpleMap("categoryId", categoryId);
		return this.list(ResourceModule.class, sql, params);
	}

}
