package com.bocloud.cmp.repository;

import com.bocloud.cmp.entity.Storage;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 存储服务器
 * @author wangyu
 * @version 1.0
 * @since 2020年5月15日
 *
 */
@Repository("storageRepository")
public class StorageRepository extends BasicGenericDao<Storage, Long>  {

    public StorageRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    
    public List<Storage> list(int page, int rows, List<Param> params, Map<String, String> sorter)
            {
        String sql = "select a.* from storage a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(Storage.class, sql, paramMap);
    }

    
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) {
        String sql = "select a.id,a.name from storage a where a.is_deleted = 0";
        sql = getQueryBuilder().build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        List<Storage> list = this.list(Storage.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<SimpleBean>();
        for (Storage storage : list) {
            beans.add(new SimpleBean(storage.getId(), storage.getName()));
        }
        return beans;
    }

    
    public int count(List<Param> params) {
        String sql = "select count(1) from storage a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    
    public Storage getByUuid(String uuid) {
        String sql = "select a.* from storage a where a.is_deleted = 0 and a.uuid = :uuid";
        Map<String, Object> params = MapTools.simpleMap("uuid", uuid);
        List<Storage> storages = list(Storage.class, sql, params);
        return storages == null || storages.size() == 0 ? null : storages.get(0);
    }

    
    public void remove(long id) {
        String sql = "update storage a set is_deleted = 1 where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        execute(sql, params);
    }

    
    public void removeByVendor(long vendorId) {
        String sql = "update storage a set is_deleted = 1 where a.vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        execute(sql, params);
    }

    
    public List<Storage> listByVendor(long vendorId) {
        String sql = "select a.* from storage a where a.is_deleted = 0 and a.vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        return list(Storage.class, sql, params);
    }

    
    public Storage get(long id) {
        String sql = "select a.* from storage a where a.id = :id and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        List<Storage> storages = list(Storage.class, sql, params);
        return storages == null || storages.size() == 0 ? null : storages.get(0);
    }

    
    public Storage getByHostName(long vendorId, String host) {
        String sql = "select a.* from storage a " +
                "where a.is_deleted = 0 and a.vendor_id = :vendorId and a.host = :host";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("host", host);
        List<Storage> storages = list(Storage.class, sql, params);
        return storages == null || storages.size() == 0 ? null : storages.get(0);
    }

    
    public Storage getByName(long vendorId, String name) {
        String sql = "select a.* from storage a " +
                "where a.is_deleted = 0 and a.vendor_id = :vendorId and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("name", name);
        List<Storage> storages = list(Storage.class, sql, params);
        return storages == null || storages.size() == 0 ? null : storages.get(0);
    }

    
    public Storage getIp(Long vendorId, String ip) {
        String sql = "select a.* from storage a " +
                "where a.is_deleted = 0 and a.vendor_id = :vendorId and a.ip = :ip";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        params.put("ip", ip);
        List<Storage> storages = list(Storage.class, sql, params);
        return storages == null || storages.size() == 0 ? null : storages.get(0);
    }

    
    public List<Storage> listByCloudServer(long serverId) {
        String sql = "select * from storage where id in (" +
                "   select storage_id from volume where id in (" +
                "       select volume_id from server_volume where server_id = :serverId" +
                "   )" +
                ")";
        Map<String, Object> params = MapTools.simpleMap("serverId", serverId);
        return list(Storage.class, sql, params);
    }
}
