package com.bocloud.cmp.service;

import java.util.List;

import com.bocloud.cmp.entity.CloudServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;

/**
 * 云主机实现类接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年7月19日
 */
public interface CloudServerService {

    /**
     * 查询云主机列表
     *
     * @param pager
     * @param context
     * @return
     */
    GeneralResult<GridBean<CloudServer>> list(Pager pager, RequestContext context);

    /**
     * 云主机批量删除
     *
     * @param ids
     * @param context
     * @return
     */
    GeneralResult batchRemove(List<Long> ids, RequestContext context);

    /**
     * 得到当前云主机的基本信息和统计信息
     *
     * @param id
     * @return the result
     */
    GeneralResult detail(Long id);

    /**
     * 云主机基本信息
     *
     * @param id
     * @return the result
     */
    GeneralResult basic(Long id);

    /**
     * 启动云主机
     *
     * @param id
     * @param context
     * @return the result
     */
    GeneralResult start(Long id, RequestContext context);

    /**
     * 云主机批量启动
     *
     * @param ids
     * @param context
     * @return
     */
    GeneralResult batchStart(List<Long> ids, RequestContext context);

    /**
     * 重启云主机
     *
     * @param id
     * @param context
     * @return the result
     */
    GeneralResult restart(Long id, RequestContext context);


    /**
     * 云主机批量重启
     *
     * @param ids
     * @param context
     * @return
     */
    GeneralResult batchReboot(List<Long> ids, RequestContext context);

    /**
     * 关闭云主机
     *
     * @param id
     * @param context
     * @return the result
     */
    GeneralResult stop(Long id, RequestContext context);

    /**
     * 休眠
     *
     * @param id
     * @param context
     * @return
     */
    GeneralResult hibernated(Long id, RequestContext context);

    /**
     * 云主机批量停止
     *
     * @param ids
     * @param context
     * @return
     */
    GeneralResult batchStop(List<Long> ids, RequestContext context);

    GeneralResult resume(Long serverId, RequestContext context);

    /**
     * 判断名称是否已存在
     *
     * @param name
     * @param vendorId
     * @return
     */
    GeneralResult exists(String name, Long vendorId);

    /**
     * 按照平台类型查询
     *
     * @param params
     * @param context
     * @return
     */
    GeneralResult list(List<Param> params, RequestContext context);

    /**
     * 汇总云主机数量
     *
     * @param userId
     * @param projectId
     * @param tenantId
     * @return
     */
    GeneralResult count(Long userId, Long projectId, Long tenantId);

    /**
     * 查询指定数据中心的模板
     *
     * @param vendorId
     * @param vdcId
     * @return
     */
    GeneralResult<List<CloudServer>> listTemplate(Long vendorId, Long vdcId);


    /**
     * 查询云主机信息
     *
     * @param id
     * @param context
     * @param action
     * @param params
     * @return
     */
    GeneralResult details(Long id, RequestContext context, String action, String params);

    GeneralResult condition(Integer page, Integer rows, String condition, RequestContext context);

    GeneralResult listAvailableVm(Long vendorId, String regionId, RequestContext context);

    GeneralResult getBusinessById(Long id);

    GeneralResult configSeq(String params, RequestContext context);

    GeneralResult availableSizes(Long id);

    GeneralResult generalize(Long id, RequestContext context);

    /**
     * 根据集群维度或平台维度汇总云主机数量
     *
     * @param params
     * @return
     */
	GeneralResult chart(String params);

	/**
	 * 虚机统计信息
	 * @param vendorId
	 * @param context
	 * @return
	 */
	GeneralResult selectAllByVendorId(Long vendorId, RequestContext context);

}
