package com.bocloud.cmp.service;

import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.Image;
import com.bocloud.cmp.entity.Volume;
import com.bocloud.cmp.model.BatchUpdateImageParam;
import com.bocloud.cmp.model.ImgBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

/**
 * 镜像实现类接口
 *
 * @author DZG
 * @since V1.0 2016年9月12日
 */
public interface ImageService {

	/**
	 * 查询镜像列表
	 *
	 * @param page
	 * @param rows
	 * @param params
	 * @param sorter
	 * @param simple
	 * @param condition
	 * @param user
	 * @return
	 */
	GeneralResult list(Pager pager, RequestContext context);

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	GeneralResult<Image> detail(Long id);

	/**
	 * 删除
	 *
	 * @param id
	 * @param user
	 * @return
	 */
	GeneralResult remove(Long id, RequestContext context);

	/**
	 * 修改
	 *
	 * @param bean
	 * @param userId
	 * @return
	 */
	GeneralResult modify(ImgBean bean, Long userId);

	/**
	 * 修改,只针对数据
	 *
	 * @param bean
	 * @param userId
	 * @return
	 */
	GeneralResult update(ImgBean bean, Long userId);

	/**
	 * 创建镜像
	 *
	 * @param bean
	 * @param context
	 * @return
	 */
	GeneralResult create(Image bean, RequestContext context);

	/**
	 * 查询指定数据存储下的镜像
	 *
	 * @param storeId
	 * @return
	 */
	GeneralResult listByStoreId(Long storeId);

	/**
	 * 查询云平台下的镜像
	 *
	 * @param vendorId
	 * @return
	 */
	GeneralResult listByVendor(Long vendorId);

	/**
	 * 根据租户查询
	 *
	 * @param params
	 * @param user
	 * @return
	 */
	GeneralResult listByTenant(String params, RequestContext context);

	/**
	 * 批量删除
	 *
	 * @param params
	 * @param user
	 * @return
	 */
	GeneralResult batchRemove(List<Long> ids, RequestContext context);

	/**
	 * 根据命名规则查询镜像(仅限于openstack,深交所专用)
	 *
	 * @param reg
	 * @param vendorId
	 * @return
	 */
	GeneralResult listByReg(String reg, Long vendorId);

	/**
	 * 汇总镜像数量
	 *
	 * @param userId
	 * @param projectId
	 * @param tenantId
	 * @return
	 */
	GeneralResult count(Long userId, Long projectId, Long tenantId);

	/**
	 * 判断物理机镜像是否存在
	 *
	 * @param osType
	 * @param osVersion
	 * @return
	 */
	GeneralResult imageExist(String osType, String osVersion);

	/**
	 * 根据osType跟osVersion删除物理机镜像
	 *
	 * @param osType
	 * @param osVersion
	 * @param userId
	 * @return
	 */
	GeneralResult remove(String osType, String osVersion, Long userId);

	/**
	 * 批量更新
	 *
	 * @param params
	 * @param user
	 * @return
	 */
	GeneralResult batchUpdate(BatchUpdateImageParam params, RequestContext context);

	/**
	 * 镜像部分修改
	 *
	 * @param id
	 * @param params
	 * @param user
	 * @param action
	 * @return
	 */
	GeneralResult patch(Long id, JSONObject params, RequestContext context, String action);

	/**
	 * 下载镜像
	 *
	 * @param id
	 * @return
	 */
	GeneralResult download(Long id);

	public GeneralResult<List<Volume>> listVolumeByImage(long imageId);

	public GeneralResult<GridBean<CloudServer>> listServerByImage(long imageId, Pager pager);

	default GeneralResult isoList() {
		return new GeneralResult(false, ImageService.class.getSimpleName() + "的isoList方法未实现");
	}

	GeneralResult condition(String condition, RequestContext context);

}
