package com.bocloud.cmp.service;

import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.Ip;
import com.bocloud.cmp.model.IpPatchParams;
import com.bocloud.cmp.model.IpScanParam;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

public interface IpService {

	/**
	 * 列表展示
	 *
	 * @param pager
	 * @return
	 */
	GeneralResult<GridBean<Ip>> list(Pager pager);

	/**
	 * 根据condition列表展示
	 *
	 * @param condition
	 * @param context
	 * @return
	 */
	GeneralResult condition(String condition, RequestContext context);

	/**
	 * 详情
	 *
	 * @param id
	 * @return
	 */
	GeneralResult<Ip> detail(Long id);

	/**
	 * 根据平台ID以及请求IP数量获取IP
	 *
	 * @param vendorId
	 * @param count
	 * @param version
	 * @return
	 */
	GeneralResult<List<Ip>> achieveIps(Long count, String version);

	/**
	 * 从指定IP池分配IP
	 *
	 * @param poolId
	 * @param count
	 * @param version
	 * @return
	 */
	GeneralResult<List<Ip>> achievePoolIps(Long poolId, Long count, String version);

	/**
	 * 根据IP池ID以及请求IP数量获取IP(随机分配，不按照顺序分配)
	 *
	 * @param poolId
	 * @param count
	 * @param version
	 * @return
	 */
	GeneralResult<List<Ip>> achieveDhcpIps(Long poolId, Long count, String version);

	/**
	 * 处理平台垃圾下的IP垃圾数据
	 *
	 * @param vendorId
	 * @return
	 */
	GeneralResult processRubbishIp(Long vendorId);

	/**
	 * 从网络资源池下的IP池中分配IP地址
	 *
	 * @param vendorId
	 * @param poolGroupId
	 * @param count
	 * @return
	 */
	GeneralResult<List<Ip>> achieveIps(Long vendorId, Long poolGroupId, Integer count);

	/**
	 * 从nsx路由器端口下子网IP池中分配
	 *
	 * @param routerPortId
	 * @param count
	 * @param version
	 * @return
	 */
	GeneralResult<List<Ip>> achieveNsxIps(Long routerPortId, Integer count, String version);

	/**
	 * 从nsx分段下子网ip池中分配
	 *
	 * @param segmentId
	 * @param count
	 * @param version
	 * @return
	 */
	GeneralResult<List<Ip>> achieveNsxSegmentIps(Long segmentId, Integer count, String version);

	GeneralResult alarmIps(Long vendorId);

	/**
	 * 根据子网id删除
	 *
	 * @param ip
	 * @param subnetId
	 * @return
	 */
	GeneralResult deleteBySubnetId(String ip, Long subnetId);

	/**
	 * 批量保存
	 *
	 * @param object
	 * @param context
	 * @return
	 */
	GeneralResult batchSave(JSONObject object, RequestContext context);

	/**
	 * 删除IP
	 *
	 * @param id
	 * @return
	 */
	GeneralResult remove(Long id);

	/**
	 * ip扫描
	 *
	 * @param param
	 * @param context
	 * @return
	 */
	GeneralResult scan(IpScanParam param, RequestContext context);

	/**
	 * 预分配IP地址
	 *
	 * @param poolId
	 * @param count
	 * @param context
	 * @return
	 */
	GeneralResult preAllocation(Long poolId, Integer count, RequestContext context);

	/**
	 * 检查IP是否被占用
	 *
	 * @param poolId
	 * @param ips
	 * @param context
	 * @return
	 */
	GeneralResult checkIp(Long poolId, List<String> ips, RequestContext context);

	/**
	 * ip操作
	 *
	 * @param id
	 * @param status
	 * @param remark
	 * @param context
	 * @return
	 */
	GeneralResult action(Long id, IpPatchParams.Action status, String remark, RequestContext context);

	/**
	 * IP批量操作
	 *
	 * @param ids
	 * @param action
	 * @param remark
	 * @param context
	 * @return
	 */
	GeneralResult batchUpdate(List<Long> ids, IpPatchParams.Action action, String remark, RequestContext context);

	/**
	 * 根据IP池查询可用ip
	 * 
	 * @param poolId
	 * @param count
	 * @param version
	 * @return
	 */
	GeneralResult<List<Ip>> assginIps(Long poolId, Long count, String version);
}
