package com.bocloud.cmp.service;

import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.ServerLabelParam;
import com.bocloud.cmp.entity.Label;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

public interface LabelService {

    /**
     * 查询标签列表
     *
     * @param pager
     * @param context
     * @return
     */
    GeneralResult<GridBean<Label>> list(Pager pager, RequestContext context);

    /**
     * 查看标签详情
     *
     * @param id
     * @return
     */
    GeneralResult<Label> detail(Long id);

    /**
     * 创建标签
     *
     * @param label
     * @return
     */
    GeneralResult<Label> create(Label label);

    /**
     * 更新标签
     *
     * @param label
     * @return
     */
    GeneralResult<Label> modify(Label label);

    /**
     * 删除标签
     *
     * @param id
     * @return
     */
    GeneralResult remove(Long id);
    

    /**
     * 资源标签绑定
     *
     * @param serverLabelParam
     * @return
     */
    GeneralResult createResources(ServerLabelParam serverLabelParam, RequestContext context);

    /**
     * @return void 数组格式，如果存在已经删除的标签，则在方法中已经移除
     * @Author: lxy
     * @Description: 云主机、浮动ip、硬盘对应标签校验，可能存在标签已被删除的情况，如果被删除，则此方法自动移除已被删除的标签
     * @Date 2021/9/6 14:11
     * @Param parseArray 数组格式，传入值为云主机、浮动ip、硬盘对应标签
     * @throws:
     **/
    public GeneralResult isExistsValues(List<JSONObject> parseArray);
}
