package com.bocloud.cmp.service;

import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.PoolGroupZone;
import com.bocloud.cmp.entity.Region;
import com.bocloud.cmp.entity.Zone;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;

/**
 * 区域Service接口(子区域为zone)
 *
 * @author dongkai
 */
public interface RegionService {
	
	/**
	 * 根据ID查询
	 *
	 * @param id
	 * @return
	 */
	public GeneralResult<List<Region>> list(Long vendorId);

	/**
	 * 根据ID查询
	 *
	 * @param id
	 * @return
	 */
	public GeneralResult<List<Region>> list(Long vendorId, String resourceType);

	/**
	 * 查询子区域
	 *
	 * @param regionId
	 * @param vendorId
	 * @return
	 */
	public GeneralResult<List<Zone>> listZone(Long vendorId, String regionId);

	/**
	 * 根据资源池查询关联区域
	 * 
	 * @param object
	 * @return
	 * @throws Exception
	 */
	public GeneralResult<List<PoolGroupZone>> listRegionByGroup(JSONObject object);

	/**
	 * 自服务端根据业务查询平台下的区域信息
	 *
	 * @param object
	 * @param projectId
	 * @return
	 */
	GeneralResult<List<Region>> listByBusiness(JSONObject object, RequestContext context);

	/**
	 * 查询region列表
	 * 
	 * @param params
	 * @param user
	 * @return
	 */
	public GeneralResult<List<Region>> condition(String condition, RequestContext context);

	public GeneralResult zones(Long id, Long vendorId, String type);

	public GeneralResult dicToMap(String vendorType);

	/**
	 * 删除区域下的资源
	 * @param regionId
	 * @param vendorId
	 * @param userId
	 * @throws Exception
	 */
	public void removeResources(String regionId, Long vendorId, Long userId) throws Exception;

	/**
	 * 同步云平台下的region
	 * @param vendor
	 * @param userId
	 * @param currentRegions
	 * @throws Exception
	 */
	public GeneralResult syncRegion(CloudVendor vendor, Long userId, List<String> currentRegions) throws Exception;
}
