package com.bocloud.cmp.service;

import java.util.List;
import java.util.Map;

import com.bocloud.cmp.entity.Rack;
import com.bocloud.cmp.entity.Room;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;


/**
 * 机房抽象Service接口
 * 
 * @author wei wei
 * @version 1.0
 * @since 2016.02.27
 */
public interface RoomService {

	/**
	 * 添加机房
	 * 
	 * @param room the room
	 * @return the result
	 */
	public GeneralResult<Room> create(Room room, Long userId);

	/**
	 * 更新机房
	 * 
	 * @param bean   the room bean
	 * @param userId the user id
	 * @return the result
	 */
	public GeneralResult<String> modify(Room bean, Long userId);

	/**
	 * 逻辑删除机房
	 * 
	 * @param id the id
	 * @param id the user id
	 * @return the result
	 */
	public GeneralResult<String> remove(Long id, Long userId);

	/**
	 * 查询机房详细
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult<Room> basic(Long id);

	/**
	 * 查询机房列表
	 * 
	 * @param page   the page
	 * @param rows   the rows
	 * @param params the params
	 * @param sorter the sorter
	 * @param simple the simple
	 * @return the result
	 */
	public GeneralResult<GridBean<Room>> list(Pager pager, RequestContext context);

	/**
	 * 得到当前机房下的机柜列表
	 * 
	 * @param page   the page
	 * @param rows   the rows
	 * @param params the params
	 * @param sorter the sorter
	 * @return the result
	 */
	public GeneralResult<GridBean<Rack>> listRacks(int page, int rows, List<Param> params, Map<String, String> sorter);

	/**
	 * 得到当前机房的基本信息和统计信息
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult<Room> detail(Long id);

	/**
	 * 加入数据中心
	 * 
	 * @param id     the room id
	 * @param dcId   the dataCenter id
	 * @param userId the user id
	 * @return the result
	 */
	public GeneralResult<String> inDC(Long id, Long dcId, Long userId);

	/**
	 * 移出数据中心
	 * 
	 * @param id     the room id
	 * @param userId the user id
	 * @return the result
	 */
	public GeneralResult<String> outDC(Long id, Long userId);

	/**
	 * 查询空闲机房列表
	 * 
	 * @return the room list
	 */
	public GeneralResult<List<Room>> freeRooms();

	/**
	 * 检查机柜位置是否被使用
	 * 
	 * @param id    the id
	 * @param axisX the axisX
	 * @param axisY the axisY
	 * @return the result
	 */
	public GeneralResult<String> checkPoint(Long id, Integer axisX, Integer axisY);

	/**
	 * 机房下没有分配的服务器
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult freeNodes(Long id);

	/**
	 * 机房下没有分配的服务器
	 * 
	 * @param id the id
	 * @return the result
	 */
	public GeneralResult freeManagers(Long id);

	/**
	 * 查询详情 and baseinfo
	 * 
	 * @param id
	 * @param action
	 * @return
	 */
	public GeneralResult<Room> details(Long id, String action);

	public GeneralResult<List<Room>> condition(String condition, RequestContext context);
}
