package com.bocloud.cmp.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.OrderMessage;
import com.bocloud.cmp.domain.QuotaBean;
import com.bocloud.cmp.domain.ResultTrend;
import com.bocloud.cmp.domain.TrendData;
import com.bocloud.cmp.entity.Business;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.Cluster;
import com.bocloud.cmp.entity.ElasticIp;
import com.bocloud.cmp.entity.Flavor;
import com.bocloud.cmp.entity.FloatingIp;
import com.bocloud.cmp.entity.Image;
import com.bocloud.cmp.entity.NetworkCard;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Region;
import com.bocloud.cmp.entity.ResourceQuota;
import com.bocloud.cmp.entity.ResourceQuota.Action;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.entity.Volume;
import com.bocloud.cmp.entity.Zone;
import com.bocloud.cmp.enums.BocloudStatus;
import com.bocloud.cmp.enums.OrderItemStatus;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.enums.ServerAction;
import com.bocloud.cmp.internal.CosInternalService;
import com.bocloud.cmp.internal.SmsBusinessService;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.model.ChargeResultTrend;
import com.bocloud.cmp.model.ChargeTrendData;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.repository.ClusterRepository;
import com.bocloud.cmp.repository.DictionaryRepository;
import com.bocloud.cmp.repository.ElasticIpRepository;
import com.bocloud.cmp.repository.FlavorRepository;
import com.bocloud.cmp.repository.FloatingIpRepository;
import com.bocloud.cmp.repository.ImageRepository;
import com.bocloud.cmp.repository.NetworkCardRepository;
import com.bocloud.cmp.repository.RegionRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.bocloud.cmp.repository.VolumeRepository;
import com.bocloud.cmp.repository.ZoneRepository;
import com.bocloud.cmp.service.CloudServerService;
import com.bocloud.cmp.util.DateUtils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

import static com.bocloud.cmp.util.DateUtils.timeToList;
import static java.util.stream.Collectors.groupingBy;

/**
 * 云主机实现类
 *
 * @author DZG
 * @since V1.0 2016年9月12日
 */
@Service("cloudServerService")
@Slf4j
public class CloudServerServiceImpl implements CloudServerService {

	@Autowired
	private SmsProjectService smsProjectService;
	@Autowired
	private FlavorRepository flavorRepository;
	@Autowired
	private RegionRepository regionRepository;
	@Autowired
	private ZoneRepository zoneRepository;
	@Autowired
	private CloudServerRepository cloudServerRepository;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private ImageRepository imageRepository;
	@Autowired
	private ServerRepository serverRepository;
	@Autowired
	private NetworkCardRepository networkCardRepository;
	@Autowired
	private VolumeRepository volumeRepository;
	@Autowired
	private AmqpTemplate amqpTemplate;
	@Autowired
	private ClusterRepository clusterRepository;
	@Autowired
	private DictionaryRepository dictionaryRepository;
	@Autowired
	private SmsTenantService smsTenantService;
	@Autowired
	private SmsUserService smsUserService;
	@Autowired
	private SmsBusinessService smsBusinessService;
	@Autowired
	private CosInternalService cosInternalService;
	@Autowired
	private FloatingIpRepository floatingIpRepository;
	@Autowired
	private ElasticIpRepository elasticIpRepository;

	@Override
	public GeneralResult<GridBean<CloudServer>> list(Pager pager, RequestContext context) {
		GridBean<CloudServer> gridBean;
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
			sorter.put("gmtCreate", Common.ONE);
			sorter.put("id", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			Long clusterId = null;
			String labelContent = null;
			for (Param param : params) {
				Map<String, Object> paramMap = param.getParam();
				if (paramMap.containsKey("clusterId")) {
					clusterId = Long.valueOf(paramMap.get("clusterId").toString());
					paramMap.remove("clusterId");
				}
				if (paramMap.containsKey("labelContent")) {
					labelContent = paramMap.get("labelContent").toString();
					paramMap.remove("labelContent");
				}
			}
			int total = 0;
			List<CloudServer> list = Lists.newArrayList();
			if (context.getCatalog().equals(Catalog.Tenant)) {
				if (simple) {
					rows = Integer.MAX_VALUE;
				}
				// 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
				List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
				List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
				total = this.cloudServerRepository.count(null, context.getTenant(), projectIds, params, labelContent);
				list = this.cloudServerRepository.list(page, rows, null, context.getTenant(), projectIds, params,
						sorter, labelContent);
				gridBean = new GridBean<>(page, rows, total, list);
			} else if (context.getCatalog().equals(Catalog.User)) {
				if (simple) {
					rows = Integer.MAX_VALUE;
				}
				// 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
				List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
				List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
				List<Long> projectIds = new ArrayList<>();
				List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!userManagerProjectsIds.isEmpty()) {
					projectIds.addAll(userManagerProjectsIds);
				}
				List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!visibleProjectsIds.isEmpty()) {
					projectIds.addAll(visibleProjectsIds);
				}
				total = this.cloudServerRepository.count(context.getTarget(), null, projectIds, params, labelContent);
				list = this.cloudServerRepository.list(page, rows, context.getTarget(), null, projectIds, params,
						sorter, labelContent);
				gridBean = new GridBean<>(page, rows, total, list);
			} else {
				// 管理端搜索
				if (null != clusterId) {
					total = this.cloudServerRepository.count(clusterId, params);
					list = this.cloudServerRepository.list(page, rows, clusterId, params, sorter);
					gridBean = new GridBean<>(page, rows, total, list);
				} else if (simple) {
					total = this.cloudServerRepository.count(params, 0L);
					List<SimpleBean> beans = this.cloudServerRepository.list(params, sorter);
					gridBean = new GridBean(1, 1, total, beans);
				} else {
					total = this.cloudServerRepository.vmCount(params, labelContent);
					list = this.cloudServerRepository.list(page, rows, params, sorter, labelContent);
					gridBean = new GridBean<>(page, rows, total, list);
				}
			}
			if (!simple) {
				List<Long> tenantIds = new ArrayList<>();
				List<Long> userIds = new ArrayList<>();
				List<Long> businessIds = new ArrayList<>();
				List<Long> projectIds = new ArrayList<>();
				for (CloudServer cloudServer : list) {
					if (cloudServer.getTenantId() != null) {
						tenantIds.add(cloudServer.getTenantId());
					}
					if (cloudServer.getBusinessId() != null) {
						businessIds.add(cloudServer.getBusinessId());
					}
					if (cloudServer.getProjectId() != null) {
						projectIds.add(cloudServer.getProjectId());
					}
					if (cloudServer.getCreatorId() != null) {
						userIds.add(cloudServer.getCreatorId());
					}
					if (cloudServer.getOwnerId() != null) {
						userIds.add(cloudServer.getOwnerId());
					}
					if (cloudServer.getMenderId() != null) {
						userIds.add(cloudServer.getMenderId());
					}
				}
				Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
						.collect(Collectors.toMap(User::getId, User::getName));
				Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
						.collect(Collectors.toMap(Project::getId, Project::getName));
				Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
						.collect(Collectors.toMap(Business::getId, Business::getName));
				Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
						.collect(Collectors.toMap(Tenant::getId, Tenant::getName));
				for (CloudServer server : list) {
					// 设置云主机归属信息
					if (server.getTenantId() != null) {
						server.setTenantName(tenantMap.get(server.getTenantId()));
					}
					if (server.getBusinessId() != null) {
						server.setBusinessName(businessMap.get(server.getBusinessId()));
					}
					if (server.getProjectId() != null) {
						server.setProjectName(projectMap.get(server.getProjectId()));
					}
					if (server.getCreatorId() != null) {
						server.setCreatorName(userMap.get(server.getCreatorId()));
					}
					if (server.getOwnerId() != null) {
						server.setOwnerName(userMap.get(server.getOwnerId()));
					}
					if (server.getMenderId() != null) {
						server.setMenderName(userMap.get(server.getMenderId()));
					}
					// 设置主机名称
					Server host = serverRepository.basic(server.getHostId());
					host = Optional.ofNullable(host).orElse(new Server());
					server.setHostName(host.getName());
					server.setClusterName(host.getClusterName());
					server.setVdcName(host.getVdcName());
					server.setDisk((null == server.getDisk() || 0 == server.getDisk()) ? server.getTotalDisks()
							: server.getDisk());
					if (!StringUtils.isEmpty(server.getResourceGroup())) {
						String resourceGroupName = dictionaryRepository.getNameByValue(server.getResourceGroup());
						server.setResourceGroupName(resourceGroupName);
					}
					if (server.getVendorType().equalsIgnoreCase("CTSTACK")) {
						FloatingIp floatingIp = floatingIpRepository.getByInstanceId(server.getInstanceId());
						if (null != floatingIp) {
							server.setFipId(floatingIp.getId());
							//server.setFloatingIp(floatingIp.getFloatingIpAddress());
							if (StringUtils.isEmpty(server.getPublicIps())) {
								server.setPublicIps(JSON.toJSONString(Collections.singletonList(floatingIp.getFloatingIpAddress())));
							}
						}
					}
				}
				gridBean.setRows(list);
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Query vm list fail:{}", e.getMessage());
			return new GeneralResult<>(false, "查询失败");
		}
	}

	public GeneralResult snapshotQuotasCallBack(RequestContext context, Action action) {
		try {
			if (null == context.getCatalog()) {
				return new GeneralResult<>(true, "快照配额分配成功");
			}
			if (context.getCatalog().equals(Catalog.User) || context.getCatalog().equals(Catalog.Tenant)) {
				List<ResourceQuota> quotas = new ArrayList<>();
				String snapshotServiceCode = "meta.snapshot";
				quotas.add(new ResourceQuota(snapshotServiceCode, 1L));
				GeneralResult updateQuotas = cosInternalService.updateQuotas(quotas, action, context.getTenant());
				if (updateQuotas.isFailed()) {
					return new GeneralResult<>(false, "快照创建失败：" + updateQuotas.getMessage());
				}
			}
			return new GeneralResult<>(true, "快照配额分配成功");
		} catch (Exception e) {
			log.error("快照创建失败：" + e.getMessage());
			return new GeneralResult<>(false, "快照创建失败：" + e.getMessage());
		}
	}

	/**
	 * @param ips
	 * @return void
	 * @Author: lxy
	 * @Description: privateIps 中获取所有ip
	 * @Date 2021/5/16 17:30
	 * @Param privateIps
	 * @throws:
	 **/
	private void getIps(String privateIps, List<String> ips) {
		List<JSONObject> privateIpsList = JSONArray.parseArray(privateIps, JSONObject.class);
		if (ListTool.isEmpty(privateIpsList)) {
			return;
		}
		for (JSONObject json : privateIpsList) {
			String address = json.getString("address");
			if (StringUtils.isEmpty(address)) {
				continue;
			}
			ips.add(address);
		}
	}

	public GeneralResult changeServerOrderStatus(CloudServer server, String result, RequestContext context) {
		try {
			OrderMessage message = new OrderMessage(server.getUuid(), OrderMessage.Operation.Cancelation,
					OrderItemStatus.UnsubscribeSuccess, null, result, null, null, null,
					ResourceCategory.Computer.name(), context);
			List<QuotaBean> quotas = new ArrayList<>();
			quotas.add(new QuotaBean(server.getVendorType().toLowerCase() + ".standard.server", 1));
			quotas.add(new QuotaBean("meta.cpu", server.getCpu()));
			quotas.add(new QuotaBean("meta.mem", server.getMemory().intValue()));
			quotas.add(new QuotaBean("meta.disk", server.getDisk()));
			message.setQuotas(quotas);
			message.setCategory(ResourceCategory.Computer.name());
			amqpTemplate.convertAndSend("resource.result.queue", message);
			return new GeneralResult(true, "订单状态更新成功！");
		} catch (Exception e) {
			log.error("订单状态更新失败：" + e.getMessage());
			return new GeneralResult(false, "订单状态更新失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult batchRemove(List<Long> ids, RequestContext context) {
		return batchAction(ids, context, ServerAction.DELETE, "删除");
	}

	@Override
	public GeneralResult<CloudServer> detail(Long id) {
		try {
			CloudServer server = cloudServerRepository.query(id);
			Assert.notNull(server, "云主机不存在");
			// 设置云主机归属
			if (null != server.getOwnerId() && 0L != server.getOwnerId()) {
				GeneralResult<User> ownerDetail = smsUserService.detail(server.getOwnerId());
				User owner = ownerDetail.getData();
				if (owner != null) {
					server.setOwnerName(owner.getName());
				}
			}
			if (null != server.getMenderId() && 0L != server.getMenderId()) {
				GeneralResult<User> menderDetail = smsUserService.detail(server.getMenderId());
				User mender = menderDetail.getData();
				if (mender != null) {
					server.setMenderName(mender.getName());
				}
			}
			if (null != server.getCreatorId() && 0L != server.getCreatorId()) {
				GeneralResult<User> creatorDetail = smsUserService.detail(server.getCreatorId());
				User creator = creatorDetail.getData();
				if (creator != null) {
					server.setCreatorName(creator.getName());
				}
			}
			if (null != server.getProjectId() && 0L != server.getProjectId()) {
				GeneralResult<Project> projectResult = smsProjectService.detail(server.getProjectId());
				if (projectResult.isFailed() || null == projectResult.getData()) {
					return new GeneralResult(false, projectResult.getMessage());
				}
				Project project = projectResult.getData();
				if (null != project) {
					server.setProjectName(project.getName());
				}
			}
			if (null != server.getBusinessId() && 0L != server.getBusinessId()) {
				GeneralResult<Business> businessDetail = smsBusinessService.detail(server.getBusinessId());
				Business business = businessDetail.getData();
				if (business != null) {
					server.setBusinessName(business.getName());
				}
			}
			// 设置模板名称
			if (null != server.getTemplateId()) {
				CloudServer temp = cloudServerRepository.get(server.getTemplateId());
				if (null != temp) {
					server.setTemplate(temp.getName());
					server.setTemplateUuid(temp.getInstanceId());
				}
			}
			// 设置主机名称
			Server host = serverRepository.query(server.getHostId());
			if (null != host) {
				server.setHostName(host.getName());
				server.setClusterName(host.getClusterName());
				server.setVdcName(host.getVdcName());
			}
			// 网络适配器
			List<NetworkCard> cards = networkCardRepository.listByHost(id, "VM");
			server.setNetcards(cards);
			server.setPassword(null);
			Flavor flavor = flavorRepository.getByFlavorId(server.getFlavorUuid(), server.getVendorId(),
					server.getRegionId());
			if (null != flavor) {
				server.setFlavorName(flavor.getName());
				server.setFlavor(flavor);
			}
			Image image = imageRepository.getByImageUuid(server.getImageUuid());
			server.setImageName(null != image ? image.getName() : server.getOsVersion());
			if (!StringUtils.isEmpty(server.getRegion())) {
				Region region = regionRepository.getByName(server.getRegion(), server.getVendorId());
				if (null != region) {
					server.setRegionName(region.getName());
				}
			}
			if (!StringUtils.isEmpty(server.getZone())) {
				List<Zone> zoneId = zoneRepository.listByZoneId(server.getZone(), server.getVendorId());
				if (null != zoneId && zoneId.size() > 0) {
					server.setZoneName(zoneId.get(0).getName());
				}
			}
			List<Volume> volumes = volumeRepository.listByServerId(server.getId(), server.getVendorId(), false);
			Integer total = 0;
			if (!ListTool.isEmpty(volumes)) {
				for (Volume volume : volumes) {
					total += volume.getSize() == null ? 0 : volume.getSize();
				}
				server.setTotalDisks(total);
			}
			// 云主机标签
			List<JSONObject> parseArray = JSON.parseArray(server.getResourceLabel(), JSONObject.class);
			List<Volume> volumesList = volumeRepository.listByServerId(server.getId(), server.getVendorId(), true);
			server.setBootFromVolume(!ListTool.isEmpty(volumesList));
			//云主机浮动ip id
			if (StringUtils.isNotBlank(server.getPublicIps())) {
				List<String> publicIps = JSONArray.parseArray(server.getPublicIps(), String.class);
				if (!ListTool.isEmpty(publicIps)) {
					if ("ALIYUN".equalsIgnoreCase(server.getVendorType()) || "HUAWEI".equalsIgnoreCase(server.getVendorType())
							|| "HCSO".equalsIgnoreCase(server.getVendorType()) || "AWS".equalsIgnoreCase(server.getVendorType())) {
						ElasticIp elasticIp = elasticIpRepository.queryByPublicIp(publicIps.get(0));
						if (null != elasticIp) {
							server.setFipId(elasticIp.getId());
						}
					}else {
						FloatingIp floatingIp = floatingIpRepository.queryByPublicIp(publicIps.get(0));
						if (null != floatingIp) {
							server.setFipId(floatingIp.getId());
						}
					}
				}
			}
			return new GeneralResult(true, server, "查询成功!");
		} catch (Exception e) {
			log.error("get cloudserver failure:", e);
			return new GeneralResult(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult basic(Long id) {
		try {
			CloudServer server = cloudServerRepository.query(id);
			return new GeneralResult(true, server, "查询成功!");
		} catch (Exception e) {
			log.error("get cloudserver failure:", e);
			return new GeneralResult(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult start(Long serverId, RequestContext context) {
		return null;
	}

	public GeneralResult batchAction(List<Long> idArray, RequestContext context, ServerAction action, String message) {
		return null;
	}

	@Override
	public GeneralResult batchStart(List<Long> ids, RequestContext context) {
		return batchAction(ids, context, ServerAction.START, "启动");
	}

	@Override
	public GeneralResult batchReboot(List<Long> ids, RequestContext context) {
		return batchAction(ids, context, ServerAction.REBOOT, "重启");
	}

	@Override
	public GeneralResult batchStop(List<Long> ids, RequestContext context) {
		return batchAction(ids, context, ServerAction.STOP, "停止");
	}

	@Override
	public GeneralResult exists(String name, Long vendorId) {
		try {
			boolean exists = cloudServerRepository.exists(name, vendorId);
			if (exists) {
				return new GeneralResult(true, "该名称已存在");
			}
			return new GeneralResult(false, "该名称不存在");
		} catch (Exception e) {
			log.error("Get error message:", e);
			return new GeneralResult(false, "查询失败");
		}
	}

	@Override
	public GeneralResult list(List<Param> params, RequestContext context) {
		try {
			Long subnetId = null;
			Long clusterId = null;
			String labelContent = null;
			if (null == params) {
				params = new ArrayList<>();
			} else {
				for (Param param : params) {
					Map<String, Object> paramMap = param.getParam();
					if (paramMap.containsKey("subnetId")) {
						subnetId = Long.valueOf(paramMap.get("subnetId").toString());
						paramMap.remove("subnetId");
					}
					if (paramMap.containsKey("clusterId")) {
						clusterId = Long.valueOf(paramMap.get("clusterId").toString());
						paramMap.remove("clusterId");
					}
					if (paramMap.containsKey("labelContent")) {
						labelContent = paramMap.get("labelContent").toString();
						paramMap.remove("labelContent");
					}
				}
			}
			List<CloudServer> list = new ArrayList<>();
			Long tenantId = context.getTenant();
			Long projectId = context.getProject();
			// 当前用户为租户
			if (0 != tenantId) {
				List<Project> projects = smsProjectService.projectsByCondition(context.getTarget(),
						context.getCatalog());
				List<Long> projectIds = new ArrayList<>();
				// 此时查询的是全部用户全部项目的信息
				if (null != projectId && 0L != projectId) {
					projectIds.add(projectId);
				} else if (!ListTool.isEmpty(projects)) {
					projects.forEach(project -> {
						projectIds.add(project.getId());
					});
				} else {
					// 当前用户不属于任何项目，则登录之后查看不到任何云主机信息
					projectIds.add(-1L);
				}
				list = this.cloudServerRepository.list(0, 0, params, null, subnetId, null, projectIds, null);
			} else if (null != clusterId) {
				list = this.cloudServerRepository.list(0, 0, clusterId, new ArrayList<>(), null);
			} else {
				list = this.cloudServerRepository.list(0, 0, params, null, labelContent, null, null);
			}
			for (CloudServer server : list) {
				// 设置主机名称
				Server host = serverRepository.query(server.getHostId());
				if (null != host) {
					server.setHostName(host.getName());
					server.setClusterName(host.getClusterName());
					server.setVdcName(host.getVdcName());
				}
			}
			return new GeneralResult(true, list, "查询成功");
		} catch (Exception e) {
			log.error("query error message：", e);
			return new GeneralResult(false, "查询云主机信息失败");
		}
	}

	private GeneralResult<JSONArray> getStatusStatistics(RequestContext context) {
		try {
			JSONArray result = new JSONArray();
			List<CloudServer> userServerList = getServerList(context, false);
			Map<String, List<CloudServer>> serverMap = userServerList.stream()
					.collect(groupingBy(CloudServer::getStatus));
			HashMap<String, List<CloudServer>> resultMap = new HashMap<>();
			for (Map.Entry<String, List<CloudServer>> serverEntry : serverMap.entrySet()) {
				String key;
				if (serverEntry.getKey().equals(BocloudStatus.Server.RUNNING.name())
						|| serverEntry.getKey().equals(BocloudStatus.Basic.ENABLED.name())) {
					key = "开机";
				} else if (serverEntry.getKey().equals(BocloudStatus.Server.STOPPED.name())) {
					key = "关机";
				} else if (serverEntry.getKey().equals(BocloudStatus.Server.ERROR.name())) {
					key = "断开";
				} else if (serverEntry.getKey().equals(BocloudStatus.Server.EXCEPTION.name())) {
					key = "异常";
				} else if (serverEntry.getKey().equals(BocloudStatus.Server.PAUSED.name())) {
					key = "暂停";
				} else if (serverEntry.getKey().equals(BocloudStatus.Server.SUSPENDED.name())) {
					key = "挂起";
				} else {
					key = "其他";
				}
				List<CloudServer> serverList = Optional.ofNullable(serverEntry.getValue()).orElse(new ArrayList<>());
				if (resultMap.containsKey(key)) {
					List<CloudServer> list = resultMap.get(key);
					list.addAll(serverList);
					resultMap.put(key, list);
				} else {
					resultMap.put(key, serverList);
				}
			}
			for (Map.Entry<String, List<CloudServer>> entry : resultMap.entrySet()) {
				JSONObject jsonObject = new JSONObject();
				jsonObject.put("name", entry.getKey());
				jsonObject.put("value", entry.getValue().size());
				result.add(jsonObject);
			}
			return new GeneralResult<>(true, result, "查询成功");
		} catch (Exception e) {
			log.error("get cloud server number count error,", e);
			return new GeneralResult<>(false, "查询失败:" + e.getMessage());
		}
	}

	private List<CloudServer> getServerList(RequestContext context, Boolean containRemove) {
		if (context.getCatalog().equals(Catalog.Tenant)) {
			// 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
			List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
			List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
			return this.cloudServerRepository.listAll(1, Integer.MAX_VALUE, null, context.getTenant(), projectIds, null,
					null, containRemove);
		} else if (context.getCatalog().equals(Catalog.User)) {
			// 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
			List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
			List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
			List<Long> projectIds = new ArrayList<>();
			List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
					.collect(Collectors.toList());
			if (!userManagerProjectsIds.isEmpty()) {
				projectIds.addAll(userManagerProjectsIds);
			}
			List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId).collect(Collectors.toList());
			if (!visibleProjectsIds.isEmpty()) {
				projectIds.addAll(visibleProjectsIds);
			}
			return this.cloudServerRepository.listAll(1, Integer.MAX_VALUE, context.getTarget(), null, projectIds, null,
					null, containRemove);
		}
		return Lists.newArrayList();
	}

	@Override
	public GeneralResult count(Long userId, Long projectId, Long tenantId) {
		Catalog catalog = Catalog.Tenant;
		Long target = tenantId;
		if (null != userId) {
			target = userId;
			catalog = Catalog.User;
		}
		List<Project> projects = smsProjectService.projectsByCondition(target, catalog);
		List<Long> projectIds = new ArrayList<>();
		// 此时查询的是全部用户全部项目的信息
		if (null != projectId && 0L != projectId) {
			projectIds.add(projectId);
		} else if (!ListTool.isEmpty(projects)) {
			projects.forEach(project -> projectIds.add(project.getId()));
		} else {
			// 当前用户不属于任何项目，则登录之后查看不到任何云主机信息
			projectIds.add(-1L);
		}
		try {
			int total = this.cloudServerRepository.count(Lists.newArrayList(), null, null, projectIds, null);
			return new GeneralResult(true, total, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult(false, "查询失败");
		}
	}

	@Override
	public GeneralResult<List<CloudServer>> listTemplate(Long vendorId, Long vdcId) {
		try {
			List<CloudServer> servers = cloudServerRepository.listTemplate(vendorId, vdcId);
			return new GeneralResult(true, servers, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult(false, "查询失败");
		}
	}

	@Override
	public GeneralResult availableSizes(Long id) {
		return null;
	}

	@Override
	public GeneralResult details(Long id, RequestContext context, String action, String params) {
		if ("realtime".equalsIgnoreCase(action)) {
//            return vmExtendService.detailInRealTime(id);
		}
		return this.detail(id);
	}

	@Override
	public GeneralResult condition(Integer page, Integer rows, String condition, RequestContext context) {
		condition = Optional.ofNullable(condition).orElse("");
		JSONObject object = JSONObject.parseObject(condition);
		String action = object.getString("condition");
		// 判断是否存在
		if ("exists".equalsIgnoreCase(action)) {
			String name = object.getString("name");
			Long vendorId = object.getLong(Common.VENDORID);
			return this.exists(name, vendorId);
		} else if ("count".equalsIgnoreCase(action)) {
			// 汇总云主机数量
			Long tenantId = object.getLong("tenantId");
			return this.count(context.getTarget(), object.getLong("projectId"), tenantId);
		} else if ("serverFault".equalsIgnoreCase(action)) {
			return new GeneralResult(false, "接口异常");
		} else if ("listByCondition".equalsIgnoreCase(action)) {
			List<CloudServer> servers = cloudServerRepository.listByCondition(object.getLong("vendorId"),
					object.getLong("vdcId"), object.getLong("clusterId"));
			return new GeneralResult(true, servers, "查询成功");
		} else if ("listByVms".equalsIgnoreCase(action)) {
			List<CloudServer> servers = cloudServerRepository
					.listByVms(object.getJSONArray("serverIds").toJavaList(Long.class));
			return new GeneralResult(true, servers, "查询成功");
		} else if ("listTenantProject".equalsIgnoreCase(action)) {
			Boolean isAll = object.getBoolean("isAll");
			List<Long> tenantIds = JSON.parseArray(object.getString("tenantIds"), Long.class);
			List<Long> projectIds = JSON.parseArray(object.getString("projectIds"), Long.class);
			List<CloudServer> servers = cloudServerRepository.listVmsByTenants(tenantIds, isAll, projectIds);
			return new GeneralResult(true, servers, "查询成功");
		} else if ("basic".equalsIgnoreCase(action)) {
			return this.basic(object.getLong("serverId"));
		} else if ("AvailableVm".equalsIgnoreCase(action)) {
			return listAvailableVm(object.getLong("vendorId"), object.getString("regionId"), context);
		} else if ("hicUnBindEip".equalsIgnoreCase(action)) {// 华为云，未绑定弹性公网IP的云主机
			return hicUnBindEip(object, context);
		} else if ("getStatusStatistics".equalsIgnoreCase(action)) {
			// 获取云主机状态概览
			return this.getStatusStatistics(context);
		} else if ("getNumberTrend".equalsIgnoreCase(action)) {
			// 获取云主机数量趋势
			return this.getNumberTrend(context);
		} else if ("listBySlb".equalsIgnoreCase(action)) {
			return this.listBySlb(object, context);
		}
		return new GeneralResult<>(false, "参数格式错误");
	}

	private GeneralResult<ChargeResultTrend> getNumberTrend(RequestContext context) {
		List<String> timeList = timeToList(new JSONObject(MapTools.simpleMap("time", DateUtils.Time.MONTHS.getName())));
		List<String> keys = new ArrayList<>(timeList);
		List<ChargeTrendData> values = new ArrayList<>();
		ArrayList<Object> total = new ArrayList<>();
		try {
			List<CloudServer> cloudServerList = getServerList(context, true);
			for (String time : timeList) {
				// 筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
				List<CloudServer> targetVms = cloudServerList.stream()
						.filter(cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus())
								&& cloudServer.getGmtCreate()
								.after(DateUtils.stringToDate(time.substring(0, 10) + " 00:00:00",
										"yyyy-MM-dd HH:mm:ss"))
								&& cloudServer.getGmtCreate().before(DateUtils
								.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss")))
						.collect(Collectors.toList());
				total.add(targetVms.size());

			}
			ChargeTrendData chargeTrendData = new ChargeTrendData("数量", total);
			values.add(chargeTrendData);
			return new GeneralResult<>(true, new ChargeResultTrend(keys, values), "查询成功");
		} catch (Exception e) {
			log.error("get cloude server number trend error", e);
			return new GeneralResult<>(false, null, "查询失败");
		}
	}

	private GeneralResult hicUnBindEip(JSONObject object, RequestContext context) {
		try {
			Long vendorId = object.getLong(Common.VENDORID);
			String regionId = object.getString(Common.REGIONID);
			List<Long> vmIds = new ArrayList<>();
			if (context.getCatalog().equals(Catalog.Tenant) || context.getCatalog().equals(Catalog.User)) {
				List<Param> params = new ArrayList<Param>();
				params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
				params.add(new Param(MapTools.simpleMap("regionId", regionId), Sign.EQ));
				Pager pager = new Pager(1, 100, params, null);
				GeneralResult<GridBean<CloudServer>> vmListResult = this.list(pager, context);
				if (vmListResult.isFailed()) {
					return vmListResult;
				}
				List<CloudServer> list = vmListResult.getData().getRows();
				if (!ListTool.isEmpty(list)) {
					vmIds = list.stream().map(CloudServer::getId).collect(Collectors.toList());
				}
				vmIds.add(-1L);
			}
			List<CloudServer> servers = cloudServerRepository.listHicUnBindEip(vendorId, regionId, vmIds);
			return new GeneralResult(true, servers, "查询成功");
		} catch (Exception e) {
			log.error("查询实例列表异常：" + e.getMessage());
			return new GeneralResult<>(false, "查询实例列表异常：" + e.getMessage());
		}
	}

	private GeneralResult listBySlb(JSONObject object, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult listAvailableVm(Long vendorId, String regionId, RequestContext context) {
		try {
			List<CloudServer> listAvailableVm = new ArrayList<>();
			if (context.getCatalog().equals(Catalog.Tenant)) {
				// 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
				List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
				List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
				listAvailableVm = cloudServerRepository.listAvailableVm(vendorId, regionId, null, context.getTenant(),
						projectIds);
			} else if (context.getCatalog().equals(Catalog.User)) {
				// 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
				List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
				List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
				List<Long> projectIds = new ArrayList<>();
				List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!userManagerProjectsIds.isEmpty()) {
					projectIds.addAll(userManagerProjectsIds);
				}
				List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!visibleProjectsIds.isEmpty()) {
					projectIds.addAll(visibleProjectsIds);
				}
				listAvailableVm = cloudServerRepository.listAvailableVm(vendorId, regionId, context.getTarget(), null,
						projectIds);
			} else {
				listAvailableVm = cloudServerRepository.listAvailableVm(vendorId, regionId);
			}
			return new GeneralResult(true, listAvailableVm, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:{}", e);
			return new GeneralResult(false, "查询失败");
		}
	}

	@Override
	public GeneralResult getBusinessById(Long id) {
		return null;
	}

	@Override
	public GeneralResult configSeq(String params, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult generalize(Long id, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult restart(Long id, RequestContext context) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public GeneralResult stop(Long id, RequestContext context) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public GeneralResult hibernated(Long id, RequestContext context) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public GeneralResult resume(Long serverId, RequestContext context) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public GeneralResult chart(String params) {
		JSONObject object = JSONObject.parseObject(params);
		try {
			Assert.notNull(object, "参数格式错误");
			String catalog = StringUtils.isEmpty(object.getString("catalog")) ? "VENDOR" : object.getString("catalog");
			if ("VENDOR".equals(catalog)) {
				List<CloudVendor> vendors = cloudVendorRepository.listByType(null);
				List<String> keys = Lists.newArrayList();
				List<String> datas = Lists.newArrayList();
				for (CloudVendor vendor : vendors) {
					keys.add(vendor.getName());
					List<CloudServer> cloudServers = cloudServerRepository.listByVendorId(vendor.getId());
					datas.add(String.valueOf(cloudServers.size()));
				}
				TrendData trendData = new TrendData("云主机个数", datas);
				List<TrendData> values = Lists.newArrayList();
				values.add(trendData);
				ResultTrend resultTrend = new ResultTrend(keys, values);
				return new GeneralResult<>(true, resultTrend, "查询成功");
			} else if ("CLUSTER".equals(catalog)) {
				List<Cluster> clusters = clusterRepository.listByType(null);
				List<String> keys = Lists.newArrayList();
				List<String> datas = Lists.newArrayList();
				for (Cluster cluster : clusters) {
					keys.add(cluster.getName());
					int vmNum = cloudServerRepository.count(cluster.getId(), new ArrayList<>());
					datas.add(String.valueOf(vmNum));
				}
				TrendData trendData = new TrendData("云主机个数", datas);
				List<TrendData> values = Lists.newArrayList();
				values.add(trendData);
				ResultTrend resultTrend = new ResultTrend(keys, values);
				return new GeneralResult<>(true, resultTrend, "查询成功");
			}
			return new GeneralResult<>(true, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:", e);
			return new GeneralResult<>(false, "查询失败");
		}

	}

	@Override
	public GeneralResult selectAllByVendorId(Long vendorId, RequestContext context) {
		try {
			List<Map<String, Object>> res = cloudServerRepository.selectAllByVendorId(vendorId);
			return new GeneralResult<>(true, res, "云平台下云主机信息（个数、cpu、men、disk）统计成功");
		} catch (Exception e) {
			log.error("查询失败：" + e.getMessage());
			return new GeneralResult<>(false, "云平台下云主机信息（个数、cpu、men、disk）统计失败：" + e.getMessage());
		}
	}

}
