package com.bocloud.cmp.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.common.ButlerRepository;
import com.bocloud.cmp.common.LicenseSerivce;
import com.bocloud.cmp.domain.Butler;
import com.bocloud.cmp.domain.CloudVendorBean;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.entity.Dictionary;
import com.bocloud.cmp.entity.ServiceCatalog;
import com.bocloud.cmp.internal.CosInternalService;
import com.bocloud.cmp.internal.SmsDictionaryService;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.CloudVendorService;
import com.bocloud.cmp.service.CloudVendorTaskService;
import com.bocloud.cmp.service.SyncService;
import com.bocloud.cmp.util.ProviderRegister;
import com.gitee.starblues.core.PluginInfo;
import com.gitee.starblues.integration.operator.PluginOperator;
import com.gitee.starblues.integration.user.PluginUser;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.*;
import com.megatron.common.utils.*;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * CloudVendor Service实现类
 *
 * @author ymh
 */
@Slf4j
@Service("cloudVendorService")
public class CloudVendorServiceImpl implements CloudVendorService {

	@Value("${spring.upload.path:/home/cmp}")
	private String uploadPath;
	@Value("${plarform.sync.schedule:0 0 0 * * ?}")
	private String defaultExpression;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private PluginUser pluginUser;
	@Autowired
	private PluginOperator pluginOperator;
	@Autowired
	private ProviderRegister providerRegister;
	@Autowired
	private CloudVendorTaskService cloudVendorTaskService;
	@Autowired
	private CosInternalService cosInternalService;
	@Autowired
	private SmsDictionaryService smsDictionaryService;
	@Autowired
	private LicenseSerivce licenseSerivce;

	@Override
	public GeneralResult<CloudVendor> create(CloudVendor vendor, RequestContext context) {
		// #TODO 后续考虑异步
		try {
			// 证书节点校验
			GeneralResult licenseCheck = licenseSerivce.licenseCheck(null, 0, 0);
			if (licenseCheck.isFailed()) {
				return licenseCheck;
			}
			// 先进行同名认证，虽然输入时已验证，可能浏览器会记录信息
			CloudVendor existedVendor = cloudVendorRepository.getByName(vendor.getName());
			Assert.isNull(existedVendor, "添加失败，名称重复");
			String type = vendor.getType();
			if (StringUtils.isEmpty(type)) {
				return new GeneralResult<CloudVendor>(false, "缺少平台类型");
			}
			List<PluginInfo> pluginInfo = pluginOperator.getPluginInfo();
			List<String> pluginIds = pluginInfo.stream().map(PluginInfo::getPluginId).collect(Collectors.toList());
			if (ListTool.isEmpty(pluginIds) || !pluginIds.contains(type.toLowerCase())) {
				return new GeneralResult<CloudVendor>(false, "当前平台插件不存在");
			}

			CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser.getBean(type.toLowerCase(),
					"cloudVendorService");
			if (null == pluginCloudVendorService) {
				return new GeneralResult<CloudVendor>(false, "当前平台插件不存在");
			}
			GeneralResult<CloudVendor> authResult = pluginCloudVendorService.auth(vendor);
			if (authResult.isFailed()) {
				return authResult;
			}
			CloudVendor pluginCloudVendor = authResult.getData();
			if (null == pluginCloudVendor) {
				return new GeneralResult<>(false, "平台检验异常");
			}
			List<CloudVendor> list = cloudVendorRepository.listAll();
			list.add(pluginCloudVendor);
			this.providerRegister.updateProvider(list);

			// 新增云平台后需要等一会儿才会触发zookeeper的节点变化事件，然后ButlerRepository才会有新增的云平台数据
			Boolean flag = this.waitProviderCache(vendor.getUuid());
			Assert.isTrue(flag, "平台添加失败！");

			pluginCloudVendor
					.setSyncCron(StringUtils.isEmpty(vendor.getSyncCron()) ? defaultExpression : vendor.getSyncCron());
			cloudVendorRepository.save(vendor);
			// 开始同步平台资源数据

			SyncService pluginSyncService = (SyncService) pluginUser.getBean(type.toLowerCase(), "syncService");
			pluginSyncService.sync(vendor.getId(), context, false);
			// 开启定时同步
			cloudVendorTaskService.create(vendor.getId(), context);

			return new GeneralResult<CloudVendor>(true, "平台【" + vendor.getName() + "】添加成功");
		} catch (Exception e) {
			log.error("云平台添加异常：{}", e.getMessage(), e);
			return new GeneralResult<CloudVendor>(false, "云平台添加异常：" + e.getMessage());
		}

	}

	/**
	 * 等待云平台新数据从zookeeper缓存到本地
	 *
	 * @param vendor
	 * @return
	 */
	private boolean waitProviderCache(String vendor) {
		Boolean flag = true;
		Long startTime = System.currentTimeMillis();
		while (flag) {
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				log.error("Get error message:", e);
				Thread.currentThread().interrupt();
			}
			// 如果拉取到缓存数据，则说明已经添加成功
			Butler butler = ButlerRepository.getInstance().get(vendor);
			if (null != butler) {
				flag = false;
				return true;
			}
			Long endTime = System.currentTimeMillis();
			// 等待30秒
			if (endTime - startTime > 30 * 1000) {
				flag = false;
				break;
			}
		}
		return false;
	}

	@Transactional
	@Override
	public GeneralResult<CloudVendor> modify(CloudVendorBean bean, RequestContext context) {
		CloudVendor cloudVendor = cloudVendorRepository.query(bean.getId());
		if (null == cloudVendor) {
			return new GeneralResult(false, "云平台不存在");
		}
		CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
				.getBean(cloudVendor.getType().toLowerCase(), "cloudVendorService");
		if (null == pluginCloudVendorService) {
			return new GeneralResult(false, "当前平台插件不存在");
		}
		return pluginCloudVendorService.modify(bean, context);
	}

	@Transactional
	@Override
	public GeneralResult remove(Long id, RequestContext context) {
		CloudVendor cloudVendor = cloudVendorRepository.query(id);
		if (null == cloudVendor) {
			return new GeneralResult(false, "云平台不存在");
		}
		CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
				.getBean(cloudVendor.getType().toLowerCase(), "cloudVendorService");
		if (null == pluginCloudVendorService) {
			return new GeneralResult(false, "当前平台插件不存在");
		}
		return pluginCloudVendorService.remove(id, context);
	}

	@Override
	public GeneralResult lock(Long id, Long userId) {
		String path = CloudVendor.class.getSimpleName() + "_" + id;
		boolean result;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			CloudVendor vendor = cloudVendorRepository.query(id);
			Assert.notNull(vendor, "数据不存在");
			result = cloudVendorRepository.lock(id, userId);
		} catch (Exception e) {
			log.error("freeze cloudVendor fail:", e);
			return new GeneralResult<>(false, "冻结失败");
		}
		return result ? new GeneralResult<>(true, "冻结成功") : new GeneralResult<>(false, "冻结失败");
	}

	@Override
	public GeneralResult active(Long id, Long userId) {
		String path = CloudVendor.class.getSimpleName() + "_" + id;
		boolean result;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			CloudVendor vendor = cloudVendorRepository.query(id);
			Assert.notNull(vendor, "数据不存在");
			result = cloudVendorRepository.active(id, userId);
		} catch (Exception e) {
			log.error("freeze cloudVendor fail:", e);
			return new GeneralResult<>(false, "激活失败");
		}
		return result ? new GeneralResult<>(true, "激活成功") : new GeneralResult<>(false, "激活失败");
	}

	@Override
	public GeneralResult<CloudVendor> detail(Long id) {
		try {
			CloudVendor vendor = cloudVendorRepository.query(id);
			Assert.notNull(vendor, "云平台不存在");
			String catalogIds = vendor.getCatalogIds();
			if (StringUtils.isEmpty(catalogIds)) {
				vendor.setCatalogIds("[]");
			} else {
				vendor.setCatalogIds("[" + catalogIds + "]");
			}
			return new GeneralResult<>(true, vendor, "查询详情成功");
		} catch (Exception e) {
			log.error("query cloudVendor detail fail:", e);
			return new GeneralResult<>(false, "查询详情失败");
		}
	}

	@Override
	public GeneralResult checkName(String name) {
		try {
			CloudVendor vendor = cloudVendorRepository.getByName(name);
			return null == vendor ? new GeneralResult<>(true, "检查名称成功！") : new GeneralResult<>(false, "检查名称重复!");
		} catch (Exception e) {
			log.error("get cloudVendor by name fail", e);
			return new GeneralResult<>(false, "检查名称失败!");
		}
	}

	/**
	 * @since 2018-05-22 modify by sxp 支持hmc平台信息的查询
	 */
	@Override
	public GeneralResult<GridBean<CloudVendor>> list(Pager pager, RequestContext context) {
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
			sorter.put("priority", Common.ONE);
			sorter.put("gmtCreate", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			List<CloudVendor> list;
			List<SimpleBean> beans;
			int total;
			GridBean gridBean;
			total = cloudVendorRepository.count(params);
			if (simple) {
				beans = this.cloudVendorRepository.list(params, sorter);
				gridBean = new GridBean(1, 1, total, beans);
			} else {
				list = cloudVendorRepository.list(page, rows, params, sorter);
				gridBean = GridHelper.getBean(page, rows, total, list);
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	private GeneralResult<List<CloudVendor>> listAllVendors() {
		try {
			List<Param> params = new ArrayList<>();
			return new GeneralResult<>(true, cloudVendorRepository.list(params), "查询成功！");
		} catch (Exception e) {
			log.error("查询全部云平台异常：" + e.getMessage());
			return new GeneralResult<>(false, "查询全部云平台异常：" + e.getMessage());
		}
	}

	/**
	 * 查询项目所在业务组中的云平台列表
	 *
	 * @param types
	 * @param context
	 * @return
	 */
	public GeneralResult poolProviders(List<String> types, RequestContext context) {
		return null;
	}

	@Override
	public List<CloudVendor> list(String type) {
		try {
			List<Param> params = Lists.newArrayList();
			params.add(new Param(MapTools.simpleMap("type", type), Sign.EQ));
			return cloudVendorRepository.list(1, Integer.MAX_VALUE, params, null, null);
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return Lists.newArrayList();
		}
	}

	@Override
	public List<CloudVendor> listAll() {
		try {
			return cloudVendorRepository.listAll();
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return Lists.newArrayList();
		}
	}

	@Override
	public GeneralResult listAuthProvider(List<String> types, Long tenantId) {
		try {
			List<CloudVendor> vendors = cloudVendorRepository.listAuthProvider(types, tenantId);
			return new GeneralResult<>(true, vendors, "查询成功");
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	/**
	 * 查看vcenter平台全部集群的内存/cpu/存储的总使用量
	 * <p>
	 * 注：不在集群中的主机不统计
	 * </p>
	 *
	 * @param id 平台唯一标识
	 * @author sxp
	 * @since Apr 23, 2018
	 * @since 2018-04-25 modify by sxp 与ClusterServiceImpl.list统计数据逻辑保持一致
	 * @since 2018-04-26 modify by sxp 优先使用实时数据
	 * @since 2018-04-26 modify by sxp 删除使用实时数据的代码
	 * @since 2018-05-18 modify by sxp 磁盘容量求和之后四舍五入保留两位小数
	 */
	@Override
	public GeneralResult<CloudVendor> queryCapacity(Long id) {
		return null;
	}

	/**
	 * 根据平台类型查询所有平台
	 *
	 * @param type
	 * @return
	 */
	@Override
	public GeneralResult getByType(String type) {
		try {
			List<CloudVendor> cloudVendors = cloudVendorRepository.listByType(type);
			return new GeneralResult<>(true, cloudVendors, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult listByPool(String params) {
		try {
			JSONObject parseObject = JSONObject.parseObject(params, JSONObject.class);
			String category = parseObject.getString("category");
			if (null == category) {
				return new GeneralResult<>(false, "参数格式有误！");
			}
			Boolean isPublic = null;
			if (parseObject.containsKey("isPublic")) {
				isPublic = parseObject.getBoolean("isPublic");
			}
			List<CloudVendor> cloudVendors = cloudVendorRepository.listByPool(category, isPublic);
			return new GeneralResult<>(true, cloudVendors, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult batchOpenAlarm(List<Long> ids, Long userId) {
		if (null == ids) {
			return ResultTools.formatErrResult();
		}
		try {
			List<CloudVendor> succIds = Lists.newArrayList();
			int succResult = 0;
			int failResult = 0;
			for (Long id : ids) {
				CloudVendor cloudVendor = cloudVendorRepository.query(id);
				cloudVendor.setIsAlarm(true);
				cloudVendor.setGmtModify(new Date());
				cloudVendor.setMenderId(userId);
				boolean result = cloudVendorRepository.update(cloudVendor);
				if (result) {
					succIds.add(cloudVendor);
					succResult++;
				} else {
					failResult++;
				}
			}
			return new GeneralResult<>(true, succIds, "批量开启告警成功【" + succResult + "】个，失败【" + failResult + "】个");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "批量开启告警失败");
		}
	}

	@Override
	public GeneralResult batchCloseAlarm(List<Long> ids, Long userId) {
		if (ListTool.isEmpty(ids)) {
			return new GeneralResult<>(false, "参数格式错误");
		}
		try {
			List<CloudVendor> succIds = Lists.newArrayList();
			int succResult = 0;
			int failResult = 0;
			for (Long id : ids) {
				CloudVendor cloudVendor = cloudVendorRepository.query(id);
				cloudVendor.setIsAlarm(false);
				cloudVendor.setGmtModify(new Date());
				cloudVendor.setMenderId(userId);
				boolean result = cloudVendorRepository.update(cloudVendor);
				if (result) {
					succIds.add(cloudVendor);
					succResult++;
				} else {
					failResult++;
				}
			}
			return new GeneralResult<>(true, succIds, "批量关闭告警成功【" + succResult + "】个，失败【" + failResult + "】个");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "批量关闭告警失败");
		}
	}

	@Override
	public GeneralResult batchOperationAlarm(List<Long> ids, String action, Long userId) {
		if ("open".equalsIgnoreCase(action)) {
			return this.batchOpenAlarm(ids, userId);
		} else {
			return this.batchCloseAlarm(ids, userId);
		}
	}

	/**
	 * 根据平台类型查询
	 *
	 * @param types
	 * @return
	 */
	@Override
	public GeneralResult listByTypes(List<String> types) {
		try {
			List<CloudVendor> vendors = cloudVendorRepository.listByTypes(types, null);
			return new GeneralResult<>(true, vendors, "查询成功");
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	private GeneralResult listByTypes(JSONObject object) {
		try {
			List<String> vendorTypes = JSONArray.parseArray(object.getString("vendorTypes"), String.class);
			Long dcId = object.getLong("dcId");
			List<CloudVendor> vendors = cloudVendorRepository.listByTypes(vendorTypes, dcId);
			return new GeneralResult<>(true, vendors, "查询成功");
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	public GeneralResult listSimpleTypes(List<String> types) {
		try {
			List<CloudVendor> vendors = cloudVendorRepository.listSimpleTypes(types);
			return new GeneralResult<>(true, vendors, "查询成功");
		} catch (Exception e) {
			log.error("list cloudVendor fail:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	/**
	 * 云平台操作
	 *
	 * @param id
	 * @param action
	 * @param context
	 * @return
	 */
	@Override
	public GeneralResult action(Long id, String action, RequestContext context) {
		switch (action) {
		case "active":
			return active(id, context.getTarget());
		case "lock":
			return lock(id, context.getTarget());
		default:
			return new GeneralResult<>(false, "暂不支持");
		}
	}

	@Override
	public GeneralResult updatePassword(Long id, CloudVendor cloudVendor, RequestContext context) {
		CloudVendor vendor = cloudVendorRepository.query(id);
		if (null == vendor) {
			return new GeneralResult<>(false, "云平台不存在");
		}
		JSONObject oldAuthentication = null;
		JSONObject callbackAuthentication = null;
		String path = CloudVendor.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Assert.notNull(cloudVendor, "参数格式错误");
			Encryptor aesEncrptor = new AESEncryptor();
			Encryptor desEncrptor = new DESEncryptor();
			String oldPassword = cloudVendor.getOldPassword();
			String newPassword = cloudVendor.getNewPassword();
			oldAuthentication = JSONObject.parseObject(vendor.getAuthentication());
			callbackAuthentication = JSONObject.parseObject(vendor.getAuthentication());
			String updatePswdType = null;
			if (!StringUtils.isEmpty(oldPassword) && !StringUtils.isEmpty(newPassword)) {
				oldPassword = aesEncrptor.decrypt(oldPassword, null);
				newPassword = aesEncrptor.decrypt(newPassword, null);
				if (oldAuthentication.containsKey("password")) {
					String password = desEncrptor.decrypt(oldAuthentication.getString("password").trim(), "BocloudCMP");
					if (!password.equalsIgnoreCase(oldPassword)) {
						return new GeneralResult<>(false, "原密码错误");
					}
					oldAuthentication.put("password", cloudVendor.getNewPassword());
					updatePswdType = "password";
				}
			} else {
				if ("AZURE".equals(vendor.getType())) {
					String secretKey = aesEncrptor.decrypt(cloudVendor.getSecretKey().trim(), null);
					secretKey = desEncrptor.encrypt(secretKey, "BocloudCMP");
					oldAuthentication.put("clientId", cloudVendor.getClientId());
					oldAuthentication.put("secretKey", secretKey);
					updatePswdType = "azure";
				} else if ("KUBERNETES".equals(vendor.getType())) {
					oldAuthentication.put("token", cloudVendor.getToken().trim());
					updatePswdType = "k8s";
				} else if ("APSARASTACK".equals(vendor.getType())){
					oldAuthentication.put("accessKey", cloudVendor.getAccessKey().trim());
					oldAuthentication.put("secretKey", cloudVendor.getSecretKey().trim());

					if (StringUtils.isNotBlank(cloudVendor.getUserAccessKey())) {
						oldAuthentication.put("userAccessKey", cloudVendor.getAccessKey().trim());
					}

					if (StringUtils.isNotBlank(cloudVendor.getUserSecretKey())) {
						oldAuthentication.put("userAccessKey", cloudVendor.getUserSecretKey().trim());
					}

					if (StringUtils.isNotBlank(cloudVendor.getAaopAccessKey())) {
						oldAuthentication.put("aaopAccessKey", cloudVendor.getAaopAccessKey().trim());
					}

					if (StringUtils.isNotBlank(cloudVendor.getAaopSecretKey())) {
						oldAuthentication.put("aaopSecretKey", cloudVendor.getAaopSecretKey().trim());
					}

					updatePswdType = "aksk";
				} else {
					oldAuthentication.put("accessKey", cloudVendor.getAccessKey().trim());
					oldAuthentication.put("secretKey", cloudVendor.getSecretKey().trim());
					updatePswdType = "aksk";
				}
			}
			vendor.setAuthentication(oldAuthentication.toJSONString());
			vendor.setMenderId(context.getTarget());

			// 校验密码是否正确
			CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
					.getBean(vendor.getType().toLowerCase(), "cloudVendorService");
			if (null == pluginCloudVendorService) {
				return new GeneralResult<>(false, "当前平台插件不存在");
			}
			GeneralResult<CloudVendor> result = pluginCloudVendorService.auth(vendor);
			Assert.isTrue(result.isSuccess(), result.getMessage());
			// 设置新密码
			if (!StringUtils.isEmpty(cloudVendor.getNewPassword())) {
				String newPwd = aesEncrptor.decrypt(cloudVendor.getNewPassword(), null);
				oldAuthentication.put("password", desEncrptor.encrypt(newPwd, "BocloudCMP"));
			}
			if (!StringUtils.isEmpty(cloudVendor.getAccessKey()) && !StringUtils.isEmpty(cloudVendor.getSecretKey())) {
				String accessKey = aesEncrptor.decrypt(cloudVendor.getAccessKey().trim(), null);
				String secretKey = aesEncrptor.decrypt(cloudVendor.getSecretKey().trim(), null);
				oldAuthentication.put("accessKey", desEncrptor.encrypt(accessKey, "BocloudCMP"));
				oldAuthentication.put("secretKey", desEncrptor.encrypt(secretKey, "BocloudCMP"));
			}
			vendor.setAuthentication(oldAuthentication.toJSONString());
			// 如果认证成功则修改数据库
			cloudVendorRepository.update(vendor);
			// 在zk注册信息
			List<CloudVendor> list = cloudVendorRepository.listAll();
			this.providerRegister.updateProvider(list);
			boolean cacheStatus = waitUpdateProviderCache(vendor, updatePswdType);
			Assert.isTrue(cacheStatus, "缓存更新刷新失败！");
			return new GeneralResult<>(true, "密码更新成功");
		} catch (Exception e) {
			if (null != callbackAuthentication) {
				vendor.setAuthentication(callbackAuthentication.toJSONString());
			}
			vendor.setMenderId(context.getTarget());
			cloudVendorRepository.update(vendor);
			log.error("Get error message:{}", e);
			return new GeneralResult<>(false, e.getMessage());
		}
	}

	private boolean waitUpdateProviderCache(CloudVendor vendor, String type) throws Exception {
		boolean flag = false;
		Encryptor encryptor = new DESEncryptor();
		JSONObject authentication = JSONObject.parseObject(vendor.getAuthentication());
		Long startTime = System.currentTimeMillis();
		Long endTime = System.currentTimeMillis();
		while (endTime - startTime < 3 * 1000) {
			Thread.sleep(500);
			Butler butler = ButlerRepository.getInstance().get(vendor.getUuid());
			if (type.equalsIgnoreCase("password")) {
				String vendorPassword = authentication.getString("password");
				String butlerPassword = encryptor.encrypt(butler.getPassword(), "BocloudCMP");
				if (vendorPassword.equalsIgnoreCase(butlerPassword)) {
					flag = true;
					break;
				}
			} else if (type.equalsIgnoreCase("aksk")) {
				String vendorAccessKey = authentication.getString("accessKey");
				String vendorSecretKey = authentication.getString("secretKey");
				String butlerAccessKey = encryptor.encrypt(butler.getAccessKey(), "BocloudCMP");
				String butlerSecretKey = encryptor.encrypt(butler.getSecretKey(), "BocloudCMP");
				if (vendorAccessKey.equalsIgnoreCase(butlerAccessKey)
						&& vendorSecretKey.equalsIgnoreCase(butlerSecretKey)) {
					flag = true;
					break;
				}
			} else if (type.equalsIgnoreCase("azure")) {
				String vendorClientId = authentication.getString("clientId");
				String vendorSecretKey = authentication.getString("secretKey");
				String butlerClientId = butler.getClientId();
				String butlerSecretKey = encryptor.encrypt(butler.getSecretKey(), "BocloudCMP");
				if (vendorClientId.equalsIgnoreCase(butlerClientId)
						&& vendorSecretKey.equalsIgnoreCase(butlerSecretKey)) {
					flag = true;
					break;
				}
			} else if (type.equalsIgnoreCase("k8s")) {
				String token = authentication.getString("token");
				String bulterToken = butler.getToken();
				if (token.equalsIgnoreCase(bulterToken)) {
					flag = true;
					break;
				}
			}
			endTime = System.currentTimeMillis();
		}
		return flag;
	}

	@Override
	public GeneralResult condition(String condition, RequestContext context) {
		try {
			condition = Optional.ofNullable(condition).orElse("");
			JSONObject object = JSON.parseObject(condition);
			if ("checkName".equalsIgnoreCase(object.getString("condition"))) {
				return this.checkName(object.getString("name"));
			} else if ("poolProviders".equalsIgnoreCase(object.getString("condition"))) {
				List<String> types = JSONArray.parseArray(object.getString("types"), String.class);
				if (null == types) {
					return ResultTools.formatErrResult();
				}
				return this.poolProviders(types, context);
			} else if ("listByType".equalsIgnoreCase(object.getString("condition"))) {
				return this.getByType(object.getString("type"));
			} else if ("listByTypes".equalsIgnoreCase(object.getString("condition"))) {
				return listByTypes(JSON.parseArray(object.getString("types"), String.class));
			} else if ("listSimpleTypes".equalsIgnoreCase(object.getString("condition"))) {
				return listSimpleTypes(JSONArray.parseArray(object.getString("types"), String.class));
			} else if ("vendorTypes".equalsIgnoreCase(object.getString("condition"))) {
				return this.listByTypes(object);
			} else if ("listByPool".equalsIgnoreCase(object.getString("condition"))) {
				return this.listByPool(condition);
			} else if ("listAllVendors".equalsIgnoreCase(object.getString("condition"))) {
				return this.listAllVendors();
			} else if ("listType".equals(object.getString("condition"))) {
				String category = object.getString("resourceCategory");
				Set<String> types;
				if (!StringUtils.isEmpty(category)) {
					types = cloudVendorRepository.listAll().stream()
							.filter(cloudVendor -> category.equals(cloudVendor.getVendorCategory()))
							.map(CloudVendor::getType).collect(Collectors.toSet());
				} else {
					types = cloudVendorRepository.listAll().stream().map(CloudVendor::getType)
							.collect(Collectors.toSet());
				}
				return new GeneralResult<>(true, types, "查询成功");
			} else if ("listTypeByCatalogId".equals(object.getString("condition"))) {
				return this.listTypeByCatalogId(object.getLong("catalogId"));
			}
		} catch (Exception e) {
			log.error("查询失败：{}", e.getMessage());
		}
		return new GeneralResult<>(false, "参数格式错误");
	}

	private GeneralResult listTypeByCatalogId(Long catalogId) {
		try {
			ServiceCatalog catalog = cosInternalService.getServiceCatalog(catalogId);
			Map<String, Boolean> vendorMap = cloudVendorRepository.listAll().stream()
					.collect(Collectors.toMap(CloudVendor::getType, CloudVendor::getIsPublic, (m1, m2) -> m2));
			GeneralResult<List<Dictionary>> typesData = smsDictionaryService.dictionaries("SERVICE_CATALOG");
			List<Dictionary> serviceDics = typesData.getData();
			List<Dictionary> types = new ArrayList<>();
			for (Dictionary dictionary : serviceDics) {
				if (dictionary.getValue().equalsIgnoreCase(catalog.getCode())) {
					types = dictionary.getChildren();
				}
			}
			Map<String, Dictionary> vendorTypeMap = types.stream()
					.collect(Collectors.toMap(Dictionary::getValue, Function.identity(), (m1, m2) -> m1));
			JSONArray result = new JSONArray();
			for (Map.Entry<String, Boolean> entry : vendorMap.entrySet()) {
				String type = entry.getKey();
				if (vendorTypeMap.containsKey(type)) {
					JSONObject jsonObject = new JSONObject();
					jsonObject.put("vendorType", type);
					jsonObject.put("name", vendorTypeMap.get(type).getName());
					jsonObject.put("public", entry.getValue());
					result.add(jsonObject);
				}
			}
			return new GeneralResult<>(true, result, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}

	}

	@Override
	public GeneralResult clearSession(Long id, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult updateExcessRatio(Long id, CloudVendor cloudVendor, RequestContext context) {
		String path = CloudVendor.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Assert.notNull(cloudVendor, "参数格式错误");
			CloudVendor vendor = cloudVendorRepository.query(id);
			Assert.notNull(vendor, "云平台不存在");
			JSONObject authentication = JSONObject.parseObject(vendor.getAuthentication());
			authentication.put("excessRatio", cloudVendor.getExcessRatio());
			vendor.setAuthentication(authentication.toJSONString());
			vendor.setMenderId(context.getTarget());
			cloudVendorRepository.update(vendor);
			return new GeneralResult<>(true, "超分比更新成功");
		} catch (Exception e) {
			log.error("超分比更新失败:{}", e.getMessage());
			return new GeneralResult<>(false, "超分比更新失败：" + e.getMessage());
		}
	}

	@Override
	public GeneralResult summary(Long id, RequestContext context) {
		CloudVendor vendor = cloudVendorRepository.query(id);
		if (null == vendor) {
			return new GeneralResult<>(false, "云平台不存在");
		}
		CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
				.getBean(vendor.getType().toLowerCase(), "cloudVendorService");
		return pluginCloudVendorService.summary(id, context);
	}

	@Override
	public GeneralResult getQuoteInfo(Long vendorId) {
		CloudVendor cloudVendor = cloudVendorRepository.query(vendorId);
		if (null == cloudVendor) {
			return new GeneralResult(false, "云平台不存在");
		}
		CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
				.getBean(cloudVendor.getType().toLowerCase(), "cloudVendorService");
		return pluginCloudVendorService.getQuoteInfo(vendorId);
	}

	@Override
	public GeneralResult auth(CloudVendor vendor) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public GeneralResult snapshotConfig(Long vendorId, CloudVendorBean bean, RequestContext context) {
		CloudVendor cloudVendor = cloudVendorRepository.query(vendorId);
		if (null == cloudVendor) {
			return new GeneralResult(false, "云平台不存在");
		}
		CloudVendorService pluginCloudVendorService = (CloudVendorService) pluginUser
				.getBean(cloudVendor.getType().toLowerCase(), "cloudVendorService");
		return pluginCloudVendorService.snapshotConfig(vendorId, bean, context);
	}
}
