package com.bocloud.cmp.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.config.TriggerTask;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.config.DefaultSchedulingConfigurer;
import com.bocloud.cmp.constant.ScheduledTasksConstant;
import com.bocloud.cmp.domain.CloudVendorBean;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.internal.CmpInternalService;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.CloudVendorTaskService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;

/**
 * @author 作者
 * @version 创建时间：2019年4月15日 下午6:04:20 类说明 定时规则修改
 */
@Service
@Slf4j
public class CloudVendorTaskServiceImpl implements CloudVendorTaskService {

	@Autowired
	private DefaultSchedulingConfigurer defaultSchedulingConfigurer;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
    private PluginUser pluginUser;
	@Autowired
	private CmpInternalService cmpInternalService;

	@Override
	public GeneralResult create(Long vendorId, RequestContext context) {
		try {
			CloudVendor vendor = cloudVendorRepository.query(vendorId);
			Assert.notNull(vendor, "云平台不存在");
			// 等待任务调度初始化完成
			while (!defaultSchedulingConfigurer.inited()) {
				Thread.sleep(5000);
			}
			String taskId = ScheduledTasksConstant.CLOUD_VENDOR + vendor.getUuid();
			defaultSchedulingConfigurer.addTriggerTask(taskId,
					new TriggerTask(new CloudVendorTaskThread(vendor.getId()), new CronTrigger(vendor.getSyncCron())),
					vendor.getSyncCron());
			log.info("平台-->{}定时同步任务加入成功！定时策略 ：{} ", vendor.getName(), vendor.getSyncCron());
			return new GeneralResult<>(true, "平台定时任务添加成功！");
		} catch (Exception e) {
			log.error("添加平台定时任务失败 ： {}", e.getMessage());
			return new GeneralResult<>(false, "添加平台定时任务失败 ： {}", e.getMessage());
		}
	}

	@Override
	public GeneralResult remove(Long vendorId, RequestContext context) {
		try {
			log.info("vendorId :" + vendorId);
			CloudVendor vendor = cloudVendorRepository.query(vendorId);
			Assert.notNull(vendor, "云平台不存在");
			// 等待任务调度初始化完成
			while (!defaultSchedulingConfigurer.inited()) {
				Thread.sleep(5000);
			}
			String taskId = ScheduledTasksConstant.CLOUD_VENDOR + vendor.getUuid();
			defaultSchedulingConfigurer.cancelTriggerTask(taskId);
			log.info(vendor.getName() + "云平台取消定时同步任务成功！");
			return new GeneralResult<>(true, "平台定时任务取消成功！");
		} catch (Exception e) {
			log.error("取消平台定时任务失败 ： " + e.getMessage());
			return new GeneralResult<>(false, "取消平台定时任务失败 ： " + e.getMessage());
		}
	}

	@Override
	public GeneralResult modify(CloudVendorBean cloudVendor, RequestContext context) {
		try {
			log.info("cloudVendor :" + JSONObject.toJSONString(cloudVendor));
			Assert.notNull(cloudVendor, "参数格式错误");
			CloudVendor vendor = cloudVendorRepository.query(cloudVendor.getId());
			Assert.notNull(vendor, "云平台不存在");
			// 等待任务调度初始化完成
			while (!defaultSchedulingConfigurer.inited()) {
				Thread.sleep(5000);
			}
			String taskId = ScheduledTasksConstant.CLOUD_VENDOR + vendor.getUuid();
			defaultSchedulingConfigurer.resetTriggerTask(taskId,
					new TriggerTask(new CloudVendorTaskThread(vendor.getId()),
							new CronTrigger(cloudVendor.getSyncCron())),
					cloudVendor.getSyncCron());
			vendor.setSyncCron(cloudVendor.getSyncCron());
			cloudVendorRepository.update(vendor);
			log.info(vendor.getName() + " --> 平台定时同步任务【重置】成功！定时策略 ： " + cloudVendor.getSyncCron());
			return new GeneralResult<>(true, "平台定时任务重置成功！");
		} catch (Exception e) {
			log.error("重置平台定时任务失败 ： " + e.getMessage());
			return new GeneralResult<>(false, "重置平台定时任务失败 ： " + e.getMessage());
		}
	}

	@Data
	class CloudVendorTaskThread implements Runnable {

		private Long vendorId;

		public CloudVendorTaskThread(Long vendorId) {
			super();
			this.vendorId = vendorId;
		}

		@Override
		public void run() {
			try {
				CloudVendor vendor = cloudVendorRepository.query(vendorId);
				Assert.notNull(vendor, "云平台不存在");
				cmpInternalService.vendorSync(vendor.getId());
//				SyncService pluginSyncService = (SyncService) pluginUser.getBean(vendor.getType().toLowerCase(), "syncService");
//				pluginSyncService.sync(vendorId, new RequestContext(RequestContext.Catalog.Manager, 1L, 0L, 0L, null), true);
			} catch (Exception e) {
				log.error("云平台同步异常：" + e.getMessage());
			}
		}

	}

}
