package com.bocloud.cmp.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudServer;
import com.bocloud.cmp.entity.DataStore;
import com.bocloud.cmp.entity.Server;
import com.bocloud.cmp.model.DataStoreBean;
import com.bocloud.cmp.repository.CloudServerRepository;
import com.bocloud.cmp.repository.DataStoreRepository;
import com.bocloud.cmp.repository.ServerRepository;
import com.bocloud.cmp.repository.VolumeRepository;
import com.bocloud.cmp.service.DataStoreService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.google.common.collect.Maps;

import lombok.extern.slf4j.Slf4j;

@Service("dataStoreService")
@Slf4j
public class DataStoreServiceImpl implements DataStoreService {

	@Autowired
	private DataStoreRepository dataStoreRepository;
	@Autowired
	private CloudServerRepository cloudServerRepository;
	@Autowired
	private ServerRepository serverRepository;
	@Autowired
	private VolumeRepository volumeRepository;

	@Override
	public GeneralResult<GridBean<DataStore>> list(Pager pager, RequestContext context) {
		try {
			GridBean gridBean = null;
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
			sorter.put("gmtCreate", Common.ONE);
            sorter.put("id", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			int total = 0;
			if (simple) {
				total = this.dataStoreRepository.simpleCount(params);
				List<SimpleBean> beans = this.dataStoreRepository.list(params, sorter);
				gridBean = new GridBean(1, 1, total, beans);
			} else {
				Long vhostId = null;
				String category = null;
				// 集群、数据中心模糊查询
				String clusterName = "";
				String vdcName = "";
				for (Param param : params) {
					Map<String, Object> paramMap = param.getParam();
					if (paramMap.containsKey("vhostId")) {
						vhostId = Long.valueOf(paramMap.get("vhostId").toString());
						paramMap.remove("vhostId");
					}
					if (paramMap.containsKey("category")) {
						category = paramMap.get("category").toString();
						paramMap.remove("category");
					}
					if (paramMap.containsKey("clusterName")) {
						clusterName = paramMap.get("clusterName").toString();
						paramMap.remove("clusterName");
					}
					if (paramMap.containsKey("vdcName")) {
						vdcName = paramMap.get("vdcName").toString();
						paramMap.remove("vdcName");
					}
				}
				total = this.dataStoreRepository.count(params, vhostId, category, clusterName, vdcName);
				List<DataStore> list = this.dataStoreRepository.list(page, rows, params, sorter, vhostId, category,
						clusterName, vdcName);
				Long vendorId = null;
				for (Param param : params) {
					if (param.getSign() == Sign.EQ) {
						for (String key : param.getParam().keySet()) {
							if ("vendorId".equals(key) && !StringUtils.isEmpty(param.getParam().get(key).toString())) {
								vendorId = Long.valueOf(param.getParam().get(key).toString());
								break;
							}
						}
					}
				}
				List<CloudServer> cloudServers = cloudServerRepository.listByVid(vendorId);
				if (null != cloudServers && null != list) {
					for (DataStore dataStore : list) {
						int dataStoreVmNums = dataStore.getVmNum() == null ? 0 : dataStore.getVmNum();
						if (cloudServers.size() - dataStoreVmNums > 0) {
							dataStore.setVmNumUm((cloudServers.size()) - (dataStore.getVmNum()));
						}
					}
				}
				for (DataStore store : list) {
					calDistributable(store);
				}
				gridBean = GridHelper.getBean(page, rows, total, list);
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Query dataStore list fail:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	/**
	 * 计算存储可分配
	 *
	 * @param store
	 */
	private void calDistributable(DataStore store) throws Exception {
		BigDecimal distributableRatio = store.getDistributableRatio();
		if (null == distributableRatio) {
			store.setDistributable(0);
			return;
		}
		BigDecimal distributable = BigDecimal.valueOf(store.getCapacity()).multiply(distributableRatio)
				.divide(BigDecimal.valueOf(100));
		int disk = volumeRepository.diskSum(store.getId());
		if (disk < distributable.intValue()) {
			store.setDistributable(distributable.intValue() - disk);
		} else {
			store.setDistributable(0);
		}
	}

	@Override
	public GeneralResult<DataStore> create(DataStore dataStore, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult<DataStore> modify(DataStoreBean bean, Long userId) {
		return null;
	}

	@Override
	public GeneralResult remove(Long id, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult<DataStore> detail(Long id) {
		try {
			DataStore dataStore = this.dataStoreRepository.query(id);
			Assert.notNull(dataStore, "数据不存在");
			return new GeneralResult<>(true, dataStore, "查询详情成功");
		} catch (Exception e) {
			log.error("Get dataStore error：", e);
			return new GeneralResult<>(false, "查询详情失败");
		}
	}

	@Override
	public GeneralResult servers(Long id) {
		return null;
	}

	@Override
	public GeneralResult<GridBean<CloudServer>> servers(Pager pager, Long id) {
		return null;
	}

	@Override
	public GeneralResult<List<Server>> hosts(Long id) {
		return null;
	}

	@Override
	public GeneralResult<GridBean<Server>> hosts(Pager pager, Long id) {
		return null;
	}

	@Override
	public GeneralResult uploadFile(Long id, String filePath, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult unmount(Long id, Long hostId, Long userId) {
		return null;
	}

	@Override
	public GeneralResult listByHost(Long hostId) {
		try {
			List<DataStore> stores = dataStoreRepository.listByHost(hostId);
			return new GeneralResult<>(true, stores, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult listByStore(Long storeId) {
		try {
			List<Server> servers = serverRepository.listByStore(storeId);
			return new GeneralResult<>(true, servers, "查询成功");
		} catch (Exception e) {
			log.error("Get error message:", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult batchModifyLabel(String params, RequestContext context) {
		JSONObject object = JSONObject.parseObject(params);
		Assert.notNull(object, "参数格式错误");
		List<Long> storeIds = JSONArray.parseArray(object.getString("storeIds"), Long.class);
		if (ListTool.isEmpty(storeIds)) {
			return new GeneralResult<>(false, "请选择数据存储");
		}
		try {
			for (Long storeId : storeIds) {
				DataStore store = dataStoreRepository.query(storeId);
				if (null != store) {
					store.setResourceLabel(object.getString("resourceLabel"));
					store.setGmtModify(new Date());
					store.setMenderId(context.getTarget());
					dataStoreRepository.update(store);
				}
			}
			return new GeneralResult<>(true, "标签配置成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "标签配置失败");
		}
	}

	@Override
	public GeneralResult condition(String condition, RequestContext context) {
		condition = Optional.ofNullable(condition).orElse("");
		JSONObject object = JSONObject.parseObject(condition);
		if ("listByHost".equalsIgnoreCase(object.getString("condition"))) {
			return this.listByHost(object.getLong("hostId"));
		} else if ("listByStore".equalsIgnoreCase(object.getString("condition"))) {
			return this.listByStore(object.getLong("storeId"));
		}
		return new GeneralResult<>(false, "参数格式错误");
	}

}
