package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.Business;
import com.bocloud.cmp.entity.ElasticIp;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.internal.SmsBusinessService;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.repository.ElasticIpRepository;
import com.bocloud.cmp.service.ElasticIpService;
import com.bocloud.cmp.service.LabelService;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

@Service("elasticIpService")
@Slf4j
public class ElasticIpServiceImpl implements ElasticIpService {

	@Autowired
	private ElasticIpRepository elasticIpRepository;
	@Autowired
	private SmsProjectService smsProjectService;
	@Autowired
	private SmsUserService smsUserService;
	@Autowired
	private SmsBusinessService smsBusinessService;
	@Autowired
	private SmsTenantService smsTenantService;
	@Autowired
	private LabelService labelService;

	@Override
	public GeneralResult<GridBean<ElasticIp>> list(Pager pager, RequestContext context) {
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			sorter.put("gmtCreate", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);

			String vendorType = null;
			String labelContent = null;
			for (Param param : params) {
				Map<String, Object> paramMap = param.getParam();
				if (paramMap.containsKey("vendorType")) {
					vendorType = String.valueOf(paramMap.get("vendorType"));
					paramMap.remove("vendorType");
				}
				if (paramMap.containsKey("labelContent")) {
					labelContent = paramMap.get("labelContent").toString();
					paramMap.remove("labelContent");
				}
			}
			int total = 0;
			List<ElasticIp> list = new ArrayList<>();
			if (context.getCatalog().equals(Catalog.Tenant)) {
				// 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
				List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
				List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
				total = this.elasticIpRepository.count(null, context.getTenant(), projectIds, params, vendorType,
						labelContent);
				list = this.elasticIpRepository.list(page, rows, null, context.getTenant(), projectIds, params, sorter,
						vendorType, labelContent);
				list = convertor(list);
			} else if (context.getCatalog().equals(Catalog.User)) {
				// 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
				List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
				List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
				List<Long> projectIds = new ArrayList<>();
				List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!userManagerProjectsIds.isEmpty()) {
					projectIds.addAll(userManagerProjectsIds);
				}
				List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!visibleProjectsIds.isEmpty()) {
					projectIds.addAll(visibleProjectsIds);
				}
				total = this.elasticIpRepository.count(context.getTarget(), null, projectIds, params, vendorType,
						labelContent);
				list = this.elasticIpRepository.list(page, rows, context.getTarget(), null, projectIds, params, sorter,
						vendorType, labelContent);
				list = convertor(list);
			} else {
				total = elasticIpRepository.count(params, labelContent);
				list = elasticIpRepository.list(page, rows, params, sorter, labelContent);
			}
			GridBean<ElasticIp> gridBean = GridHelper.getBean(page, rows, total, list);
			gridBean.setRows(list);
			return new GeneralResult<GridBean<ElasticIp>>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list elastic ip failure:", e);
			return new GeneralResult<GridBean<ElasticIp>>(false, "查询失败", null, null);
		}
	}

	public List<ElasticIp> convertor(List<ElasticIp> list) {
		try {
			if (!ListTool.isEmpty(list)) {
				List<Long> tenantIds = new ArrayList<>();
				List<Long> userIds = new ArrayList<>();
				List<Long> businessIds = new ArrayList<>();
				List<Long> projectIds = new ArrayList<>();
				for (ElasticIp clasticIp : list) {
					if (clasticIp.getTenantId() != null) {
						tenantIds.add(clasticIp.getTenantId());
					}
					if (clasticIp.getBusinessId() != null) {
						businessIds.add(clasticIp.getBusinessId());
					}
					if (clasticIp.getProjectId() != null) {
						projectIds.add(clasticIp.getProjectId());
					}
					if (clasticIp.getCreatorId() != null) {
						userIds.add(clasticIp.getCreatorId());
					}
					if (clasticIp.getOwnerId() != null) {
						userIds.add(clasticIp.getOwnerId());
					}
					if (clasticIp.getMenderId() != null) {
						userIds.add(clasticIp.getMenderId());
					}
				}
				Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
						.collect(Collectors.toMap(User::getId, User::getName));
				Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
						.collect(Collectors.toMap(Project::getId, Project::getName));
				Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
						.collect(Collectors.toMap(Business::getId, Business::getName));
				Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
						.collect(Collectors.toMap(Tenant::getId, Tenant::getName));
				for (ElasticIp clasticIp : list) {
					// 设置云主机归属信息
					if (clasticIp.getTenantId() != null) {
						clasticIp.setTenantName(tenantMap.get(clasticIp.getTenantId()));
					}
					if (clasticIp.getBusinessId() != null) {
						clasticIp.setBusinessName(businessMap.get(clasticIp.getBusinessId()));
					}
					if (clasticIp.getProjectId() != null) {
						clasticIp.setProjectName(projectMap.get(clasticIp.getProjectId()));
					}
					if (clasticIp.getCreatorId() != null) {
						clasticIp.setCreatorName(userMap.get(clasticIp.getCreatorId()));
					}
					if (clasticIp.getOwnerId() != null) {
						clasticIp.setOwnerName(userMap.get(clasticIp.getOwnerId()));
					}
					if (clasticIp.getMenderId() != null) {
						clasticIp.setMenderName(userMap.get(clasticIp.getMenderId()));
					}
				}
			}
		} catch (Exception e) {
			log.error("获取浮动IP基本信息异常：" + e.getMessage());
		}
		return list;
	}

	@Override
	public GeneralResult<ElasticIp> detail(Long id) {
		try {
			ElasticIp elasticIp = elasticIpRepository.query(id);
			if (null == elasticIp) {
				return new GeneralResult<>(false, "资源不存在");
			}
			// 设置云主机归属
			if (null != elasticIp.getOwnerId() && 0L != elasticIp.getOwnerId()) {
				GeneralResult<User> ownerDetail = smsUserService.detail(elasticIp.getOwnerId());
				User owner = ownerDetail.getData();
				if (owner != null) {
					elasticIp.setOwnerName(owner.getName());
				}
			}
			if (null != elasticIp.getMenderId() && 0L != elasticIp.getMenderId()) {
				GeneralResult<User> menderDetail = smsUserService.detail(elasticIp.getMenderId());
				User mender = menderDetail.getData();
				if (mender != null) {
					elasticIp.setMenderName(mender.getName());
				}
			}
			if (null != elasticIp.getCreatorId() && 0L != elasticIp.getCreatorId()) {
				GeneralResult<User> creatorDetail = smsUserService.detail(elasticIp.getCreatorId());
				User creator = creatorDetail.getData();
				if (creator != null) {
					elasticIp.setCreatorName(creator.getName());
				}
			}
			if (null != elasticIp.getProjectId() && 0L != elasticIp.getProjectId()) {
				GeneralResult<Project> projectResult = smsProjectService.detail(elasticIp.getProjectId());
				if (projectResult.isFailed() || null == projectResult.getData()) {
					return new GeneralResult(false, projectResult.getMessage());
				}
				Project project = projectResult.getData();
				if (null != project) {
					elasticIp.setProjectName(project.getName());
				}
			}
			if (null != elasticIp.getBusinessId() && 0L != elasticIp.getBusinessId()) {
				GeneralResult<Business> businessDetail = smsBusinessService.detail(elasticIp.getBusinessId());
				Business business = businessDetail.getData();
				if (business != null) {
					elasticIp.setBusinessName(business.getName());
				}
			}
			if (null != elasticIp.getTenantId() && 0L != elasticIp.getTenantId()) {
				GeneralResult<Tenant> tenantDetail = smsTenantService.detail(elasticIp.getTenantId());
				Tenant tenant = tenantDetail.getData();
				if (tenant != null) {
					elasticIp.setTenantName(tenant.getName());
				}
			}
			// 浮动ip标签
			List<JSONObject> parseArray = JSON.parseArray(elasticIp.getResourceLabel(), JSONObject.class);
			if (!ListTool.isEmpty(parseArray)) {
				GeneralResult existsValues = labelService.isExistsValues(parseArray);
				if (existsValues.isFailed()) {
					return existsValues;
				}
				elasticIp.setResourceLabel(JSONObject.toJSONString(existsValues.getData()));
				elasticIpRepository.update(elasticIp);
			}
			return new GeneralResult<ElasticIp>(true, elasticIp, "查询成功");
		} catch (Exception e) {
			log.error("detail elastic ip failure:", e);
			return new GeneralResult<ElasticIp>(false, "查询失败", null, null);
		}
	}

	@Override
	public GeneralResult<List<ElasticIp>> listFrees(Long vendorId, String regionId, RequestContext context) {
		try {
			List<ElasticIp> list = new ArrayList<>();
			if (context.getCatalog().equals(Catalog.Tenant)) {
				// 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
				List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
				List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
				list = elasticIpRepository.listEips(null, context.getTenant(), projectIds, regionId, vendorId);
			} else if (context.getCatalog().equals(Catalog.User)) {
				// 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
				List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
				List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
				List<Long> projectIds = new ArrayList<>();
				List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!userManagerProjectsIds.isEmpty()) {
					projectIds.addAll(userManagerProjectsIds);
				}
				List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!visibleProjectsIds.isEmpty()) {
					projectIds.addAll(visibleProjectsIds);
				}
				list = elasticIpRepository.listEips(context.getTarget(), null, projectIds, regionId, vendorId);
			} else {
				list = elasticIpRepository.listByVidFrees(regionId, vendorId);
			}

			return new GeneralResult<List<ElasticIp>>(true, list, "查询成功!");
		} catch (Exception e) {
			log.error("list elasticIp failure:", e);
			return new GeneralResult<List<ElasticIp>>(false, "查询失败!");
		}
	}

}
