package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.Business;
import com.bocloud.cmp.entity.FloatingIp;
import com.bocloud.cmp.entity.OSTenant;
import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.internal.SmsBusinessService;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.repository.FloatingIpRepository;
import com.bocloud.cmp.repository.OSTenantRepository;
import com.bocloud.cmp.service.FloatingIpService;
import com.bocloud.cmp.service.LabelService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 浮动IP Service实现类
 *
 * @author weiwei
 */
@Service("floatingIpService")
@Slf4j
public class FloatingIpServiceImpl implements FloatingIpService {

	@Autowired
	private FloatingIpRepository floatingIpRepository;
	@Autowired
	private SmsProjectService smsProjectService;
	@Autowired
	private SmsUserService smsUserService;
	@Autowired
	private OSTenantRepository osTenantRepository;
	@Autowired
	private SmsBusinessService smsBusinessService;
	@Autowired
	private SmsTenantService smsTenantService;
	@Autowired
	private LabelService labelService;

	@Override
	public GeneralResult<GridBean<FloatingIp>> list(Pager pager, RequestContext context) {
		try {
			int page = pager.getPage();
			int rows = pager.getRows();
			List<Param> params = pager.getParams();
			Map<String, String> sorter = pager.getSorter();
			boolean simple = pager.getSimple();
			List<FloatingIp> list = Lists.newArrayList();
			GridBean gridBean = null;
			int total = 0;
			if (null == sorter) {
				sorter = Maps.newHashMap();
			}
			sorter.put("gmtCreate", Common.ONE);
			String vendorType = null;
			String labelContent = null;
			for (Param param : params) {
				Map<String, Object> paramMap = param.getParam();
				if (paramMap.containsKey("vendorType")) {
					vendorType = String.valueOf(paramMap.get("vendorType"));
					paramMap.remove("vendorType");
				}
				if (paramMap.containsKey("labelContent")) {
					labelContent = paramMap.get("labelContent").toString();
					paramMap.remove("labelContent");
				}
			}
			if (context.getCatalog().equals(Catalog.Tenant)) {
				// 租户能看到 1.属于当前租户的资源 2.租户下所有项目的资源
				List<Project> tenantProjects = smsProjectService.listTenantProjects(context.getTenant());
				List<Long> projectIds = tenantProjects.stream().map(Project::getId).collect(Collectors.toList());
				total = floatingIpRepository.count(null, context.getTenant(), projectIds, params, vendorType,
						labelContent);
				list = floatingIpRepository.list(page, rows, null, context.getTenant(), projectIds, params, sorter,
						vendorType, labelContent);
				list = convertor(list);
				gridBean = new GridBean<>(page, rows, total, list);
			} else if (context.getCatalog().equals(Catalog.User)) {
				// 用户能看到 1.自己申请的资源 2.自己所在的资源可见项目（资源对项目下所有成员可见）下创建的资源 3.自己管理的项目下的资源
				List<Project> userManagerProjects = smsProjectService.listUserManagerProjects(context.getTarget());
				List<Project> visibleProjects = smsProjectService.listUserVisibleProjects(context.getTarget());
				List<Long> projectIds = new ArrayList<>();
				List<Long> userManagerProjectsIds = userManagerProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!userManagerProjectsIds.isEmpty()) {
					projectIds.addAll(userManagerProjectsIds);
				}
				List<Long> visibleProjectsIds = visibleProjects.stream().map(Project::getId)
						.collect(Collectors.toList());
				if (!visibleProjectsIds.isEmpty()) {
					projectIds.addAll(visibleProjectsIds);
				}
				total = floatingIpRepository.count(context.getTarget(), null, projectIds, params, vendorType,
						labelContent);
				list = floatingIpRepository.list(page, rows, context.getTarget(), null, projectIds, params, sorter,
						vendorType, labelContent);
				list = convertor(list);
				gridBean = new GridBean<>(page, rows, total, list);
			} else {
				total = floatingIpRepository.count(params, labelContent);
				if (simple) {
					if (0 == context.getTenant()) {
						params.add(new Param(MapTools.simpleMap(Common.TENANTID, 0), Sign.EQ));
					}
					List<SimpleBean> beans = floatingIpRepository.list(params, sorter, labelContent);
					gridBean = new GridBean(1, 1, total, beans);
				} else {
					list = floatingIpRepository.list(page, rows, params, sorter, labelContent);
					gridBean = GridHelper.getBean(page, rows, total, list);
				}
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list floating ip fail:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}

	private List<FloatingIp> convertor(List<FloatingIp> list) {

		try {
			if (!ListTool.isEmpty(list)) {
				List<Long> tenantIds = new ArrayList<>();
				List<Long> userIds = new ArrayList<>();
				List<Long> businessIds = new ArrayList<>();
				List<Long> projectIds = new ArrayList<>();
				for (FloatingIp floatingIp : list) {
					if (floatingIp.getTenantId() != null) {
						tenantIds.add(floatingIp.getTenantId());
					}
					if (floatingIp.getBusinessId() != null) {
						businessIds.add(floatingIp.getBusinessId());
					}
					if (floatingIp.getProjectId() != null) {
						projectIds.add(floatingIp.getProjectId());
					}
					if (floatingIp.getCreatorId() != null) {
						userIds.add(floatingIp.getCreatorId());
					}
					if (floatingIp.getOwnerId() != null) {
						userIds.add(floatingIp.getOwnerId());
					}
					if (floatingIp.getMenderId() != null) {
						userIds.add(floatingIp.getMenderId());
					}
				}
				Map<Long, String> userMap = smsUserService.listUsersByIds(userIds).stream()
						.collect(Collectors.toMap(User::getId, User::getName));
				Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
						.collect(Collectors.toMap(Project::getId, Project::getName));
				Map<Long, String> businessMap = smsBusinessService.listBusinesssByIds(businessIds).stream()
						.collect(Collectors.toMap(Business::getId, Business::getName));
				Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
						.collect(Collectors.toMap(Tenant::getId, Tenant::getName));
				for (FloatingIp floatingIp : list) {
					// 设置云主机归属信息
					if (floatingIp.getTenantId() != null) {
						floatingIp.setTenantName(tenantMap.get(floatingIp.getTenantId()));
					}
					if (floatingIp.getBusinessId() != null) {
						floatingIp.setBusinessName(businessMap.get(floatingIp.getBusinessId()));
					}
					if (floatingIp.getProjectId() != null) {
						floatingIp.setProjectName(projectMap.get(floatingIp.getProjectId()));
					}
					if (floatingIp.getCreatorId() != null) {
						floatingIp.setCreatorName(userMap.get(floatingIp.getCreatorId()));
					}
					if (floatingIp.getOwnerId() != null) {
						floatingIp.setOwnerName(userMap.get(floatingIp.getOwnerId()));
					}
					if (floatingIp.getMenderId() != null) {
						floatingIp.setMenderName(userMap.get(floatingIp.getMenderId()));
					}
				}
			}
		} catch (Exception e) {
			log.error("获取浮动IP基本信息异常：" + e.getMessage());
		}
		return list;

	}

	@Override
	public GeneralResult condition(String condition) {
		try {
			JSONObject object = JSONObject.parseObject(condition);
			if ("count".equalsIgnoreCase(object.getString("condition"))) {
				return count(object.getLong("userId"), object.getLong("projectId"), object.getLong("tenantId"));
			}

			return new GeneralResult<>(false, "查询失败");
		} catch (Exception e) {
			log.error("list floating ip fail:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}

	@Override
	public GeneralResult<FloatingIp> detail(Long id) {
		try {
			FloatingIp fip = floatingIpRepository.query(id);
			if (null == fip) {
				return new GeneralResult<>(false, "浮动IP不存在");
			}
			// 设置云主机归属
			if (null != fip.getOwnerId() && 0L != fip.getOwnerId()) {
				GeneralResult<User> ownerDetail = smsUserService.detail(fip.getOwnerId());
				User owner = ownerDetail.getData();
				if (owner != null) {
					fip.setOwnerName(owner.getName());
				}
			}
			if (null != fip.getMenderId() && 0L != fip.getMenderId()) {
				GeneralResult<User> menderDetail = smsUserService.detail(fip.getMenderId());
				User mender = menderDetail.getData();
				if (mender != null) {
					fip.setMenderName(mender.getName());
				}
			}
			if (null != fip.getCreatorId() && 0L != fip.getCreatorId()) {
				GeneralResult<User> creatorDetail = smsUserService.detail(fip.getCreatorId());
				User creator = creatorDetail.getData();
				if (creator != null) {
					fip.setCreatorName(creator.getName());
				}
			}
			if (null != fip.getProjectId() && 0L != fip.getProjectId()) {
				GeneralResult<Project> projectResult = smsProjectService.detail(fip.getProjectId());
				if (projectResult.isFailed() || null == projectResult.getData()) {
					return new GeneralResult(false, projectResult.getMessage());
				}
				Project project = projectResult.getData();
				if (null != project) {
					fip.setProjectName(project.getName());
				}
			}
			if (null != fip.getBusinessId() && 0L != fip.getBusinessId()) {
				GeneralResult<Business> businessDetail = smsBusinessService.detail(fip.getBusinessId());
				Business business = businessDetail.getData();
				if (business != null) {
					fip.setBusinessName(business.getName());
				}
			}
			if (null != fip.getTenantId() && 0L != fip.getTenantId()) {
				GeneralResult<Tenant> tenantDetail = smsTenantService.detail(fip.getTenantId());
				Tenant tenant = tenantDetail.getData();
				if (tenant != null) {
					fip.setTenantName(tenant.getName());
				}
			}
			if (!StringUtils.isEmpty(fip.getTenantUuid())) {
				OSTenant osTenant = osTenantRepository.getByTenantUuid(fip.getTenantUuid(), fip.getVendorId());
				if (null != osTenant) {
					fip.setOsTenantName(osTenant.getName());
				}
			}
			// 浮动ip标签
			List<JSONObject> parseArray = JSON.parseArray(fip.getResourceLabel(), JSONObject.class);
			if (!ListTool.isEmpty(parseArray)) {
				GeneralResult existsValues = labelService.isExistsValues(parseArray);
				if (existsValues.isFailed()) {
					return existsValues;
				}
				fip.setResourceLabel(JSONObject.toJSONString(existsValues.getData()));
				floatingIpRepository.update(fip);
			}
			return new GeneralResult<>(true, fip, "查询成功!");
		} catch (Exception e) {
			log.error("query router detail failure:", e);
			return new GeneralResult<>(false, "查询详情失败!");
		}
	}

	@Override
	public GeneralResult count(Long userId, Long projectId, Long tenantId) {
		try {
			List<Param> params = Lists.newArrayList();
			Catalog catalog = Catalog.Tenant;
			Long target = tenantId;
			if (null != userId) {
				target = userId;
				catalog = Catalog.User;
			}
			List<Project> projects = smsProjectService.projectsByCondition(target, catalog);
			List<Long> projectIds = new ArrayList<>();
			// 此时查询的是全部用户全部项目的信息
			if (null != projectId && 0L != projectId) {
				projectIds.add(projectId);
			} else if (!ListTool.isEmpty(projects)) {
				projects.forEach(project -> projectIds.add(project.getId()));
			} else {
				// 当前用户不属于任何项目，则登录之后查看不到任何信息
				projectIds.add(-1L);
			}
			int total = floatingIpRepository.count(params, null, projectIds);
			return new GeneralResult<>(true, total, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

}
