package com.bocloud.cmp.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.*;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.model.BatchUpdateImageParam;
import com.bocloud.cmp.model.ImgBean;
import com.bocloud.cmp.repository.*;
import com.bocloud.cmp.service.ImageService;
import com.bocloud.cmp.service.ResourcePoolService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.*;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 镜像实现类
 *
 * @author DZG
 * @since V1.0 2016年9月13日
 */
@Service("imageService")
@Slf4j
public class ImageServiceImpl implements ImageService {

	@Autowired
	private ImageRepository imageRepository;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private CloudServerRepository cloudServerRepository;
	@Autowired
	private SmsProjectService smsProjectService;
	@Autowired
	private ResourcePoolService resourcePoolService;
	@Autowired
	private FlavorRepository flavorRepository;
	@Autowired
	private NetworkCardRepository networkCardRepository;

	@Override
	public GeneralResult list(Pager pager, RequestContext context) {
		try {
			List<Image> list = new ArrayList<>();
			List<SimpleBean> beans = null;
			int total = 0;
			GridBean gridBean = null;

			Boolean applyList = false;
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			for (Param param : params) {
				Map<String, Object> paramMap = param.getParam();
				if (paramMap.containsKey("applyList")) {
					applyList = Boolean.valueOf(String.valueOf(paramMap.get("applyList")));
					paramMap.remove("applyList");
					break;
				}
			}

			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			sorter.put("gmt_create", Common.ONE);
			sorter.put("os_type", Common.ZERO);
			Catalog catalog = context.getCatalog();
			if ((Catalog.Tenant == catalog || Catalog.User == catalog) && !applyList) {
				String vendorType = null;
				for (Param param : params) {
					Map<String, Object> paramMap = param.getParam();
					if (paramMap.containsKey("vendorType")) {
						vendorType = String.valueOf(paramMap.get("vendorType"));
						paramMap.remove("vendorType");
					}
				}
				Long tenantId = context.getTenant();
				params.add(new Param(MapTools.simpleMap("tenantId", tenantId), Sign.EQ));
				List<Long> tenantVendorIds = resourcePoolService.listTenantVendorIds(tenantId, vendorType);
				tenantVendorIds.add(-1L);
				total = imageRepository.count(params, tenantVendorIds);
				List<Image> imageList = this.imageRepository.list(page, rows, params, sorter, tenantVendorIds);
				gridBean = GridHelper.getBean(page, rows, total, imageList);
			} else {
				total = this.imageRepository.count(params);
				if (simple) {
					beans = this.imageRepository.list(params, sorter);
					gridBean = new GridBean(1, 1, total, beans);
				} else {
					list = this.imageRepository.list(page, rows, params, sorter);
					// 镜像数据处理
					for (Image image : list) {
						if(null != image.getSize()) {
							image.setSize(new BigDecimal(image.getSize()).setScale(2, RoundingMode.HALF_UP).doubleValue());
						}
					}
					gridBean = GridHelper.getBean(page, rows, total, list);
				}
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Query image list fail:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}

	private GeneralResult listTenantPublic(JSONObject object, RequestContext context) {
		if (null == object) {
			return new GeneralResult<>(false, "参数格式错误");
		}
		Long vendorId = object.getLong(Common.VENDORID);
		String regionId = object.getString("regionId");
		String status = object.getString("status");
		Long serverId = object.getLong("serverId");
		try {
			List<Param> params = new ArrayList<>();
			if (null != vendorId) {
				params.add(new Param(MapTools.simpleMap("regionId", regionId), Sign.EQ));
			}
			if (!StringUtils.isEmpty(regionId)) {
				params.add(new Param(MapTools.simpleMap("status", status), Sign.EQ));
			}
			if (!StringUtils.isEmpty(status)) {
				params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
			}
			if (null != serverId) {
				CloudServer cloudServer = cloudServerRepository.query(serverId);
				String flavorUuid = cloudServer.getFlavorUuid();
				if (!StringUtils.isEmpty(flavorUuid)) {
					Flavor flavor = flavorRepository.getByFlavorId(flavorUuid, cloudServer.getVendorId(), null);
					if (null != flavor && flavor.getCpuArchitecture().contains("kunpeng")) {
						params.add(new Param(MapTools.simpleMap("support", "Arm"), Sign.LK));
					}
					if (null != flavor && flavor.getCpuArchitecture().contains("x86")) {
						params.add(new Param(MapTools.simpleMap("support", "Kvm"), Sign.LK));
					}
				}
			}
			// 查询租户下的镜像
			Map<String, String> sorter = Maps.newHashMap();
			sorter.put("gmt_create", Common.ONE);
			sorter.put("os_type", Common.ZERO);
			List<Image> list = this.imageRepository.list(1, Integer.MAX_VALUE, params, sorter, context.getTenant(),
					null);
			// 查询共有镜像
			params.add(new Param(MapTools.simpleMap("imageType", "PUBLIC"), Sign.EQ));
			List<Image> publicList = this.imageRepository.list(1, Integer.MAX_VALUE, params, sorter, 0L, null);
			publicList.addAll(list);
			return new GeneralResult<>(true, publicList, "查询成功!");
		} catch (Exception e) {
			log.error("get image failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}

	}

	private GeneralResult listByCategory(JSONObject object) {
		try {
			CloudVendor vendor = cloudVendorRepository.query(object.getLong("vendorId"));
			if (null == vendor) {
				return new GeneralResult<>(false, "平台信息异常！");
			}
			// 操作系统类型(Linux/Windows)
			String osCategory = object.getString("osCategory");
			// 操作系统版本
			String osVersion = object.getString("osVersion");

			if (StringUtils.isEmpty(osCategory) || StringUtils.isEmpty(osVersion)) {
				return new GeneralResult<>(false, "参数格式错误！");
			}
			List<Image> images = imageRepository.listByVendorId(vendor.getId());
			Image targetImage = null;
			if ("MANAGEONE".equals(vendor.getType())) {
				targetImage = images.stream().filter(image -> !StringUtils.isEmpty(image.getImageLabel())
						&& image.getImageLabel().contains(osCategory) && image.getImageLabel().contains(osVersion)
						&& "public".equalsIgnoreCase(image.getImageType())).findFirst()
						.orElseThrow(() -> new Exception("未找到合适的镜像"));
			} else {
				targetImage = images.stream().filter(image -> !StringUtils.isEmpty(image.getImageLabel())
						&& image.getImageLabel().contains(osCategory) && image.getImageLabel().contains(osVersion))
						.findFirst().orElseThrow(() -> new Exception("未找到合适的镜像"));
			}
			if (null == targetImage) {
				return new GeneralResult<>(false, "未找到合适的镜像!");
			} else {
				List<NetworkCard> cards = networkCardRepository.listByHost(targetImage.getTemplateId(), "VM");
				targetImage.setNetworkCards(cards);
				return new GeneralResult<>(true, targetImage, "查询成功！");
			}
		} catch (Exception e) {
			log.error("查询镜像失败：" + e.getMessage());
			return new GeneralResult<>(false, "查询镜像失败：" + e.getMessage());
		}
	}

	private GeneralResult listByImageType(String condition, RequestContext context) {
		try {
			JSONObject object = JSONObject.parseObject(condition);
			Long vendorId = object.getLong("vendorId");
			CloudVendor vendor = cloudVendorRepository.query(vendorId);
			if (null == vendor) {
				return new GeneralResult<>(false, "云平台信息不存在");
			}
			List<Image> list = new ArrayList<>();
			List<Param> params = new ArrayList<>();
			params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
			if (object.containsKey("status")) {
				params.add(new Param(MapTools.simpleMap("status", object.getString("status")), Sign.EQ));
			}
			if (object.containsKey("regionId")) {
				params.add(new Param(MapTools.simpleMap("regionId", object.getString("regionId")), Sign.EQ));
			}
			if (object.containsKey("support")) {
				params.add(new Param(MapTools.simpleMap("support", object.getString("support")), Sign.LK));
			}
			if (object.containsKey("architecture")) {
				params.add(new Param(MapTools.simpleMap("architecture", object.getString("architecture")), Sign.LK));
			}
			if (object.containsKey("imageType")) {
				String imageType = object.getString("imageType");
				params.add(new Param(MapTools.simpleMap("imageType", imageType), Sign.EQ));
				if ("PRIVATE".equalsIgnoreCase(imageType)) {
					Catalog catalog = context.getCatalog();
					if (catalog.equals(Catalog.Tenant) || catalog.equals(Catalog.User)) {
						params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
					}
				}
			}
			if ("CTCLOUD".equals(vendor.getType())) {
				list = this.imageRepository.listHighConfigForCTCloud(1, Integer.MAX_VALUE, params, null);
			} else if ("CTSTACK".equals(vendor.getType())) {
				list = this.imageRepository.listHighConfigForCTStack(1, Integer.MAX_VALUE, params, null);
			} else {
				list = this.imageRepository.list(1, Integer.MAX_VALUE, params, null);
			}
			Map<String, List<Image>> imageMap = new HashMap<String, List<Image>>();
			for (Image image : list) {
				if (imageMap.containsKey(image.getOsType())) {
					List<Image> oldList = imageMap.get(image.getOsType());
					oldList.add(image);
					imageMap.put(image.getOsType(), oldList);
				} else {
					List<Image> newList = new ArrayList<Image>();
					newList.add(image);
					imageMap.put(image.getOsType(), newList);
				}
			}

			return new GeneralResult<>(true, imageMap, "查询成功!");
		} catch (Exception e) {
			log.error("get image failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult<Image> detail(Long id) {
		try {
			Image image = imageRepository.query(id);
			CloudVendor cloudVendor = cloudVendorRepository.query(image.getVendorId());
			if (null != cloudVendor) {
				image.setVendorName(cloudVendor.getName());
			}
			if (null != image.getTemplateId()) {
				CloudServer server = cloudServerRepository.query(image.getTemplateId());
				if (null != server) {
					List<NetworkCard> cards = networkCardRepository.listByHost(server.getId(), "VM");
					server.setNetcards(cards);
				}
				image.setServer(server);
			}
			return new GeneralResult<>(true, image, "查询成功!");
		} catch (Exception e) {
			log.error("get image failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

    @Override
    public GeneralResult remove(Long id, RequestContext context) {
    	return null;
    }

	@Override
	public GeneralResult modify(ImgBean bean, Long userId) {
		return null;
	}

	@Override
	public GeneralResult update(ImgBean bean, Long userId) {
		String path = Image.class.getSimpleName() + "_" + bean.getId();
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			Image image = imageRepository.query(bean.getId());
			if (null == image) {
				log.warn("image does not exist!");
				return new GeneralResult<>(false, "数据不存在!");
			}
			CloudVendor vendor = cloudVendorRepository.query(image.getVendorId());
			if (null != vendor && "MANAGEONE".equals(vendor.getType())
					&& "private".equals(image.getImageType())) {
				return new GeneralResult<>(false, "此镜像为私有镜像，不能用于公共镜像配置");
			}
			image.setImageLabel(bean.getImageLabel());
			image.setRemark(bean.getRemark());
			image.setMenderId(userId);
			imageRepository.update(image);
			return new GeneralResult<>(true, "镜像配置成功");
		} catch (Exception e) {
			log.error("Modify image fail:", e);
			return new GeneralResult<>(false, "镜像配置失败");
		}
	}

	@Override
	public GeneralResult create(Image bean, RequestContext context) {
		return null;
	}

	@Override
	public GeneralResult listByStoreId(Long storeId) {
		try {
			List<Image> list = imageRepository.listByStoreId(storeId);
			return new GeneralResult<>(true, list, "查询成功!");
		} catch (Exception e) {
			log.error("get image failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult listByVendor(Long vendorId) {
		try {
			List<Image> list = imageRepository.listByVendorId(vendorId);
			return new GeneralResult<>(true, list, "查询成功!");
		} catch (Exception e) {
			log.error("get image failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}

	@Override
	public GeneralResult listByTenant(String params, RequestContext context) {
		JSONObject object = JSONTools.isJSONObj(params);
		if (null == object) {
			return new GeneralResult<>(false, "参数格式错误");
		}
		String osCategory = object.getString("osCategory");
		String osName = object.getString("osName");
		String osVersion = object.getString("osVersion");
		Long vendorId = object.getLong(Common.VENDORID);
		String region = object.getString("regionId");
		Boolean hasConfig = object.getBoolean("hasConfig");
		String regionId = object.getString("regionId");
		try {
			List<Image> images = imageRepository.listByTenant(vendorId, context.getTenant(), region);
			List<Image> filterImgs = Lists.newArrayList();
			// 筛选镜像
			for (Image image : images) {
				String imgLabel = image.getImageLabel();
				if (StringUtils.isEmpty(imgLabel)) {
					continue;
				}
				JSONObject imgLabelObj = JSONObject.parseObject(imgLabel);
				if (null != hasConfig && hasConfig && null != imgLabelObj) {
					if (!StringUtils.isEmpty(regionId) && !regionId.equalsIgnoreCase(image.getRegionId())) {
						continue;
					}
					filterImgs.add(image);
				} else if (null != imgLabelObj && null != osCategory && null != osName && null != osVersion
						&& osCategory.equals(imgLabelObj.getString("os_category"))
						&& osName.equals(imgLabelObj.getString("os_name"))
						&& osVersion.equals(imgLabelObj.getString("os_version"))) {
					if (!StringUtils.isEmpty(regionId) && !regionId.equalsIgnoreCase(image.getRegionId())) {
						continue;
					}
					filterImgs.add(image);
				}
			}
			// 前端取值 minSize
			for (Image image : filterImgs) {
				if (null == image.getMinDisk() || image.getMinDisk() <= 0) {
					image.setMinDisk(null == image.getSize() ? 0 : image.getSize().longValue());
				}
			}
			return new GeneralResult<>(true, filterImgs, "查询成功!");
		} catch (Exception e) {
			log.error("get image failure:", e);
			return new GeneralResult<>(false, "查询失败!");
		}
	}


	@Override
	public GeneralResult batchRemove(List<Long> ids, RequestContext context) {
		try {
			if (ids.isEmpty()) {
				return new GeneralResult<>(false, "没有需要删除的镜像！");
			}
			for (Long id : ids) {
				GeneralResult result = remove(id, context);
				if (result.isFailed()) {
					return result;
				}
			}
		} catch (Exception e) {
			log.error("batch remove image fail:", e);
			return new GeneralResult<>(false, "批量删除任务下发失败");
		}
		return new GeneralResult<>(true, "批量删除任务已下发，正在执行...");
	}

	@Override
	public GeneralResult listByReg(String reg, Long vendorId) {
		try {
			List<Image> images = imageRepository.listByReg(reg, vendorId);
			return new GeneralResult<>(true, images, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult count(Long userId, Long projectId, Long tenantId) {
		try {
			List<Param> params = Lists.newArrayList();
			Catalog catalog = Catalog.Tenant;
			Long target = tenantId;
			if (null != userId) {
				target = userId;
				catalog = Catalog.User;
			}
			List<Project> projects = smsProjectService.projectsByCondition(target, catalog);
			List<Long> projectIds = new ArrayList<>();
			// 此时查询的是全部用户全部项目的信息
			if (null != projectId && 0L != projectId) {
				projectIds.add(projectId);
			} else if (!ListTool.isEmpty(projects)) {
				projects.forEach(project -> projectIds.add(project.getId()));
			} else {
				// 当前用户不属于任何项目，则登录之后查看不到任何信息
				projectIds.add(-1L);
			}
			int total = this.imageRepository.count(params, 0L, projectIds);
			return new GeneralResult<>(true, total, "查询成功");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "查询失败");
		}
	}

	@Override
	public GeneralResult imageExist(String osType, String osVersion) {
		try {
			if (this.imageRepository.exists(osType, osVersion)) {
				return new GeneralResult<>(true, "镜像已经存在");
			} else {
				return new GeneralResult<>(false, "镜像不存在");
			}
		} catch (Exception e) {
			log.error("检查image是否存在异常:", e);
			return new GeneralResult<>(false, "检查image是否存在异常");
		}
	}

	@Override
	public GeneralResult remove(String osType, String osVersion, Long userId) {
		try {
			boolean success = this.imageRepository.deletePhysicalImage(osType, osVersion, userId);
			return new GeneralResult<>(success, success ? "删除物理镜像成功" : "删除物理镜像失败");
		} catch (Exception e) {
			log.error("删除image异常:", e);
			return new GeneralResult<>(false, "删除物理机镜像异常");
		}
	}

	@Override
	public GeneralResult batchUpdate(BatchUpdateImageParam params, RequestContext context) {
		List<Long> ids = params.getIds();
		String imageLabel = params.getImageLabel();
		String remark = params.getRemark();
		try {
			List<GeneralResult> succResult = Lists.newArrayList();
			List<GeneralResult> failResult = Lists.newArrayList();
			for (Long id : ids) {
				ImgBean bean = new ImgBean();
				bean.setId(id);
				bean.setRemark(remark);
				bean.setImageLabel(imageLabel);
				GeneralResult result = update(bean, context.getTarget());
				if (result.isFailed()) {
					failResult.add(result);
				} else {
					succResult.add(result);
				}
			}
			return new GeneralResult<>(true, "批量保存成功【" + succResult.size() + "】个，失败【" + failResult.size() + "】个");
		} catch (Exception e) {
			log.error("Get error message：", e);
			return new GeneralResult<>(false, "批量保存失败");
		}
	}


	/**
	 * 镜像部分修改
	 *
	 * @param params
	 * @param context
	 * @return
	 */
	@Override
	public GeneralResult patch(Long id, JSONObject params, RequestContext context, String action) {
		if (null == action || "".equals(action)) {
			return new GeneralResult<>(false, "暂不支持此操作");
		}
		switch (action) {
		case "update":
			ImgBean bean = JSONObject.parseObject(params.toJSONString(), ImgBean.class);
			bean.setId(id);
			return this.update(bean, context.getTarget());
		default:
			return new GeneralResult<>(false, "暂不支持此操作");
		}
	}

	/**
	 * 下载镜像
	 *
	 * @param id
	 * @return
	 */
	@Override
	public GeneralResult download(Long id) {
		return null;
	}


	@Override
	public GeneralResult<List<Volume>> listVolumeByImage(long imageId) {
		return null;
	}

    @Override
    public GeneralResult<GridBean<CloudServer>> listServerByImage(long imageId, Pager pager) {
        List<CloudServer> list = cloudServerRepository.listServerByImageId(imageId, pager);
        int count = cloudServerRepository.count(imageId, pager.getParams());
        GridBean<CloudServer> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), count, list);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

	@Override
	public GeneralResult condition(String condition, RequestContext context) {
		condition = Optional.ofNullable(condition).orElse("");
		JSONObject object = JSONObject.parseObject(condition);
		if ("listByVendor".equalsIgnoreCase(object.getString("condition"))) {
			return this.listByVendor(object.getLong(Common.VENDORID));
		} else if ("listByStoreId".equalsIgnoreCase(object.getString("condition"))) {
			return this.listByStoreId(object.getLong("storeId"));
		} else if ("listByTenant".equalsIgnoreCase(object.getString("condition"))) {
			return this.listByTenant(condition, context);
		} else if ("listByReg".equalsIgnoreCase(object.getString("condition"))) {
			return this.listByReg(object.getString("reg"), object.getLong(Common.VENDORID));
		} else if ("count".equalsIgnoreCase(object.getString("condition"))) {
			return this.count(object.getLong("userId"), object.getLong("projectId"), object.getLong("tenantId"));
		} else if ("exist".equalsIgnoreCase(object.getString("condition"))) {
			return this.imageExist(object.getString("osType"), object.getString("osVersion"));
		} else if ("listByCondition".equalsIgnoreCase(object.getString("condition"))) {
			List<Image> images = imageRepository.listByCondition(object.getString("region"),
					object.getString("availablitiyZone"));
			return new GeneralResult<>(true, images, "查询成功");
		} else if ("listPublicImage".equalsIgnoreCase(object.getString("condition"))) {
			List<Image> images = imageRepository.listPublicImage(object.getLong("vendorId"));
			return new GeneralResult<>(true, images, "查询成功");
		} else if ("listByCategory".equalsIgnoreCase(object.getString("condition"))) {
			return listByCategory(object);
		} else if ("listTenantImages".equalsIgnoreCase(object.getString("condition"))) {
			return listTenantImages(object, context);
		} else if ("listsoftImages".equalsIgnoreCase(object.getString("condition"))) {
			return listSoftImages(object, context);
		} else if ("listTenantPublic".equalsIgnoreCase(object.getString("condition"))) {
			return listTenantPublic(object, context);
		} else if ("listByImageType".equalsIgnoreCase(object.getString("condition"))) {
			return listByImageType(condition, context);
		} else if ("listByService".equalsIgnoreCase(object.getString("condition"))) {
			return listByService(object);
		}
		return new GeneralResult<>(false, "参数格式错误");
	}

	private GeneralResult listSoftImages(JSONObject object, RequestContext context) {
		try {
			CloudVendor vendor = cloudVendorRepository.query(object.getLong("vendorId"));
			if (null == vendor) {
				return new GeneralResult<>(false, "平台信息异常！");
			}
			String service = object.getString("service");
			List<Image> images = imageRepository.listByVendorId(vendor.getId());
			Map<String, List<Image>> imageMap = new HashMap<String, List<Image>>();
			for (Image img : images) {
				if (StringUtils.isEmpty(img.getImageLabel()) || !img.getImageLabel().contains(service)) {
					continue;
				}
				JSONObject labelObject = JSONObject.parseObject(img.getImageLabel(), JSONObject.class);
				String osCategory = labelObject.getString("osCategory");
				List<Image> list = imageMap.containsKey(osCategory) ? imageMap.get(osCategory) : new ArrayList<>();
				list.add(img);
				imageMap.put(osCategory, list);
			}
			if (imageMap.keySet().size() <= 0) {
				return new GeneralResult<>(false, "未找到合适的镜像!");
			}
			Set<String> osCategoryKeySet = imageMap.keySet();
			Map<String, List<String>> resultMap = new HashMap<>();
			for (String osCategroy : osCategoryKeySet) {
				List<String> versions = new ArrayList<>();
				List<Image> osCagetoryLists = imageMap.get(osCategroy);
				for (Image image : osCagetoryLists) {
					JSONObject labelObject = JSONObject.parseObject(image.getImageLabel(), JSONObject.class);
					String version = labelObject.getString("osVersion");
					if (!versions.contains(version)) {
						versions.add(version);
					}
				}
				resultMap.put(osCategroy, versions);
			}
			return new GeneralResult<>(true, resultMap, "查询成功！");
		} catch (Exception e) {
			log.error("查询镜像失败：" + e.getMessage());
			return new GeneralResult<>(false, "查询镜像失败：" + e.getMessage());
		}
	}

	private GeneralResult listTenantImages(JSONObject object, RequestContext context) {
		return null;
	}

	private GeneralResult listByService(JSONObject object) {
        try {
            CloudVendor vendor = cloudVendorRepository.query(object.getLong("vendorId"));
            if (null == vendor) {
                return new GeneralResult<>(false, "平台信息异常！");
            }
            // 操作系统类型(Linux/Windows)
            String osCategory = object.getString("osCategory");
            // 操作系统版本
            String osVersion = object.getString("osVersion");
            // 中间件服务
            String serviceType = object.getString("serviceType");
            if (StringUtils.isEmpty(osCategory) || StringUtils.isEmpty(osVersion) || StringUtils.isEmpty(serviceType)) {
                return new GeneralResult<>(false, "参数格式错误！");
            }
            List<Image> images = imageRepository.listByVendorId(vendor.getId());
            Image targetImage = images.stream()
                    .filter(image -> !StringUtils.isEmpty(image.getImageLabel())
                            && image.getImageLabel().contains(osCategory) && image.getImageLabel().contains(osVersion)
                            && image.getImageLabel().contains(serviceType))
                    .findFirst().orElseThrow(() -> new Exception("未找到合适的镜像"));
            if (null == targetImage) {
                return new GeneralResult<>(false, "未找到合适的镜像!");
            }
            List<NetworkCard> cards = networkCardRepository.listByHost(targetImage.getTemplateId(), "VM");
            targetImage.setNetworkCards(cards);
            return new GeneralResult<>(true, targetImage, "查询成功！");
        } catch (Exception e) {
            log.error("查询镜像失败：" + e.getMessage());
            return new GeneralResult<>(false, "查询镜像失败：" + e.getMessage());
        }
    }

}