package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.entity.ResourceEvent;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.User;
import com.bocloud.cmp.internal.SmsManagerService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.internal.SmsUserService;
import com.bocloud.cmp.repository.ResourceEventRepository;
import com.bocloud.cmp.service.ResourceEventService;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;

import lombok.extern.slf4j.Slf4j;

/**
 * 资源事件Service接口实现
 * 
 * @author dongkai
 *
 */
@Service("resourceEventService")
@Slf4j
public class ResourceEventServiceImpl implements ResourceEventService {

	@Autowired
	private ResourceEventRepository resourceEventRepository;
	@Autowired
	private SmsUserService smsUserService;
	@Autowired
	private SmsTenantService smsTenantService;
	@Autowired
	private SmsManagerService smsManagerService;

	@Override
	public GeneralResult<GridBean<ResourceEvent>> list(Pager pager, RequestContext context) {
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
			sorter.put("gmtOperate", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
			List<ResourceEvent> list = resourceEventRepository.list(page, rows, params, sorter);
			for (ResourceEvent event : list) {
				if (null == event.getCatalog()) {
					continue;
				}
				switch (event.getCatalog()) {
				case Manager:
					GeneralResult<User> managerResult = smsManagerService.detail(event.getOperatorId());
					if (managerResult.isSuccess() && null != managerResult.getData()) {
						event.setOperator(managerResult.getData().getName());
					}
					break;
				case Tenant:
					GeneralResult<Tenant> tenantResult = smsTenantService.detail(event.getOperatorId());
					if (tenantResult.isSuccess() && null != tenantResult.getData()) {
						event.setOperator(tenantResult.getData().getName());
					}
					break;
				case User:
					GeneralResult<User> userResult = smsUserService.detail(event.getOperatorId());
					if (userResult.isSuccess() && null != userResult.getData()) {
						event.setOperator(userResult.getData().getName());
					}
					break;
				default:
					break;
				}
			}
			int total = resourceEventRepository.count(params);
			GridBean<ResourceEvent> gridBean = GridHelper.getBean(page, rows, total, list);
			return new GeneralResult<GridBean<ResourceEvent>>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list resourceEvent failure:", e);
			return new GeneralResult<GridBean<ResourceEvent>>(false, "查询失败", null, null);
		}
	}

}
