package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cmp.entity.ResourceModule;
import com.bocloud.cmp.entity.VendorCategory;
import com.bocloud.cmp.repository.ResourceModuleRepository;
import com.bocloud.cmp.repository.ResourcePropertyRepository;
import com.bocloud.cmp.service.ResourceModuleService;
import com.google.common.collect.Maps;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 平台类型 实现类
 */
@Service("resourceModuleService")
@Slf4j
public class ResourceModuleServiceImpl implements ResourceModuleService {

	@Autowired
	private ResourceModuleRepository resourceModuleRepository;
	@Autowired
	private LockFactory lockFactory;
	@Autowired
	private ResourcePropertyRepository resourcePropertyRepository;

	@Override
	public GeneralResult<ResourceModule> create(ResourceModule resourceModule, RequestContext context) {
		try {
			resourceModule.setCreatorId(context.getTarget());
			resourceModule.setMenderId(context.getTarget());
			resourceModule.setStatus(BaseStatus.NORMAL.name());
			log.info("name: " + resourceModule.getName() + " content : " + resourceModule.getContent());
			resourceModuleRepository.save(resourceModule);
			return new GeneralResult<>(true, resourceModule, "添加成功");
		} catch (Exception e) {
			log.error("create vendor category fail:", e);
			return new GeneralResult<>(false, e.getMessage(), "", "");
		}
	}

	@Override
	public GeneralResult<ResourceModule> modify(ResourceModule resourceModule, RequestContext context) {
		try {
			ResourceModule dbResourceModule = resourceModuleRepository.query(resourceModule.getId());
			if(null == dbResourceModule) {
				return new GeneralResult<>(false, "数据不存在");
			}
			dbResourceModule.setName(resourceModule.getName());
			dbResourceModule.setRemark(resourceModule.getRemark());
			resourceModuleRepository.update(dbResourceModule);
			return new GeneralResult<>(true, resourceModule, "编辑成功");
		} catch (Exception e) {
			log.error("modifyreate vendor category fail:", e);
			return new GeneralResult<>(false, e.getMessage());
		}
	}

	@Override
	public GeneralResult remove(Long id, RequestContext context) {
		String path = VendorCategory.class.getSimpleName() + "_" + id;
		try (AutoCloseLock lock = lockFactory.getACLock(path)) {
			Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
			ResourceModule resourceModule = resourceModuleRepository.query(id);
			if (null == resourceModule) {
				return new GeneralResult(true, "删除成功");
			}
			resourceModuleRepository.remove(id, context.getTarget());
			// 删除模型属性
			resourcePropertyRepository.removeByModuleId(id, context.getTarget());
			return new GeneralResult<>(true, "删除成功");
		} catch (Exception e) {
			log.error("Remove data center fail:", e);
			return new GeneralResult<>(false, "删除失败", null, null);
		}
	}

	@Override
	public GeneralResult<GridBean<ResourceModule>> list(Pager pager, RequestContext context) {
		List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
		Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
		sorter.put("gmtCreate", Common.ONE);
		Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
		Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);
		List<ResourceModule> list = null;
		int total = 0;
		GridBean gridBean = null;
		try {
			total = this.resourceModuleRepository.count(params);
			list = this.resourceModuleRepository.list(page, rows, params, sorter);
			gridBean = GridHelper.getBean(page, rows, total, list);
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("Query resource module list fail:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}

	@Override
	public GeneralResult<ResourceModule> detail(Long id) {
		// TODO Auto-generated method stub
		return null;
	}

}