package com.bocloud.cmp.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.domain.AlterationParam;
import com.bocloud.cmp.domain.ApplicationParam;
import com.bocloud.cmp.domain.CancelationParam;
import com.bocloud.cmp.domain.ExtensionParam;
import com.bocloud.cmp.domain.LocationModel;
import com.bocloud.cmp.domain.OrderItem;
import com.bocloud.cmp.domain.OrderMessage;
import com.bocloud.cmp.domain.QuotaBean;
import com.bocloud.cmp.domain.ServiceOrder;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.enums.OrderItemStatus;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.PluginResourceOperationService;
import com.bocloud.cmp.service.ResourceOperationService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;

/**
 * 资源申请处理服务实现类
 */
@Slf4j
@Service("resourceOperationService")
public class ResourceOperationServiceImpl implements ResourceOperationService {

	@Autowired
	private AmqpTemplate amqpTemplate;
	@Autowired
	private PluginUser pluginUser;
	@Autowired
	private CloudVendorRepository cloudVendorRepository;

	/**
	 * 通知order 修改对应订单状态
	 * 
	 * @param uuid
	 * @param operation
	 * @param message
	 * @param status
	 * @param context
	 * @param newSn
	 */
	private void syncOrderStatus(String uuid, OrderMessage.Operation operation, String message, OrderItemStatus status,
			RequestContext context, String newSn, List<QuotaBean> quotas, String category) {
		OrderMessage orderMessage = new OrderMessage(uuid, operation, status, null, message, null, null,
				new ArrayList<>(), category, context);
		orderMessage.setNewSn(newSn);
		orderMessage.setQuotas(quotas);
		amqpTemplate.convertAndSend("resource.result.queue", orderMessage);
	}

	@Override
	public GeneralResult process(ResourceCategory category, JSONObject param, OrderMessage.Operation operation) {
		RequestContext context = JSONObject.parseObject(param.getString(Common.CONTEXT), RequestContext.class);
		GeneralResult result = null;
		CloudVendor cloudVendor = null;
		PluginResourceOperationService pluginsResourceOperationService = null;
		switch (operation) {
		case Application:
			ApplicationParam aplicaiton = null;
			try {
				aplicaiton = JSONObject.parseObject(param.toJSONString(), ApplicationParam.class);
				LocationModel location = aplicaiton.getLocation();
				cloudVendor = cloudVendorRepository.query(location.getVendorId());
				Assert.notNull(cloudVendor, "暂未查询到该平台信息");
				pluginsResourceOperationService = (PluginResourceOperationService) pluginUser
						.getBean(cloudVendor.getType().toLowerCase(), "pluginResourceOperationService");
				Assert.notNull(pluginsResourceOperationService, "暂未查询到插件信息");
				result = pluginsResourceOperationService.application(category, aplicaiton, context, operation);
			} catch (Exception e) {
				log.error("开通失败：{}", e);
				result = new GeneralResult(false, "开通失败：" + e.getMessage());
			}
			if (result.isFailed()) {
				if (!StringUtils.isEmpty(aplicaiton.getCallback())) {
					amqpTemplate.convertAndSend(aplicaiton.getCallback(), JSONObject.toJSON(result));
				}
				ServiceOrder order = aplicaiton.getOrder();
				List<OrderItem> items = order.getItems();
				List<QuotaBean> quotas = aplicaiton.getQuotas() == null ? new ArrayList<>() : aplicaiton.getQuotas();
				List<QuotaBean> beans = new ArrayList<>();
				for (QuotaBean bean : quotas) {
					beans.add(new QuotaBean(bean.getCode(), bean.getValue() / items.size()));
				}
				List<OrderItem> lastOrderItems = JSONObject.parseArray(JSONObject.toJSONString(result.getData()), OrderItem.class);
				if(!ListTool.isEmpty(lastOrderItems)) {
					items = lastOrderItems;
				}
				for (OrderItem item : items) {
					// 通知订单修改订单状态
					syncOrderStatus(item.getSn(), operation, result.getMessage(), OrderItemStatus.OpenedFailure,
							context, null, beans, category.name());
				}
				return result;
			}
			return new GeneralResult(true, "资源申请处理成功");
		case Upgrade:
		case ScaleIn:
		case ScaleOut:
			AlterationParam upgradeAlteration = null;
			try {
				upgradeAlteration = JSONObject.parseObject(param.toJSONString(), AlterationParam.class);
				cloudVendor = cloudVendorRepository.query(upgradeAlteration.getVendorId());
				Assert.notNull(cloudVendor, "暂未查询到该平台信息");
				pluginsResourceOperationService = (PluginResourceOperationService) pluginUser
						.getBean(cloudVendor.getType().toLowerCase(), "pluginResourceOperationService");
				Assert.notNull(pluginsResourceOperationService, "暂未查询到插件信息");
				result = pluginsResourceOperationService.alteration(category, upgradeAlteration, context, operation);
			} catch (Exception e) {
				log.error("变更失败：{}", e);
				result = new GeneralResult(false, "变更失败：" + e.getMessage());
			}
			if (null != result && result.isFailed()) {
				// 通知订单修改订单状态
				syncOrderStatus(upgradeAlteration.getInstance(), operation, result.getMessage(), OrderItemStatus.ChangeFailure,
						context, upgradeAlteration.getNewInstance(), upgradeAlteration.getQuotas(), category.name());
				return result;
			}
			return new GeneralResult(true, "资源变更成功处理成功");
		case Extension:
			ExtensionParam extension = null;
			try {
				extension = JSONObject.parseObject(param.toJSONString(), ExtensionParam.class);
				cloudVendor = cloudVendorRepository.query(extension.getVendorId());
				Assert.notNull(cloudVendor, "暂未查询到该平台信息");
				pluginsResourceOperationService = (PluginResourceOperationService) pluginUser
						.getBean(cloudVendor.getType().toLowerCase(), "pluginResourceOperationService");
				Assert.notNull(pluginsResourceOperationService, "暂未查询到插件信息");
				result = pluginsResourceOperationService.extension(category, extension, context, operation);
			} catch (Exception e) {
				log.error("变更失败：{}", e);
				result = new GeneralResult(false, "变更失败：" + e.getMessage());
			}
			if (null != result && result.isFailed()) {
				// 通知订单修改订单状态
				syncOrderStatus(extension.getInstance(), operation, result.getMessage(),
						OrderItemStatus.ExtensionFailure, context, null, null, category.name());
				// 通知订单修改订单状态
				return result;
			}
			return new GeneralResult(true, "资源变更成功处理成功");
		case Cancelation:
			CancelationParam cancelation = null;
			try {
				cancelation = JSONObject.parseObject(param.toJSONString(), CancelationParam.class);
				cloudVendor = cloudVendorRepository.query(cancelation.getVendorId());
				Assert.notNull(cloudVendor, "暂未查询到该平台信息");
				pluginsResourceOperationService = (PluginResourceOperationService) pluginUser
						.getBean(cloudVendor.getType().toLowerCase(), "pluginResourceOperationService");
				Assert.notNull(pluginsResourceOperationService, "暂未查询到插件信息");
				result = pluginsResourceOperationService.cancelation(category, cancelation, context, operation);
			} catch (Exception e) {
				log.error("退订失败：{}", e);
				result = new GeneralResult(false, "退订失败：" + e.getMessage());
			}
			if (null != result && result.isFailed()) {
				// 通知订单修改订单状态
				syncOrderStatus(cancelation.getInstance(), operation, result.getMessage(), OrderItemStatus.UnsubscribeFailure,
						context, null, null, category.name());
				return result;
			}
			return new GeneralResult(true, "资源退订处理成功");
		default:
			return new GeneralResult(true, "暂不支持资源的其他类型操作");
		}
	}

}
