package com.bocloud.cmp.service.impl;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.RosOperationService;
import com.bocloud.cmp.service.RosPluginsOperationService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.JSONTools;

import lombok.extern.slf4j.Slf4j;

/**
 * 编排服务申请
 * 
 * @author hp
 *
 */
@Service
@Slf4j
public class RosOperationServiceImpl implements RosOperationService {

    @Autowired
    private PluginUser pluginUser;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;

    @Override
    public GeneralResult<String> process(String content) {
        JSONObject object = JSONTools.isJSONObj(content);
        String queue = object.getString("callback");
        try {
            RequestContext context = JSONObject.parseObject(object.getString("context"), RequestContext.class);
            JSONObject paramObj = object.getJSONObject("params");
            if (null == paramObj) {
                return new GeneralResult<>(false, queue, "暂无申请参数");
            }
            paramObj.put("origin", object.getString("origin"));
            Long vendorId = paramObj.getLong("vendorId");
            if (null == vendorId) {
                // 获取location参数
                JSONObject locationObject = paramObj.getJSONObject("location");
                vendorId = locationObject.getLong("vendorId");
            }
            if (null == vendorId) {
                return new GeneralResult<>(false, queue, "申请参数异常，缺少vendorId参数");
            }
            CloudVendor cloudVendor = cloudVendorRepository.query(vendorId);
            RosPluginsOperationService rosPluginsOperationService = (RosPluginsOperationService)pluginUser
                .getBean(cloudVendor.getType().toLowerCase(), "rosPluginsOperationService");
            if (null == rosPluginsOperationService) {
                log.error("编排资源申请异常：暂未找到" + cloudVendor.getType() + "插件信息！");
                return new GeneralResult<>(false, queue, "暂未找到插件信息！");
            }
            switch (object.getString("code")) {
                case "ros.resource.easystack.vpc":
                case "ros.resource.apsarastack.vpc":
                case "ros.resource.aliyun.vpc":
                case "ros.resource.openstack.vpc":
                case "ros.resource.huawei.vpc":
                case "ros.resource.tencent.vpc":
                    rosPluginsOperationService.createVpc(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.apsarastack.ecs":
                case "ros.resource.vmware.ecs":
                case "ros.resource.easystack.ecs":
                case "ros.resource.aliyun.ecs":
                case "ros.resource.openstack.ecs":
                case "ros.resource.huawei.ecs":
                case "ros.resource.tencent.ecs":
                    rosPluginsOperationService.createServer(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.easystack.volume":
                case "ros.resource.apsarastack.volume":
                case "ros.resource.aliyun.volume":
                case "ros.resource.openstack.volume":
                case "ros.resource.huawei.volume":
                case "ros.resource.tencent.volume":
                    rosPluginsOperationService.createVolume(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.cfw.osp":
                    rosPluginsOperationService.createFirewall(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.fwp.osp":
                    rosPluginsOperationService.createFirewallPolicy(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.fwr.osp":
                    rosPluginsOperationService.createFirewallRule(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.clb.osp":
                    rosPluginsOperationService.createLoadBalancer(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.cls.osp":
                    rosPluginsOperationService.createListener(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.clm.osp":
                    rosPluginsOperationService.createLbMember(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.vpc.osp":
                    rosPluginsOperationService.createNetwork(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.crs.osp":
                    rosPluginsOperationService.createRouter(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.easystack.securitygroup":
                case "ros.resource.apsarastack.securitygroup":
                case "ros.resource.aliyun.securitygroup":
                case "ros.resource.openstack.securitygroup":
                case "ros.resource.huawei.securitygroup":
                case "ros.resource.tencent.securitygroup":
                    rosPluginsOperationService.createSecGroup(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.sgr.osp":
                    rosPluginsOperationService.createSecGroupRule(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.easystack.subnet":
                case "ros.resource.apsarastack.subnet":
                case "ros.resource.aliyun.subnet":
                case "ros.resource.openstack.subnet":
                case "ros.resource.huawei.subnet":
                case "ros.resource.tencent.subnet":
                    rosPluginsOperationService.createSubnet(paramObj.toJSONString(), context, queue);
                    break;
                case "ros.resource.vpc.vip":
                case "ros.resource.vmware.vip":
                    rosPluginsOperationService.createVip(paramObj.toJSONString(), context, queue);
                    break;
                default:
                    return new GeneralResult<>(false, queue, "平台不支持该类资源");
            }
            return new GeneralResult<>(true, "success");
        } catch (Exception e) {
            log.error("编排资源申请异常：" + e);
            return new GeneralResult<>(false, queue, "编排资源申请异常：" + e);
        }
    }

    @Override
    public GeneralResult callbackProcess(String content) {
        try {
            List<JSONObject> resourceObjects = JSON.parseArray(content, JSONObject.class);
            GeneralResult generalResult = new GeneralResult<>(true, "编排资源删除成功");
            for (JSONObject jsonObject : resourceObjects) {
                String code = jsonObject.getString("code");
                Long vendorId = jsonObject.getLong("vendorId");
                Long id = jsonObject.getLong("id");
                if (null == vendorId || StringUtils.isEmpty(code)) {
                    continue;
                }
                CloudVendor cloudVendor = cloudVendorRepository.query(vendorId);
                RosPluginsOperationService rosPluginsOperationService = (RosPluginsOperationService)pluginUser
                    .getBean(cloudVendor.getType().toLowerCase(), "rosPluginsOperationService");
                switch (code) {
                    case "ros.resource.apsarastack.vpc":
                    case "ros.resource.easystack.vpc":
                    case "ros.resource.aliyun.vpc":
                    case "ros.resource.openstack.vpc":
                    case "ros.resource.huawei.vpc":
                    case "ros.resource.tencent.vpc":
                        generalResult = rosPluginsOperationService.removeVpc(id);
                        break;
                    case "ros.resource.apsarastack.ecs":
                    case "ros.resource.vmware.ecs":
                    case "ros.resource.aliyun.ecs":
                    case "ros.resource.openstack.ecs":
                    case "ros.resource.huawei.ecs":
                    case "ros.resource.tencent.ecs":
                        generalResult = rosPluginsOperationService.removeServer(id);
                        break;
                    case "ros.resource.apsarastack.volume":
                    case "ros.resource.easystack.volume":
                    case "ros.resource.aliyun.volume":
                    case "ros.resource.openstack.volume":
                    case "ros.resource.huawei.volume":
                    case "ros.resource.tencent.volume":
                        generalResult = rosPluginsOperationService.removeVolume(id);
                        break;
                    case "ros.resource.apsarastack.securitygroup":
                    case "ros.resource.easystack.securitygroup":
                    case "ros.resource.aliyun.securitygroup":
                    case "ros.resource.openstack.securitygroup":
                    case "ros.resource.huawei.securitygroup":
                    case "ros.resource.tencent.securitygroup":
                        generalResult = rosPluginsOperationService.removeSecGroup(id);
                        break;
                    case "ros.resource.apsarastack.subnet":
                    case "ros.resource.easystack.subnet":
                    case "ros.resource.aliyun.subnet":
                    case "ros.resource.openstack.subnet":
                    case "ros.resource.huawei.subnet":
                    case "ros.resource.tencent.subnet":
                        generalResult = rosPluginsOperationService.removeSubnet(id);
                        break;
                    default:
                        generalResult = new GeneralResult<>(false, "平台不支持该类资源");
                        break;
                }
                log.info("编排删除资源：{}, code:{}, 平台ID:{},删除结果：{}", generalResult.isSuccess(), code, vendorId,
                    generalResult.getMessage());
            }
            return new GeneralResult<>(true, "编排资源删除成功");
        } catch (Exception e) {
            log.error("编排资源删除异常：" + e);
            return new GeneralResult<>(false, "编排资源删除异常：" + e);
        }
    }

}
