package com.bocloud.cmp.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.entity.Dictionary;
import com.bocloud.cmp.entity.*;
import com.bocloud.cmp.enums.ResourceCategory;
import com.bocloud.cmp.repository.*;
import com.bocloud.cmp.service.ServerService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.*;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service("ServerService")
public class ServerServiceImpl implements ServerService {
    private static Logger logger = LoggerFactory.getLogger(ServerServiceImpl.class);

    @Autowired
    private ServerRepository serverRepository;
    @Autowired
    private ResourceRepository resourceRepository;
    @Autowired
    private CloudVendorRepository cloudVendorRepository;
    @Autowired
    private NetworkCardRepository networkCardRepository;
    @Autowired
    private StoreHostRepository storeHostRepository;
    @Autowired
    private ImageRepository imageRepository;
    @Autowired
    private ClusterRepository clusterRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ServerConfigRepository serverConfigRepository;
    @Autowired
    private ServerAssetsRepository serverAssetsRepository;
    @Autowired
    private CloudServerRepository cloudServerRepository;
    @Autowired
    private DictionaryRepository dictionaryRepository;
    @Autowired
    private PoolGroupRepository poolGroupRepository;

    @Override
    public GeneralResult<GridBean> list(Pager pager) {
        GridBean gridBean = null;
        try {

            int page = pager.getPage();
            int rows = pager.getRows();
            List<Param> params = pager.getParams();
            Map<String, String> sorter = pager.getSorter();
            boolean simple = pager.getSimple();

            if (null == params) {
                params = Lists.newArrayList();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            String vendorType = null;
            //集群、数据中心模糊查询
            String clusterName = "";
            String vdcName = "";
            //标签
            String labelContent = null;
            for (Param param : params) {
                Map<String, Object> paramMap = param.getParam();
                if (paramMap.containsKey("vendorType")) {
                    vendorType = paramMap.get("vendorType").toString();
                    paramMap.remove("vendorType");
                }
                if (paramMap.containsKey("clusterName")) {
                    clusterName = paramMap.get("clusterName").toString();
                    paramMap.remove("clusterName");
                }
                if (paramMap.containsKey("vdcName")) {
                    vdcName = paramMap.get("vdcName").toString();
                    paramMap.remove("vdcName");
                }
                if (paramMap.containsKey("labelContent")) {
                    labelContent = paramMap.get("labelContent").toString();
                    paramMap.remove("labelContent");
                }
            }
            sorter.put("gmtCreate", Common.ONE);
            if (simple) {
                int total = this.serverRepository.count(params, labelContent, vendorType);
                List<SimpleBean> beans = this.serverRepository.list(params, sorter, labelContent);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                int total = this.serverRepository.counts(params, labelContent, vendorType, clusterName, vdcName);
                List<Server> list = this.serverRepository.list(page, rows, params, sorter, labelContent, vendorType, clusterName, vdcName);
                for (Server server : list) {
                    // add server_config info for baremetal server
                    if ("BARE_METAL".equals(server.getAssetCategory())) {
                        server.setConfig(serverConfigRepository.getBySid(server.getId()));
                    }
                    List<NetworkCard> networkCards = networkCardRepository.listByHostIdAndSwitcherAvailable(server.getId(), true);
                    server.setNetcards(networkCards);
                    params.clear();
                    params.add(new Param(MapTools.simpleMap("hostId", server.getId()), Sign.EQ));
                    int vmNum = cloudServerRepository.count(params, 0L);
                    params.add(new Param(MapTools.simpleMap("isTemplate", 1), Sign.EQ));
                    int templateNum = cloudServerRepository.count(params, 0L);
                    server.setVmNum(vmNum - templateNum);
                    server.setTemplateNum(templateNum);
                    if (!StringUtils.isEmpty(server.getResourceGroup())) {
                        Dictionary dictionary = dictionaryRepository.query(server.getResourceGroup(), "RESOURCE_GROUP");
                        if (null != dictionary) {
                            server.setResourceGroupName(dictionary.getName());
                        }
                    }
                    // 设置这个物理机的资源池
                    if ("X86_SERVER".equals(server.getAssetCategory())) {
                        // 这是一个x86物理机
                        server.setGroupName(poolGroupRepository.getGroupNameByResourceTarget(server.getId(),
                                ResourceCategory.Computer.name(), "PHYSICAL"));
                    }
                    Optional.ofNullable(this.serverConfigRepository.getBySid(server.getId()))
                            .ifPresent(server::setConfig);
                    Optional.ofNullable(server.getRole()).ifPresent(role -> {
                        server.setRoles(new ArrayList<>(Arrays.asList(role.split(","))));
                        server.getRoles().remove("nfs_gateway");
                    });
                }
                gridBean = GridHelper.getBean(page, rows, total, list);
            }
            return new GeneralResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query server list fail:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<String> modify(String params, RequestContext context) {
        return new GeneralResult(false, ServerService.class.getSimpleName() + "的modify方法未实现");
    }

    @Override
    public GeneralResult modifyByUuid(String params, String uuid, RequestContext context) {
        try {
            if (StringUtils.isEmpty(uuid)) {
                return new GeneralResult(false, "修改物理主机失败，原因：uuid的值为空");
            }
            Long id = serverRepository.queryIdByUuid(uuid);
            if (id == null) {
                return new GeneralResult(true, "系统中不存在uuid为[" + uuid + "]的物理主机信息，无需修改");
            }

            // begin--追加server,server_config,server_assets的id属性
            JSONObject json = JSONTools.isJSONObj(params);
            if (null == json) {
                return new GeneralResult(false, "根据uuid修改物理主机时数据不能为空!");
            }

            String serverStr = json.getString("server");
            String serverConfigStr = json.getString("serverConfig");
            String serverAssetsStr = json.getString("serverAssets");

            Server server = JSONObject.parseObject(serverStr, Server.class);
            ServerConfig config = JSONObject.parseObject(serverConfigStr, ServerConfig.class);
            ServerAssets assets = JSONObject.parseObject(serverAssetsStr, ServerAssets.class);

            if (server != null) {
                server.setId(id);
            }
            if (config != null) {
                config.setServerId(id);
                ServerConfig configOfDb = serverConfigRepository.getBySid(id);
                if (configOfDb != null) {
                    config.setId(configOfDb.getId());
                }
            }
            if (assets != null) {
                assets.setServerId(id);
                ServerAssets assetsOfDb = serverAssetsRepository.getBySid(id);
                if (assetsOfDb != null) {
                    assets.setId(assetsOfDb.getId());
                }
            }

            json.put("server", server);
            json.put("serverConfig", config);
            json.put("serverAssets", assets);
            params = json.toJSONString();
            // end----追加server,server_config,server_assets的id属性
            return modify(params, context);
        } catch (Exception e) {
            logger.error("修改物理主机失败," + e.getMessage(), e);
            return new GeneralResult(false, "修改物理主机失败");
        }
    }

    @Override
    public GeneralResult remove(Long id, Boolean force, RequestContext context) {
        try {
            Server node = this.serverRepository.query(id);
            if (null == node) {
                logger.warn("Server data does not exist!");
                return new GeneralResult(false, "主机不存在");
            }

            // 如果rackhdNodeId不为空，则需要调用BMS接口来执行删除
            if (StringUtils.isNotEmpty(node.getRackhdNodeId())) {
//                bmsInternalService.serverRemove(id);
            }

            serverRepository.remove(id, context.getTarget());
            serverConfigRepository.removeBySid(id);
            serverAssetsRepository.removeBySid(id);
            networkCardRepository.removeByHost(id, "PM", context.getTarget());
            resourceRepository.remove(id, ResourceCategory.Computer.name(), "PHYSICAL", context.getTarget());
            return new GeneralResult(true, id, "删除成功");
        } catch (Exception e) {
            logger.error("remove failed:", e);
            return new GeneralResult(false, "删除失败");
        }

    }

    @Override
    public GeneralResult<Server> basic(Long id) {
        try {
            Server server = this.serverRepository.getServerBean(id);
            if (null == server) {
                logger.warn("Server does not exist!");
                return new GeneralResult(false, server, "该服务器不存在");
            }
            return new GeneralResult(true, server, "查询成功");
        } catch (Exception e) {
            logger.error("Query server fail:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Server> detail(Long id) {
        try {
            Server server = serverRepository.query(id);
            if (null == server) {
                logger.warn("server does not exist!");
                return new GeneralResult(false, "数据不存在");
            }
            server.setNetcards(networkCardRepository.listByHost(id, "PM"));
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("hostId", id), Sign.EQ));
            int vmNum = cloudServerRepository.count(params, 0L);
            params.add(new Param(MapTools.simpleMap("isTemplate", 1), Sign.EQ));
            int templateNum = cloudServerRepository.count(params, 0L);
            server.setVmNum(vmNum - templateNum);
            server.setTemplateNum(templateNum);
            int storeNum = storeHostRepository.list(server.getId(), "PM").size();
            server.setDatastoreNum(storeNum);
            ServerConfig config = serverConfigRepository.getBySid(id);
            ServerAssets assets = serverAssetsRepository.getBySid(id);
            server.setConfig(null == config ? new ServerConfig() : config);
            server.setAssets(null == assets ? new ServerAssets() : assets);
            if (null != server.getImageId()) {
                Image image = imageRepository.queryPhysical(server.getImageId());
                if (image != null) {
                    server.setImageName(image.getName());
                    server.setImageIsoName(image.getIsoName());
                }
            }
//            if (null != server.getVendorId()) {
//                CloudVendor cloudVendor = cloudVendorRepository.query(server.getVendorId());
//                if (cloudVendor != null && CloudProvider.POWERVC.name().equals(cloudVendor.getType())) {
//                    server.setPowervcHmcs(powervcHmcServerRepository.listHmcByServer(server.getId()));
//                }
//            }
            Optional.ofNullable(server.getRole()).ifPresent(role -> {
                server.setRoles(Arrays.stream(role.split(",")).collect(Collectors.toList()));
                server.getRoles().remove("nfs_gateway");
            });
            return new GeneralResult(true, server, "查询成功");
        } catch (Exception e) {
            logger.error("Query server detail fail:", e);
            return new GeneralResult(false, "查询失败");
        }
    }


    @Override
    public GeneralResult start(Long id, RequestContext context) {
        return null;
    }

    @Override
    public GeneralResult stop(Long id, RequestContext context) {
        return null;
    }

    @Override
    public GeneralResult restart(Long id, RequestContext context) {
        return null;
    }

    @Override
    public GeneralResult<String> isolate(Long id, RequestContext context) {
        String path = Server.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            this.serverRepository.isolate(id, context.getTarget());
            return new GeneralResult(true, "隔离成功");
        } catch (Exception e) {
            logger.error("Isolate server fail:", e);
            return new GeneralResult(false, "隔离失败");
        }
    }

    @Override
    public GeneralResult<String> recover(Long id, RequestContext context) {
        String path = Server.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            this.serverRepository.recover(id, context.getTarget());
            return new GeneralResult(true, "恢复成功");
        } catch (Exception e) {
            logger.error("Recover server fail:", e);
            return new GeneralResult(false, "恢复失败");
        }
    }

    @Override
    public GeneralResult exists(String ip) {
        try {
            boolean result = this.serverRepository.exists(ip);
            if (result) {
                return new GeneralResult(true, "IP存在");
            }
            return new GeneralResult(false, "IP不存在");
        } catch (Exception e) {
            logger.error("Check vm ip fail:", e);
            return new GeneralResult(false, "确定vm ip是否存在失败");
        }
    }

    /**
     * 通过IP查询当前主机信息
     *
     * @author sxp
     * @since V1.0 Nov 4, 2016
     */
    @Override
    public Server detail(String ip) {
        Server server = null;

        try {
            server = this.serverRepository.get(ip);
        } catch (Exception e) {
            logger.error("Query server information by IP fail.", e);
        }

        return server;
    }

    @Override
    public GeneralResult<List<CloudServer>> listvmByvhost(long venderId, String hostName) {
        try {
            List<CloudServer> servers = this.cloudServerRepository.listvmByvhost(venderId, hostName);
            for (CloudServer server : servers) {
                server.setPassword(null);
            }
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("exception info", e);
            return new GeneralResult(false, "查询云主机失败");
        }
    }

    @Override
    public GeneralResult<String> updateResourceGroup(String[] ids, String resourceGroup, Long id) {
        List<Resource> resources = new ArrayList<>();
        List<Server> servers = new ArrayList<>();
        try {
            for (String serverid : ids) {
                Server server = serverRepository.query(Long.valueOf(serverid));
                if (null != server) {
                    server.setResourceGroup(resourceGroup);
                    servers.add(server);
                }
                Resource resource = resourceRepository.query(ResourceCategory.Computer.name(),
                        "PHYSICAL", Long.valueOf(serverid));
                if (null != resource) {
                    resource.setResourceGroup(resourceGroup);
                    resources.add(resource);
                }
            }
            for (Server server : servers) {
                try {
                    serverRepository.update(server);
                } catch (Exception e) {
                    logger.error("绑定资源异常", e);
                    return new GeneralResult(false, "更新配置分组失败");
                }
            }
            if (resources != null && resources.size() > 0) {
                for (Resource resource : resources) {
                    resourceRepository.update(resource);
                }
            } else {
                return new GeneralResult(false, "更新配置分组失败");
            }
        } catch (Exception e) {
            logger.error("查询资源异常", e);
            return new GeneralResult(false, "更新配置分组失败");
        }
        return new GeneralResult(true, "更新配置分组成功");
    }

    @Override
    public GeneralResult<List<Server>> listByCluster(Long clusterId) {
        try {
            List<Server> servers = serverRepository.listByCluster(clusterId);
            for (Server server : servers) {
                server.setConfig(serverConfigRepository.getBySid(server.getId()));
            }
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<Server>> listByClusters(List<Long> clusterIds) {
        try {
            List<Server> servers = serverRepository.listByClusters(clusterIds);
            for (Server server : servers) {
                server.setConfig(serverConfigRepository.getBySid(server.getId()));
            }
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<Server>> listByDefaultCluster(long vendorId) {
        try {
            Cluster cluster = clusterRepository.queryByUuid("1", vendorId);
            if (cluster == null) {
                return new GeneralResult(false, "未找到默认主机组");
            }
            List<Server> servers = serverRepository.listByCluster(cluster.getId());
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<Server>> listByVendor(Long vendorId) {
        try {
            List<Server> servers = serverRepository.listByVendor(vendorId);
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    /**
     * license校验
     *
     * @param vendorId
     * @param pm
     * @param vm
     * @return
     */
    @Override
    public GeneralResult licenseCheck(Long vendorId, Integer pm, Integer vm) {
//        try {
//            Integer pmCount = serverRepository.licenseNodeCount();
//            pmCount += pm;
//            int nodeCount = pmCount;
//            if (nodeCount > licenseSerivce.license().getNode()) {
//                return new GeneralResult(false, "当前系统节点数（宿主机和云主机）已超过license限制，创建云主机失败!", "500", "");
//            }
//            return new GeneralResult(true, "license校验通过");
//        } catch (Exception e) {
//            logger.error("Get error message: ", e);
//            return new GeneralResult(false, "license校验失败");
//        }
    	return ResultTools.noImplementedResult();
    }

    @Override
    public GeneralResult<Server> queryByIp(String ip) {
        try {
            Server server = serverRepository.queryByIp(ip);
            return new GeneralResult(true, server, "查询成功");
        } catch (Exception e) {
            logger.error("Query server by ip is error", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<String> batchModifyLabel(String params, RequestContext context) {
        JSONObject object = JSONObject.parseObject(params);
        try {
            Assert.notNull(object, "参数格式错误");
            List<Long> hostIds = JSONArray.parseArray(object.getString("hostIds"), Long.class);
            if (ListTool.isEmpty(hostIds)) {
                return new GeneralResult(false, "请选择宿主机");
            }
            for (Long hostId : hostIds) {
                Server server = serverRepository.query(hostId);
                if (null != server) {
                    server.setResourceLabel(object.getString("resourceLabel"));
                    server.setGmtModify(new Date());
                    server.setMenderId(context.getTarget());
                    serverRepository.update(server);
                }
            }
            return new GeneralResult(true, "标签配置成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new GeneralResult(false, "标签配置失败");
        }
    }

    @Override
    public GeneralResult listServerOutAccessPath(int page, int rows, long id, Map<String, Object> params,
                                                 Map<String, String> sorter) {
        try {
            List<Server> servers = serverRepository.listServerOutAccessPath(page, rows, id, params, sorter);
            int total = serverRepository.countListServerOutAccessPath(id, params);
            GridBean gridBean = new GridBean(page, rows, total, servers);
            return new GeneralResult(true, gridBean, "查询详情成功");
        } catch (Exception e) {
            logger.error("query server detail fail:", e);
            return new GeneralResult(false, "查询详情失败", "", "");
        }
    }

    @Override
    public GeneralResult listServerInAccessPath(int page, int rows, long id, Map<String, Object> params,
                                                Map<String, String> sorter) {
        try {
            List<Server> servers = serverRepository.listServerInAccessPath(page, rows, id, params, sorter);
            int total = serverRepository.countListServerInAccessPath(id, params);
            GridBean gridBean = new GridBean(page, rows, total, servers);
            return new GeneralResult(true, gridBean, "查询详情成功");
        } catch (Exception e) {
            logger.error("query server detail fail:", e);
            return new GeneralResult(false, "查询详情失败", "", "");
        }
    }

    @Override
    public GeneralResult<List<Server>> list(String params) {
        try {
            JSONObject conditionObj = JSONTools.isJSONObj(params);
            // 平台ID
            List<Long> vendorIds = new ArrayList<Long>();
            Long vendorId = conditionObj.getLong("vendorId");
            if (null != vendorId) {
                vendorIds.add(vendorId);
            }

            Long vdcId = conditionObj.getLong("vdcId");
            Long clusterId = conditionObj.getLong("clusterId");
            String serverName = conditionObj.getString("name");
            String vendorType = conditionObj.getString("vendorType");
            List<CloudVendor> type = new ArrayList<>();
            if (!StringUtils.isEmpty(vendorType)) {
                type = cloudVendorRepository.listByType(vendorType);
            }
            if (!ListTool.isEmpty(type)) {
                vendorIds.addAll(type.stream().map(CloudVendor::getId).collect(Collectors.toList()));
            }
            List<Server> servers = serverRepository.listByCondition(vdcId, clusterId, serverName, vendorIds);
            for (Server server : servers) {
                server.setConfig(serverConfigRepository.getBySid(server.getId()));
            }
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<List<ServerConfig>> listByServerConfigs(JSONArray serverIds) {
        try {
            List<ServerConfig> serverConfigs = serverConfigRepository.listByHostIds(serverIds.toJavaList(Long.class));
            return new GeneralResult(true, serverConfigs, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult listByVendorType(String conditionStr) {
        try {
            if (StringUtils.isEmpty(conditionStr)) {
                return new GeneralResult<>(false, "参数格式错误");
            }
            JSONObject parseObject = JSON.parseObject(conditionStr, JSONObject.class);
            String vendorType = parseObject.getString("vendorType");

            List<CloudVendor> cloudVendors = cloudVendorRepository.listByType(vendorType);
            if (ListTool.isEmpty(cloudVendors)) {
                return new GeneralResult<>(true, new ArrayList<>(), "查询成功");
            }
            List<Long> vendorIds = cloudVendors.stream().map(CloudVendor::getId).collect(Collectors.toList());
            List<Server> servers = serverRepository.listByVendorIds(vendorIds, new JSONObject());
            return new GeneralResult(true, servers, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    public GeneralResult performance(String params, Long id, RequestContext context) {
//        JSONObject object = JSONTools.isJSONObj(params);
//        try {
//            CloudVendor vendor = this.cloudVendorRepository.query(object.getLong(Common.VENDORID));
//            if (null == vendor) {
//                logger.warn("Cloud vendor does not exist!");
//                return new GeneralResult(false, "云平台不存在");
//            }
//            List<String> types = Lists.newArrayList();
//            types.add("CPU");
//            types.add("DISK");
//            types.add("POWER");
//            types.add("MEM");
//            types.add("NET");
//            types.add("SYSTEM");
//            return new VmwareMonitorUtils(monitorDriver).getPerformance(object, vendor.getUuid(), types, "HostSystem");
//        } catch (Exception e) {
//            logger.error("查询数据失败", e);
//            return new GeneralResult(false, "查询失败");
//        }
    	return ResultTools.noImplementedResult();

    }

}