package com.bocloud.cmp.service.impl;

import com.bocloud.cmp.entity.CloudVendor;
import com.bocloud.cmp.repository.CloudVendorRepository;
import com.bocloud.cmp.service.SyncService;
import com.gitee.starblues.integration.user.PluginUser;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

@Slf4j
@Service("defaultSyncService")
public class SyncServiceImpl implements SyncService {

	@Autowired
	private CloudVendorRepository cloudVendorRepository;
	@Autowired
	private PluginUser pluginUser;

	@Override
	public GeneralResult sync(Long id, RequestContext context, boolean isAuto) {
		try {
			CloudVendor vendor = cloudVendorRepository.query(id);
			Assert.notNull(vendor, "平台数据不存在！");
			String vendorType = vendor.getType().toLowerCase();
			Object bean = pluginUser.getBean(vendorType, "syncService");
			if (null == bean) {
				return new GeneralResult(false, "当前平台插件不存在");
			}
			log.info("----------1-----------");
			log.info(bean.getClass().getName());
			log.info(bean.getClass().getClassLoader().toString());
			log.info("----------2-----------");
			for (Class<?> intf : bean.getClass().getInterfaces()) {
				log.info(intf.getName());
				log.info(intf.getClassLoader().toString());
			}
			log.info("----------3-----------");
			log.info(SyncService.class.getName());
			log.info(SyncService.class.getClassLoader().toString());
			log.info("----------4-----------");
			SyncService syncService = (SyncService) pluginUser.getBean(vendorType, "syncService");
			if (null == syncService) {
				return new GeneralResult(false, "当前平台插件不存在");
			}
			return syncService.sync(vendor.getId(), context, isAuto);
		} catch (Exception e) {
			log.error("sync cloudvendor exception ", e);
			return new GeneralResult<>(false, "云平台数据同步失败: " + e.getMessage());
		}
	}
}
