package com.bocloud.cmp.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.bocloud.cmp.entity.Project;
import com.bocloud.cmp.entity.Tenant;
import com.bocloud.cmp.entity.Vpc;
import com.bocloud.cmp.internal.SmsProjectService;
import com.bocloud.cmp.internal.SmsTenantService;
import com.bocloud.cmp.repository.VpcRepository;
import com.bocloud.cmp.service.VpcService;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.MapTools;
import com.megatron.common.utils.ResultTools;

import lombok.extern.slf4j.Slf4j;

/**
 * vpc service实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年2月2日
 */
@Service("vpcService")
@Slf4j
public class VpcServiceImpl implements VpcService {

	@Autowired
	private VpcRepository vpcRepository;
	@Autowired
	private SmsProjectService smsProjectService;
	@Autowired
	private SmsTenantService smsTenantService;

	@Override
	public GeneralResult<GridBean<Vpc>> list(Pager pager, RequestContext context) {
		List<Vpc> list = Lists.newArrayList();
		int total = 0;
		GridBean gridBean = null;
		try {
			List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
			Map<String, String> sorter = Optional.ofNullable(pager.getSorter())
					.orElse(com.google.common.collect.Maps.newHashMap());
			Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
			sorter.put("gmtCreate", Common.ONE);
			Integer page = Optional.ofNullable(pager.getPage()).orElse(1);
			Integer rows = Optional.ofNullable(pager.getRows()).orElse(10);

			Boolean hasEip = false;
			Boolean hasIpv6 = false;
			Boolean notGateWay = false;
			String vendorType = null;
			String labelContent = null;
			for (Param param : params) {
				Map<String, Object> paramMap = param.getParam();
				if (paramMap.containsKey("vendorType")) {
					vendorType = String.valueOf(paramMap.get("vendorType"));
					paramMap.remove("vendorType");
				}
				if (paramMap.containsKey("hasEip")) {
					hasEip = Boolean.valueOf(paramMap.get("hasEip").toString());
					simple = false;
					paramMap.remove("hasEip");
				}
				if (paramMap.containsKey("hasIpv6")) {
					hasIpv6 = Boolean.valueOf(paramMap.get("hasIpv6").toString());
					simple = false;
					paramMap.remove("hasIpv6");
				}
				if (paramMap.containsKey("notGateWay")) {
					notGateWay = Boolean.valueOf(paramMap.get("notGateWay").toString());
					simple = false;
					paramMap.remove("notGateWay");
				}
				if (paramMap.containsKey("labelContent")) {
					labelContent = paramMap.get("labelContent").toString();
					paramMap.remove("labelContent");
				}
			}
			if (context.getCatalog().equals(RequestContext.Catalog.Tenant)
					|| context.getCatalog().equals(RequestContext.Catalog.User)) {
				params.add(new Param(MapTools.simpleMap("tenant_id", context.getTenant()), Sign.EQ));
				total = this.vpcRepository.count(params, vendorType, labelContent);
				list = this.vpcRepository.list(page, rows, params, sorter, vendorType, labelContent);
				gridBean = new GridBean<>(page, rows, total, list);
			} else {
				total = vpcRepository.countByLabel(params, labelContent);
				if (simple) {
					List<SimpleBean> beans = vpcRepository.list(params, sorter, labelContent);
					gridBean = new GridBean(1, 1, total, beans);
				} else {
					list = vpcRepository.list(page, rows, params, sorter, hasEip, hasIpv6, notGateWay, labelContent);
					gridBean = GridHelper.getBean(page, rows, total, list);
				}
			}

			List<Long> tenantIds = new ArrayList<>();
			List<Long> projectIds = new ArrayList<>();
			for (Vpc vpc : list) {
				if (vpc.getTenantId() != null) {
					tenantIds.add(vpc.getTenantId());
				}
				if (vpc.getProjectId() != null) {
					projectIds.add(vpc.getProjectId());
				}
			}
			Map<Long, String> projectMap = smsProjectService.listProjectsByIds(projectIds).stream()
					.collect(Collectors.toMap(Project::getId, Project::getName));
			Map<Long, String> tenantMap = smsTenantService.listTenantsByIds(tenantIds).stream()
					.collect(Collectors.toMap(Tenant::getId, Tenant::getName));
			for (Vpc vpc : list) {
				if (vpc.getTenantId() != null) {
					vpc.setTenantName(tenantMap.get(vpc.getTenantId()));
				}
				if (vpc.getProjectId() != null) {
					vpc.setProjectName(projectMap.get(vpc.getProjectId()));
				}
			}
			return new GeneralResult<>(true, gridBean, "查询成功");
		} catch (Exception e) {
			log.error("list vpc failure:", e);
			return new GeneralResult<>(false, "查询失败", null, null);
		}
	}



	
	

	@Override
	public GeneralResult<List<Vpc>> condition(String condition, RequestContext context) {
		return ResultTools.formatErrResult();
	}

}